/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.commons;

/**
 * @author Oliver Plohmann
 *
 */
public class Result
{
	protected boolean success = true;
	protected Object value = null;
	protected int reasonCode = -1;
	protected String reason = null;
	protected Throwable exception = null;
	
	/**
	 * Creates a new <code>Result</code> object.
	 */
	public Result()
	{
		super();
	}
	
	/**
	 * Creates a new <code>Result</code> object.
	 */
	public Result(Object value)
	{
		super();
		this.value = value;
	}
	
	/**
	 * Creates a new <code>Result</code> object.
	 */
	public Result(String reason)
	{
		super();
		success = false;
		this.reason = reason;
	}
	
	/**
	 * Creates a new <code>Result</code> object.
	 */
	public Result(boolean success)
	{
		super();
		this.success = success;
	}
	
	/**
	 * Creates a new <code>Result</code> object.
	 */
	public Result(boolean success, int reason)
	{
		super();
		this.success = success;
		this.reasonCode = reason;
	}
	
	/**
	 * Creates a new <code>Result</code> object.
	 */
	public Result(Throwable exception)
	{
		super();
		this.success = false;
		this.exception = exception;
	}
	
	/**
	 * @return Returns the value.
	 */
	public Object getValue()
	{
		return value;
	}
	
	/**
	 * @param value The value to set.
	 */
	public void setValue(Object value)
	{
		this.value = value;
	}
	
	/**
	 * @return Returns the success.
	 */
	public boolean isSuccess()
	{
		return success;
	}
	
	/**
	 * @param success The success to set.
	 */
	public void setSuccess(boolean success)
	{
		this.success = success;
	}
	
	/**
	 * @return Returns the reasonCode.
	 */
	public String getReason()
	{
		return reason;
	}
	
	/**
	 * @param reasonCode The reasonCode to set.
	 */
	public void setReasonCode(int reasonCode)
	{
		this.reasonCode = reasonCode;
	}
	
	/**
	 * @return Returns the exception.
	 */
	public Throwable getException()
	{
		return exception;
	}
	
	/**
	 * @param exception The exception to set.
	 */
	public void setException(Throwable exception)
	{
		this.exception = exception;
	}
	/**
	 * @param reason The reason to set.
	 */
	public void setReason(String reason)
	{
		this.reason = reason;
	}
	/**
	 * @return Returns the reasonCode.
	 */
	public int getReasonCode()
	{
		return reasonCode;
	}
}
