/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.commons;

import java.util.Hashtable;
import java.util.Map;

/**
 * Trait that implements a class to which user-defined attributes can be added at runtime. 
 * For a further introduction into traits see http://www.iam.unibe.ch/~scg/Research/Traits/
 * 
 * @author Oliver Plohmann
 */

public class TAttribute
{

	/**
	 * Map that holds values of attributes for a given key.
	 */
	protected Map attributes = new Hashtable();
	
	/**
	 * Creates a new <code>TAttribute</code> object.
	 */
	public TAttribute()
	{
		super();
	}

	/**
	 * Set the value of an attribute.
	 * @param attributeName the name of the attribute
	 * @param value the value to be set
	 */
	public void setAttributeValue(String attributeName, Object value)
	{
		if(value != null)
			attributes.put(attributeName, value);
	}
	
	/**
	 * Return the value of the attribute.
	 * @param attributeName
	 * @return the attribute value
	 */
	public Object getAttributeValue(String attributeName)
	{
		return attributes.get(attributeName);
	}
	
	/**
	 * Answer whether an attribute with a given name is defined.
	 * @param attributeName
	 * @return
	 */
	public boolean hasAttribute(String attributeName)
	{
		return attributes.containsKey(attributeName);
	}

}
