/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.commons.events;

import java.io.Serializable;

import org.almendra.commons.INamedObject;


/**
 * Defines an event by its name and contains the event parameters in <code>params</code>.
 * 
 * @author Oliver Plohmann
 */
public class Event implements INamedObject, Serializable
{
	/**
	 * Event name
	 */
	protected String name = null;
	
	/**
	 * Attribute to hold event parameters
	 */
	protected IEventParams params = null;
	
	/**
     * Creates a new <code>Event</code> object. Declared private to enforce the use
     * of the public constructors to make sure the object is initialized propperly.
	 */
	private Event()
	{
		super();
	}
	
	/**
	 * Creates a new <code>Event</code> object. 
	 */
	public Event(String name)
	{
		super();
		this.name = name;
	}
	
	/**
	 * Creates a new <code>Event</code> object. 
	 */
	public Event(String name, IEventParams params)
	{
		super();
		this.name = name;
		this.params = params;
	}

	/**
	 * Return the event name.
	 * @see org.almendra.commons.INamedObject#getName()
	 */
	public String getName()
	{
		return name;
	}

	/**
	 * @return Returns the params.
	 */
	public IEventParams getParams()
	{
		return params;
	}

	/**
	 * @param params The params to set.
	 */
	public void setParams(IEventParams params)
	{
		this.params = params;
	}

	/**
	 * @return
	 */
	public String toDisplayString()
	{
		StringBuffer buffer = new StringBuffer();
		buffer.append(super.toString());
		buffer.append("{"); //$NON-NLS-1$
		buffer.append("name=" + name); //$NON-NLS-1$
		buffer.append("}"); //$NON-NLS-1$
		return buffer.toString();
	}

}
