/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.commons.events;

import java.awt.Component;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;

import javax.swing.JOptionPane;

import org.almendra.commons.Result;


/**
 * @author Oliver Plohmann
 *
 */
public class SimpleVetoableEventRegistry extends SimpleEventRegistry implements IVetoableEventRegistry
{
	/**
	 * Creates a new <code>SimpleVetoableEventRegistry</code> object.
	 */
	public SimpleVetoableEventRegistry()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.events.IVetoableEventRegistry#register(java.lang.String, org.almendra.commons.commons.events.IVetoableCallbackHandler)
	 */
	public void register(String eventName, IVetoableCallbackHandler handler) 
	throws IllegalArgumentException
	{
		registerPrim(eventName, handler);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.events.IVetoableEventRegistry#deregister(java.lang.String, org.almendra.commons.commons.events.IVetoableCallbackHandler)
	 */
	public void deregister(String eventName, IVetoableCallbackHandler handler) 
	throws NoSuchElementException, IllegalArgumentException
	{
		deregisterPrim(eventName, handler);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.events.IVetoableEventRegistry#askAnyMustVeto(org.almendra.commons.commons.events.Event)
	 */
	public List askAnyMustVeto(Event event)
	{
		try
		{
			lock.acquire();
			List results = new ArrayList();
			List handlers = (List) registry.get(event.getName());
			if(handlers == null)
				return results;
			Iterator it = handlers.iterator();
			while (it.hasNext())
			{
				IVetoableCallbackHandler handler = (IVetoableCallbackHandler) it.next();
				Result result = handler.askMustVeto(event);
				if(!result.isSuccess())
					results.add(result);
			}
			return results;
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			// TODO - write ex to logger
		}
		finally
		{
			lock.release();
		}
		return new ArrayList();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.events.IVetoableEventRegistry#displayVetos(java.util.List)
	 */
	public boolean askUserIgnoreVetos(List listOfResults, Component componentOrNull, String title, String requestActionMessage)
	{
		if(listOfResults == null || listOfResults.isEmpty())
			return false;
		StringBuffer message = new StringBuffer();	
		Iterator it = listOfResults.iterator();
		while (it.hasNext())
		{
			Result result = (Result) it.next();
			String reason = result.getReason();
			message.append(reason);
			if(it.hasNext())
				message.append("\n\n"); //$NON-NLS-1$
		}
		message.append("\n\n" + requestActionMessage); //$NON-NLS-1$
		int result = JOptionPane.showConfirmDialog(null, message.toString(), title, javax.swing.JOptionPane.YES_NO_OPTION);
		return result == JOptionPane.YES_OPTION;
	}

}
