/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.commons.threads;


/**
 * @author Oliver
 *
 */
public class Acknowledge extends Future
{

	ISignalable callbackHandler = null; 
	
	/**
	 * Creates a new <code>Acknowledge</code> object.
	 */
	public Acknowledge()
	{
		super();
	}

	/**
	 * Creates a new <code>Acknowledge</code> object.
	 */
	public Acknowledge(String name)
	{
		super(name);
	}
	
	/**
	 * Creates a new <code>Acknowledge</code> object.
	 */
	public Acknowledge(ISignalable callbackHandler)
	{
		super();
		this.callbackHandler = callbackHandler;
	}
	
	/**
	 * Creates a new <code>Acknowledge</code> object.
	 */
	public Acknowledge(ISignalable callbackHandler, String name)
	{
		super();
		this.callbackHandler = callbackHandler;
		this.name = name;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.threads.Future#signal()
	 */
	public void signal()
	{
		synchronized(lock)
		{
			if(callbackHandler != null)
			{
				callbackHandler.signal();
				if(callbackHandler.holds())
					return;
			}
			super.signal();					
		}
	}

	/**
	 * @param callbackHandler The callbackHandler to set.
	 */
	public void setCallbackHandler(ISignalable callbackHandler)
	{
		this.callbackHandler = callbackHandler;
	}

}
