/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
 
package org.almendra.commons.threads;

/**
 * Contains several helper methods to deal with threads. The idea is to define a single place
 * to deal with <code>InterruptedException</code> uniformingly avoiding code repretition.
 * 
 * @author Oliver Plohmann
 */

public class ThreadUtility
{

	/**
	 * Creates a new <code>ThreadUtility</code> object. Not used since all methods are static.
	 */
	private ThreadUtility()
	{
		super();
	}

	/**
	 * Make current thread sleep for <code>millis</code> milliseconds. 
	 * @param millis the number of milliseconds to make the current thread sleep
	 */
	public static void sleep(long millis)
	{
		try
		{
			Thread.sleep(millis);
		}
		catch (InterruptedException e)
		{
			System.err.println(e.getMessage());
			e.printStackTrace();
			// TODO - has to be made more robust
			// see http://gee.cs.oswego.edu/dl/cpj/cancel.html
		}
	}
	
	/**
	 * CollectionBlock the thread that owns <code>object</code>.
	 * @param object
	 */
	public static void wait(Object object)
	{
		try
		{
			object.wait();
		}
		catch (InterruptedException e)
		{
			System.err.println(e.getMessage());
			e.printStackTrace();
			// TODO - has to be made more robust
			// see http://gee.cs.oswego.edu/dl/cpj/cancel.html
		}
	}
}
