/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.commons.util;

import java.util.GregorianCalendar;

/**
 * Contains several helper methods to deal with time stamps.
 * 
 * @author Oliver Plohmann
 */
public class TimeUtility
{

	/**
	 * Creates a new <code>TimeUtility</code> object. Not used since all methods are static.
	 */
	private TimeUtility()
	{
		super();
	}

	/**
	 * Prepend a 0 if parameter num has only 1 digit.
	 * @param num
	 * @return
	 */
	protected static String zeroPadded(int num)
	{
		String value = String.valueOf(num);
		if (value.length() == 1)
			value = "0" + value; //$NON-NLS-1$
		return value;
	}

	/**
	 * Answer current time as a string in the format dd.mm.yy-hh:mm:ss-ms.
	 * @return current time as timestamp
	 */
	public static String toPreattyString(GregorianCalendar cal)
	{
		StringBuffer now = new StringBuffer();
		now.append(zeroPadded(cal.get(GregorianCalendar.DAY_OF_MONTH)));
		now.append("."); //$NON-NLS-1$
		now.append(zeroPadded(cal.get(GregorianCalendar.MONTH) + 1));
		now.append("."); //$NON-NLS-1$
		now.append(cal.get(GregorianCalendar.YEAR));
		now.append("-"); //$NON-NLS-1$
		now.append(zeroPadded(cal.get(GregorianCalendar.HOUR_OF_DAY)));
		now.append(":"); //$NON-NLS-1$
		now.append(zeroPadded(cal.get(GregorianCalendar.MINUTE)));
		now.append(":"); //$NON-NLS-1$
		now.append(zeroPadded(cal.get(GregorianCalendar.SECOND)));
		now.append("-"); //$NON-NLS-1$
		now.append(cal.get(GregorianCalendar.MILLISECOND));
		return now.toString();
	}
	
	public static String toShortPreattyString(GregorianCalendar cal)
	{
		StringBuffer now = new StringBuffer();
		now.append(zeroPadded(cal.get(GregorianCalendar.DAY_OF_MONTH)));
		now.append("."); //$NON-NLS-1$
		now.append(zeroPadded(cal.get(GregorianCalendar.MONTH) + 1));
		now.append("."); //$NON-NLS-1$
		now.append(cal.get(GregorianCalendar.YEAR));
		now.append("-"); //$NON-NLS-1$
		now.append(zeroPadded(cal.get(GregorianCalendar.HOUR_OF_DAY)));
		now.append(":"); //$NON-NLS-1$
		now.append(zeroPadded(cal.get(GregorianCalendar.MINUTE)));
		return now.toString();
	}
	
	public static String now()
	{
		return toPreattyString((GregorianCalendar) GregorianCalendar.getInstance());
	}
	
	public static String nowShort()
	{
		return toShortPreattyString((GregorianCalendar) GregorianCalendar.getInstance());
	}
	
	public static GregorianCalendar fromShort(String timestamp)
	{
		if(timestamp == null)
			return null;
		int offset1 = timestamp.indexOf("."); //$NON-NLS-1$
		int day = StringUtility.toInteger(timestamp.substring(0, offset1)).intValue();
		int offset2 = timestamp.lastIndexOf("."); //$NON-NLS-1$
		int month = StringUtility.toInteger(timestamp.substring(offset1 + 1, offset2)).intValue() - 1;
		offset1 = offset2;
		offset2 = timestamp.indexOf("-"); //$NON-NLS-1$
		int year = StringUtility.toInteger(timestamp.substring(offset1 + 1, offset2)).intValue();
		offset1 = timestamp.indexOf("-"); //$NON-NLS-1$
		offset2 = timestamp.indexOf(":"); //$NON-NLS-1$
		int hour = StringUtility.toInteger(timestamp.substring(offset1 + 1, offset2)).intValue();
		offset1 = offset2;
		int minute = StringUtility.toInteger(timestamp.substring(offset1 + 1, timestamp.length())).intValue();
		return new GregorianCalendar(year, month, day, hour, minute);
	}

}
