/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
 
package org.almendra.commons.util.collection;

import java.util.Iterator;

/**
 * Defines a minimal interface for any kind of collection that implements sequential queue accees. 
 * 
 * @author Oliver Plohmann
 */
public interface IQueue
{
	/**
	 * Answer whether ther is at least another object in the queue.
	 * @return
	 */
	public boolean hasNext();
	
	/**
	 * Remove the first object from the queue and return it.
	 * @return
	 */
	public Object getNext();
	
	/**
	 * Add an object to the queue.
	 * @param object the object to be added
	 * @return true if adding the object to the queue was successful otherwise false
	 */
	public boolean add(Object object);
	
	/**
	 * Remove an object from the queue.
	 * @param object
	 * @return the removed object or null if not found.
	 */
	public Object remove(Object object);
	
	/**
	 * Answer the number of objects in the queue.
	 * @return
	 */
	public int size();
	
	/**
	 * Answer true in case the queue is empty otherwise false.
	 * @return
	 */
	public boolean isEmpty();
	
	/**
	 * Return an iterator with which to iterate over the queue objects.
	 * @return
	 */
	public Iterator iterator();
	
}
