/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.commons.util.collection;

import java.util.ArrayList;
import java.util.List;

/**
 * A simple implementation of the <code>IListEvaluator</code> interface that simply returns a list.
 * Needed in case an <code>IListEvaluator</code> is declared for sake of generality but deferred 
 * creation of the list is actually not required. Declaring a <code>IListEvaluator</code> allows the 
 * user the create the required list when asked for and not before.
 * 
 * @author Oliver Plohmann
 *
 */

public class ListHolder implements IListEvaluator
{

	/**
	 * The list to be returned.
	 */
	protected List list = new ArrayList();
	
	/**
	 * Creates a new <code>ListHolder</code> object.
	 */
	public ListHolder()
	{
		super();
	}
	
	/**
	 * Creates a new <code>ListHolder</code>.
	 * @param object an object added to the <code>ListHolder</code> list.
	 */
	public ListHolder(Object object)
	{
		super();
		list.add(object);
	}
	
	/**
	 * Creates a new <code>ListHolder</code> object.
	 * @param list the list to be returned
	 */
	public ListHolder(List list)
	{
		super();
		this.list.addAll(list);
	}

	/**
	 * Return the list contained by the <code>ListHolder</code>.
	 * @see org.almendra.commons.IListEvaluator#get()
	 */
	public List get()
	{
		return list;
	}

}
