/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project (http://www.oliverplohmann.de/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.commons.util.collection;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.almendra.commons.util.RandomUtility;


 /** 
  * Defines some utility methods to deal with lists not found 
  * in other classes of the Java collection framework.
  * 
  * @author Oliver Plohmann
  */

public class ListUtility
{

	/**
	 * Creates a new <code>ListUtility</code> object. Not used since all methods are static.
	 */
	protected ListUtility()
	{
		super();
	}

	/**
	 * Create a list from two given objects.
	 * @param object1
	 * @param object2
	 * @return
	 */
	public static List create(Object object1, Object object2)
	{
		List list = new ArrayList();
		list.add(object1);
		list.add(object2);
		return list;
	}

	/**
	 * Create a list from three given objects.
	 * @param object1
	 * @param object2
	 * @param object3
	 * @return
	 */
	public static List create(Object object1, Object object2, Object object3)
	{
		List list = new ArrayList();
		list.add(object1);
		list.add(object2);
		list.add(object3);
		return list;
	}

	/**
	 * Create a list from four given objects.
	 * @param object1
	 * @param object2
	 * @param object3
	 * @param object4
	 * @return
	 */
	public static List create(Object object1, Object object2, Object object3, Object object4)
	{
		List list = new ArrayList();
		list.add(object1);
		list.add(object2);
		list.add(object3);
		list.add(object4);
		return list;
	}

	/**
	 * Turn an array into a list. Returns a <code>List</code> from which you can remove
	 * elements contrary to Arrays.asList(...) which returns an ArrayList.
	 * @param objects the array of objects to be converted
	 * @return the list
	 */
	public static List toList(Object[] objects)
	{
		ArrayList list = new ArrayList(objects.length);
		for (int i = 0; i < objects.length; i++)
			list.add(objects[i]);
		return list;
	}

	/**
	 * Turn a set into a list. Returns a <code>List</code> from which you can remove
	 * elements contrary to Arrays.asList(...) which returns an ArrayList.
	 * @param set to be converted
	 * @return
	 */
	public static List toList(Set set)
	{
		Iterator it = set.iterator();
		ArrayList list = new ArrayList(set.size());
		while (it.hasNext())
			list.add(it.next());
		return list;
	}

	/**
	 * Turn a collection into a list. Returns a <code>List</code> from which you can remove
	 * elements contrary to Arrays.asList(...) which returns an ArrayList.
	 * @param set to be converted
	 * @return
	 */
	public static List toList(Collection c)
	{
		Iterator it = c.iterator();
		ArrayList list = new ArrayList(c.size());
		while (it.hasNext())
			list.add(it.next());
		return list;
	}
	
	/**
	 * @param entry
	 * @return
	 */
	public static List toList(Object object)
	{
		List list = new ArrayList();
		list.add(object);
		return list;
	}

	/**
	 * @param localCapabilityPaths1
	 * @param localCapabilityPaths2
	 * @return
	 */
	public static List getIntersection(Collection coll1, List coll2)
	{
		List intersection = new ArrayList();
		Collection smallerColl = null;
		Collection largerColl = null;
		if(coll1.size() >= coll2.size())
		{
			smallerColl = coll2;
			largerColl = coll1;
		}
		else
		{
			smallerColl = coll1;
			largerColl = coll2;
		}
		Iterator it = smallerColl.iterator();
		while (it.hasNext())
		{
			Object obj = it.next();
			if(largerColl.contains(obj))
				intersection.add(obj);
		}
		return intersection;
	}
	
	public static Object pickAtRandom(List list)
	{
		return list.get(RandomUtility.Instance.nextInt(list.size()));
	}

}
