/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.commons.util.collection;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.almendra.commons.util.RandomUtility;



public class MapUtility
{
	/**
	 * Creates a new <code>MapUtility</code> object.
	 */
	public MapUtility()
	{
		super();
	}
	
	public static Object keyOfValue(Map map, Object value)
	{
		Iterator it = map.entrySet().iterator();
		while (it.hasNext())
		{
			Map.Entry entry = (Map.Entry) it.next();
			if(entry.getValue().equals(value))
				return entry.getKey();
		}
		return null;
	}
	
	public static List getValuesInOrder(Map map, List keysInOrder) throws IllegalArgumentException
	{
		List values = new LinkedList();
		Iterator it = keysInOrder.iterator();
		while (it.hasNext())
		{
			Object obj = it.next();
			Object value = map.get(obj);
			if(value == null)
				throw new IllegalArgumentException("value for " + obj + " missing in map");  //$NON-NLS-1$ //$NON-NLS-2$
			values.add(value);
		}
		return values;
	}
	
	public static List getKeysWithSmallestIntegerValue(Map map)
	{		
		List keys = new ArrayList();
		if(map.isEmpty())
			return keys;
		List values = ListUtility.toList(map.values());
		Collections.sort(values);
		int smallest = ((Integer) values.get(0)).intValue();
		Iterator it = map.entrySet().iterator();
		while (it.hasNext())
		{
			Map.Entry entry = (Map.Entry) it.next();
			int value = ((Integer) entry.getValue()).intValue();
			if(value == smallest)
				keys.add(entry.getKey());
		}
		return keys;
	}
	
	public static Object getKeyWithGreatestIntegerValue(Map map)
	{
		List keys = getKeysWithGreatestIntegerValue(map);
		if(keys.isEmpty())
			return null;
		return keys.get(RandomUtility.Instance.nextInt(keys.size()));
	}
	
	public static List getKeysWithGreatestIntegerValue(Map map)
	{		
		List keys = new ArrayList();
		if(map.isEmpty())
			return keys;
		List values = ListUtility.toList(map.values());
		Collections.sort(values);
		int largest = ((Integer) values.get(values.size() - 1)).intValue();
		Iterator it = map.entrySet().iterator();
		while (it.hasNext())
		{
			Map.Entry entry = (Map.Entry) it.next();
			int value = ((Integer) entry.getValue()).intValue();
			if(value == largest)
				keys.add(entry.getKey());
		}
		return keys;
	}
	
	public static List getKeysWithSmallestLongValue(Map map)
	{		
		List keys = new ArrayList();
		if(map.isEmpty())
			return keys;
		List values = ListUtility.toList(map.values());
		Collections.sort(values);
		int smallest = ((Long) values.get(0)).intValue();
		Iterator it = map.entrySet().iterator();
		while (it.hasNext())
		{
			Map.Entry entry = (Map.Entry) it.next();
			int value = ((Long) entry.getValue()).intValue();
			if(value == smallest)
				keys.add(entry.getKey());
		}
		return keys;
	}

	/**
	 * @param queueSizeByAgentPath
	 * @return
	 */
	public static Object getKeyWithSmallestIntegerValue(Map queueSizeByAgentPath)
	{		
		List keys = getKeysWithSmallestIntegerValue(queueSizeByAgentPath);
		if(keys.isEmpty())
			return null;
		return keys.get(RandomUtility.Instance.nextInt(keys.size()));
	}

	/**
	 * @param agent
	 * @return
	 */
	public static Object getKeyOfValue(Map map, Object value)
	{
		Iterator it = map.entrySet().iterator();
		while (it.hasNext())
		{
			Map.Entry entry = (Map.Entry) it.next();
			if(entry.getValue() == value)
				return entry.getKey();
		}
		return null;
	}

	/**
	 * @param capabilitiesByCandidateNode
	 * @param string
	 * @param nodes
	 */
	public static void putAll(Map map, Object value, Collection collection)
	{
		Iterator it = collection.iterator();
		while (it.hasNext())
		{
			Object key = it.next();
			List values = (List) map.get(key);
			if(values == null)
			{
				values = new ArrayList();
				map.put(key, values);
			}
			values.add(value);
		}
	}
	
}
