/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.commons.util.collection;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author Oliver Plohmann
 *
 */
public class OrderedHashtable extends Object implements OrderedMap
{
	protected List keysInInsertionOrder = new ArrayList();
	protected Map map = new Hashtable();
	
	/**
	 * Creates a new <code>OrderedHashtable</code> object.
	 */
	public OrderedHashtable()
	{
		super();
	}
	
	/**
	 * 
	 */
	public void clear()
	{
		map.clear();
	}
	/**
	 * @param key
	 * @return
	 */
	public boolean containsKey(Object key)
	{
		return map.containsKey(key);
	}
	/**
	 * @param value
	 * @return
	 */
	public boolean containsValue(Object value)
	{
		return map.containsValue(value);
	}
	/**
	 * @return
	 */
	public Set entrySet()
	{
		return map.entrySet();
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj)
	{
		return map.equals(obj);
	}
	/**
	 * @param key
	 * @return
	 */
	public Object get(Object key)
	{
		return map.get(key);
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode()
	{
		return map.hashCode();
	}
	/**
	 * @return
	 */
	public boolean isEmpty()
	{
		return map.isEmpty();
	}
	/**
	 * @return
	 */
	public Set keySet()
	{
		return map.keySet();
	}
	/**
	 * @param key
	 * @param value
	 * @return
	 */
	public Object put(Object key, Object value)
	{
		keysInInsertionOrder.add(key);
		return map.put(key, value);
	}
	/**
	 * @param t
	 */
	public void putAll(Map t)
	{
		map.putAll(t);
	}
	/**
	 * @param key
	 * @return
	 */
	public Object remove(Object key)
	{
		keysInInsertionOrder.remove(key);
		return map.remove(key);
	}
	/**
	 * @return
	 */
	public int size()
	{
		return map.size();
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		return map.toString();
	}
	/**
	 * @return
	 */
	public Collection values()
	{
		return map.values();
	}
	
	public Collection valuesInOrder()
	{
		List valuesInOrder = new ArrayList();
		Iterator it = keysInInsertionOrder.iterator();
		while (it.hasNext())
			valuesInOrder.add(map.get(it.next()));
		return valuesInOrder;
	}
}
