/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.commons.util.collection;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.NoSuchElementException;


public class SortedList implements Collection
{
	protected List sortedList = new ArrayList();
	protected Comparator comparator = null;
	
	/**
	 * @param o
	 * @return
	 */
	public boolean add(Comparable c)
	{
		for (int i = 0; i < sortedList.size(); i++)
		{
			Comparable element = (Comparable) sortedList.get(i);
			if(comparator != null)
			{
				if(comparator.compare(element, c) > 0)
				{	
					sortedList.add(i, c);
					return true;
				}
				sortedList.add(c);
				return true;
			}
			if(element.compareTo(c) > 0)
			{
				sortedList.add(i, c);
				return true;
			}	
		}
		sortedList.add(c);
		return true;
	}

	/**
	 * @param c
	 * @return
	 */
	public boolean addAll(SortedList l)
	{
		Iterator it = l.iterator();
		while (it.hasNext())
		{
			add((Comparable) it.next());
		}
		return l.size() > 0;
	}

	/**
	 * 
	 */
	public void clear()
	{
		sortedList.clear();
	}

	/**
	 * @param o
	 * @return
	 */
	public boolean contains(Object o)
	{
		return sortedList.contains(o);
	}

	/**
	 * @param c
	 * @return
	 */
	public boolean containsAll(Collection c)
	{
		return sortedList.containsAll(c);
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj)
	{
		return sortedList.equals(obj);
	}

	/**
	 * @param index
	 * @return
	 */
	public Object get(int index)
	{
		return sortedList.get(index);
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode()
	{
		return sortedList.hashCode();
	}

	/**
	 * @param o
	 * @return
	 */
	public int indexOf(Object o)
	{
		return sortedList.indexOf(o);
	}

	/**
	 * @return
	 */
	public boolean isEmpty()
	{
		return sortedList.isEmpty();
	}

	/**
	 * @return
	 */
	public Iterator iterator()
	{
		return sortedList.iterator();
	}

	/**
	 * @param o
	 * @return
	 */
	public int lastIndexOf(Object o)
	{
		return sortedList.lastIndexOf(o);
	}

	/**
	 * @return
	 */
	public ListIterator listIterator()
	{
		return sortedList.listIterator();
	}

	/**
	 * @param index
	 * @return
	 */
	public ListIterator listIterator(int index)
	{
		return sortedList.listIterator(index);
	}

	/**
	 * @param index
	 * @return
	 */
	public Object remove(int index)
	{
		return sortedList.remove(index);
	}

	/**
	 * @param o
	 * @return
	 */
	public boolean remove(Object o)
	{
		return sortedList.remove(o);
	}

	/**
	 * @param c
	 * @return
	 */
	public boolean removeAll(Collection c)
	{
		return sortedList.removeAll(c);
	}

	/**
	 * @param c
	 * @return
	 */
	public boolean retainAll(Collection c)
	{
		return sortedList.retainAll(c);
	}

	/**
	 * @return
	 */
	public int size()
	{
		return sortedList.size();
	}

	/**
	 * @param fromIndex
	 * @param toIndex
	 * @return
	 */
	public List subList(int fromIndex, int toIndex)
	{
		return sortedList.subList(fromIndex, toIndex);
	}

	/**
	 * @return
	 */
	public Object[] toArray()
	{
		return sortedList.toArray();
	}

	/**
	 * @param a
	 * @return
	 */
	public Object[] toArray(Object[] a)
	{
		return sortedList.toArray(a);
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		return sortedList.toString();
	}

	/**
	 * Creates a new <code>SortedList</code> object.
	 */
	public SortedList()
	{
		super();
	}
	
	/**
	 * Creates a new <code>SortedList</code> object.
	 */
	public SortedList(Comparator comparator)
	{
		super();
		this.comparator = comparator;
	}
	
	/**
	 * Creates a new <code>SortedList</code> object.
	 */
	public SortedList(Collection c)
	{
		super();
		addAll(c);
	}
	
	/**
	 * Creates a new <code>SortedList</code> object.
	 */
	public SortedList(SortedList l)
	{
		super();
		addAll(l);
	}
	
	/**
	 * Creates a new <code>SortedList</code> object.
	 */
	public SortedList(SortedList l, Comparator comparator)
	{
		super();
		this.comparator = comparator;
		addAll(l);
	}
	
	/**
	 * Creates a new <code>SortedList</code> object.
	 */
	public SortedList(Collection c, Comparator comparator)
	{
		super();
		this.comparator = comparator;
		addAll(c);
	}

	/* (non-Javadoc)
	 * @see java.util.SortedSet#first()
	 */
	public Object first() throws NoSuchElementException
	{
		if(sortedList.isEmpty())
			throw new NoSuchElementException();
		return sortedList.get(0);
	}

	/* (non-Javadoc)
	 * @see java.util.SortedSet#last()
	 */
	public Object last()throws NoSuchElementException
	{
		if(sortedList.isEmpty())
			throw new NoSuchElementException();
		return sortedList.get(sortedList.size() - 1);
	}

	/* (non-Javadoc)
	 * @see java.util.SortedSet#comparator()
	 */
	public Comparator comparator()
	{
		return comparator;
	}

	/* (non-Javadoc)
	 * @see java.util.SortedSet#headSet(java.lang.Object)
	 */
	public SortedList headSet(Object toElement) throws IllegalArgumentException
	{
		int pos = sortedList.indexOf(toElement);
		if(pos == -1)
			throw new IllegalArgumentException("toElement out of range: " + toElement); //$NON-NLS-1$
		return new SortedList(sortedList.subList(0, pos));
	}

	/* (non-Javadoc)
	 * @see java.util.SortedSet#tailSet(java.lang.Object)
	 */
	public SortedList tailSet(Object fromElement)
	{
		int pos = sortedList.indexOf(fromElement);
		if(pos == -1)
			throw new IllegalArgumentException("fromElement out of range: " + fromElement); //$NON-NLS-1$
		return new SortedList(sortedList.subList(pos, sortedList.size() - 1));
	}

	/* (non-Javadoc)
	 * @see java.util.SortedSet#subSet(java.lang.Object, java.lang.Object)
	 */
	public SortedList subSet(Object fromElement, Object toElement)
	{
		int fromPos = sortedList.indexOf(fromElement);
		if(fromPos == -1)
			throw new IllegalArgumentException("fromElement out of range: " + fromElement); //$NON-NLS-1$
		int toPos = sortedList.indexOf(toElement);
		if(toPos == -1)
			throw new IllegalArgumentException("toElement out of range: " + toElement); //$NON-NLS-1$
		return new SortedList(sortedList.subList(fromPos, toPos));
	}

	/* (non-Javadoc)
	 * @see java.util.Set#addAll(java.util.Collection)
	 */
	public boolean addAll(Collection c)
	{
		Iterator it = c.iterator();
		while (it.hasNext())
		{
			add((Comparable) it.next());
		}
		return c.size() > 0;
	}

	/* (non-Javadoc)
	 * @see java.util.Collection#add(java.lang.Object)
	 */
	public boolean add(Object o)
	{
		if(o instanceof Comparable)
		{
			Comparable c = (Comparable) o;
			add(c);
			return true;
		}
		return false;
	}

}
