/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.commons.util.collection;

import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;


public class StringVector
{
	protected Vector strings = new Vector();
	
	/**
	 * @return
	 */
	public Iterator iterator()
	{
		return strings.iterator();
	}

	/**
	 * Creates a new <code>StringVector</code> object.
	 * 
	 */
	public StringVector()
	{
		super();
	}
	
	/**
	 * Creates a new <code>StringVector</code> object.
	 * 
	 */
	public StringVector(StringVector vector)
	{
		this(vector.toArray());
	}
	
	/**
	 * Creates a new <code>StringVector</code> object.
	 * 
	 */
	public StringVector(String[] stringArray)
	{
		super();
		for (int i = 0; i < stringArray.length; i++)
		{
			strings.add(stringArray[i]);
		}
	}

	/**
	 * Creates a new <code>StringVector</code> object.
	 */
	public StringVector(Collection stringCollection)
	{
		super();
		addAll(stringCollection);
	}

	/**
	 * Creates a new <code>StringVector</code> object.
	 */
	public StringVector(String string)
	{
		super();
		add(string);
	}

	/**
	 * @param index
	 * @param element
	 */
	public void add(int index, String element)
	{
		strings.add(index, element);
	}

	/**
	 * @param o
	 * @return
	 */
	public boolean add(String element)
	{
		return strings.add(element);
	}

	/**
	 * @param index
	 * @param c
	 * @return
	 */
	public boolean addAll(int index, Collection c) throws IllegalArgumentException
	{
		checkContainsStringsOnly(c);
		return strings.addAll(index, c);
	}

	/**
	 * @param c
	 * @return
	 */
	public boolean addAll(Collection c) throws IllegalArgumentException
	{
		checkContainsStringsOnly(c);
		return strings.addAll(c);
	}
	
	/**
	 * @param c
	 */
	protected void checkContainsStringsOnly(Collection c) throws IllegalArgumentException
	{
		Iterator it = c.iterator();
		while (it.hasNext())
		{
			Object obj = it.next();
			boolean isString = obj instanceof String;
			if(!isString)
				throw new IllegalArgumentException("collection contains non-string value: " + obj); //$NON-NLS-1$
		}
	}

	/**
	 * @param c
	 * @return
	 */
	public boolean addAll(StringVector v)
	{
		Iterator it = v.iterator();
		while (it.hasNext())
		{
			strings.add(it.next());
		}
		return true;
	}
	
	/**
	 * @param o
	 * @return
	 */
	public boolean contains(String element)
	{
		return strings.contains(element);
	}

	/**
	 * @param c
	 * @return
	 */
	public boolean containsAll(Collection c)
	{
		return strings.containsAll(c);
	}

	/**
	 * @return
	 */
	public Enumeration elements()
	{
		return strings.elements();
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj)
	{
		return strings.equals(obj);
	}

	/**
	 * @return
	 */
	public String firstElement()
	{
		return (String) strings.firstElement();
	}

	/**
	 * @param index
	 * @return
	 */
	public String get(int index)
	{
		return (String) strings.get(index);
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode()
	{
		return strings.hashCode();
	}

	/**
	 * @param o
	 * @return
	 */
	public int indexOf(String element)
	{
		return strings.indexOf(element);
	}

	/**
	 * @param elem
	 * @param index
	 * @return
	 */
	public int indexOf(String elem, int index)
	{
		return strings.indexOf(elem, index);
	}

	/**
	 * @param obj
	 * @param index
	 */
	public void insertElementAt(String elem, int index)
	{
		strings.insertElementAt(elem, index);
	}

	/**
	 * @return
	 */
	public boolean isEmpty()
	{
		return strings.isEmpty();
	}

	/**
	 * @return
	 */
	public String lastElement()
	{
		return (String) strings.lastElement();
	}

	/**
	 * @param o
	 * @return
	 */
	public int lastIndexOf(String elem)
	{
		return strings.lastIndexOf(elem);
	}

	/**
	 * @param elem
	 * @param index
	 * @return
	 */
	public int lastIndexOf(String elem, int index)
	{
		return strings.lastIndexOf(elem, index);
	}

	/**
	 * @param index
	 * @return
	 */
	public String remove(int index)
	{
		return (String) strings.remove(index);
	}

	/**
	 * @param o
	 * @return
	 */
	public boolean remove(String o)
	{
		return strings.remove(o);
	}

	/**
	 * @param c
	 * @return
	 */
	public boolean removeAll(Collection c)
	{
		return strings.removeAll(c);
	}

	/**
	 * 
	 */
	public void removeAllElements()
	{
		strings.removeAllElements();
	}

	/**
	 * @param obj
	 * @return
	 */
	public boolean removeElement(String elem)
	{
		return strings.removeElement(elem);
	}

	/**
	 * @param index
	 */
	public void removeElementAt(int index)
	{
		strings.removeElementAt(index);
	}

	/**
	 * @param index
	 * @param element
	 * @return
	 */
	public String set(int index, String element)
	{
		return (String) strings.set(index, element);
	}

	/**
	 * @return
	 */
	public int size()
	{
		return strings.size();
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		return strings.toString();
	}

	/**
	 * 
	 */
	public String[] toArray()
	{
		String[] array = new String[strings.size()];
		for (int i = 0; i < strings.size(); i++)
		{
			array[i] = (String) strings.get(i); 
		}
		return array;
	}
	
	public List toList()
	{
		List list = new Vector();
		for (int i = 0; i < strings.size(); i++)
			list.add(strings.get(i));
		return list;
	}

}
