/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.commons.util.collection;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * @author Oliver
 *
 */
public class TreeNode
{
	protected Object value = null;
	protected Object visitorObject = null;
	protected TreeNode parent = null;
	protected List children = new LinkedList();
	
	/**
	 * Creates a new <code>TreeNode</code> object.
	 */
	public TreeNode()
	{
		super();
	}
	
	public TreeNode(Object value)
	{
		super();
		this.value = value;
	}
	
	public TreeNode(TreeNode parent, Object value)
	{
		super();
		this.parent = parent;
		this.value = value;
	}

	/**
	 * @return Returns the value.
	 */
	public Object getValue()
	{
		return value;
	}

	/**
	 * @return Returns the children.
	 */
	public List getChildren()
	{
		return children;
	}
	
	public TreeNode addChild(Object child)
	{
		TreeNode node = new TreeNode(this, child);
		children.add(node);
		return node;
	}
	
	public void addChildren(List children)
	{
		Iterator it = children.iterator();
		while (it.hasNext())
			addChild(it.next());
	}
	
	public TreeNode addChildTree(TreeNode child)
	{ 
		child.parent = this;
		children.add(child);
		return child;
	}
	
	public void visit(IVisitor visitor)
	{
		visitor.accept(this);
		Iterator it = getChildren().iterator();
		while (it.hasNext())
		{
			TreeNode node = (TreeNode) it.next();
			node.visit(visitor);
		}
	}

	/**
	 * @return Returns the visitorObject.
	 */
	public Object getVisitorObject()
	{
		return visitorObject;
	}

	/**
	 * @param visitorObject The visitorObject to set.
	 */
	public void setVisitorObject(Object visitorObject)
	{
		this.visitorObject = visitorObject;
	}

	/**
	 * @return Returns the parent.
	 */
	public TreeNode getParent()
	{
		return parent;
	}

}
