/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.oliverplohmann.de/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.commons.util.exception;

/**
 * @author Oliver
 *  
 */
public class PropagatedException extends Exception
{

	protected Exception exception = null;

	/**
	 * Creates a new <code>PropagatedException</code> object.
	 */
	public PropagatedException()
	{
		super();
	}

	/**
	 * Creates a new <code>PropagatedException</code> object.
	 */
	public PropagatedException(String message)
	{
		super(message);
	}

	/**
	 * Creates a new <code>PropagatedException</code> object.
	 */
	public PropagatedException(String message, Throwable cause)
	{
		super(message, cause);
	}

	/**
	 * Creates a new <code>PropagatedException</code> object.
	 */
	public PropagatedException(Throwable cause)
	{
		super(cause);
	}

	/**
	 * @return Returns the exception.
	 */
	public Exception getException()
	{
		return exception;
	}

	/**
	 * @param exception
	 *            The exception to set.
	 */
	public void setException(Exception exception)
	{
		this.exception = exception;
	}

	public static void propagate(Exception ex) throws PropagatedException
	{
		PropagatedException propagatedEx = new PropagatedException(ex.getMessage(), ex.getCause());
		propagatedEx.setException(ex);
		throw propagatedEx;
	}

}
