/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
 
package org.almendra.commons.util.logging;

import java.util.ArrayList;
import java.util.List;

// TODO - code not yet implemented commented

/**
 * An <code>Accessor</code> that controls access to all 
 * 
 * @author Oliver Plohmann
 */
public class LoggerAccessor
{

	protected Logger logger = null;
	
	/**
	 * @param freeText
	 */
	public void freeText(String freeText)
	{
		logger.freeText(freeText);
	}
	public LoggerAccessor()
	{
		super();
	}
	
	public LoggerAccessor(Logger logger)
	{
		super();
		this.logger = logger;
	}	
	
	public List getRegisteredEvents()
	{
		List events = new ArrayList();
		events.addAll(logger.registeredEvents);
		return events;
	}
	
	public List getSubscribtedEvents()
	{
		List events = new ArrayList();
		events.addAll(logger.subscribedEvents);
		return events;
	}
	
	public void subscribeForEvents(List events)
	{
		logger.subscribeForEvents(events);
	}
	
	public void subscribeAll()
	{
		subscribeForEvents(getRegisteredEvents());
	}

	/**
	 * @param listener
	 */
	protected void addListener(ILogListener listener)
	{
		logger.addListener(listener);
	}

	/**
	 * @param listener
	 * @return
	 */
	public boolean removeListener(ILogListener listener)
	{
		return logger.removeListener(listener);
	}

	/**
	 * @param entry
	 */
	public void debug(LogEntry entry)
	{
		logger.debug(entry);
	}

	/**
	 * @param entry
	 */
	public void error(LogEntry entry)
	{
		logger.error(entry);
	}

	/**
	 * @param entry
	 */
	public void exception(LogEntry entry)
	{
		logger.exception(entry);
	}

	/**
	 * @param entry
	 */
	public void fatal(LogEntry entry)
	{
		logger.fatal(entry);
	}

	/**
	 * @param entry
	 */
	public void freeText(LogEntry entry)
	{
		logger.freeText(entry);
	}

	/**
	 * @return
	 */
	public LogEntry getNewLogEntry()
	{
		return logger.getNewLogEntry();
	}

	/**
	 * @param entry
	 */
	public void info(LogEntry entry)
	{
		logger.info(entry);
	}

	/**
	 * @param eventName
	 * @return
	 */
	public boolean isRegistered(String eventName)
	{
		return logger.isRegistered(eventName);
	}

	/**
	 * @param eventName
	 * @return
	 */
	public boolean isSubscribed(String eventName)
	{
		return logger.isSubscribed(eventName);
	}

	/**
	 * @param stringsOfEventNames
	 */
	public void registerForEvents(List stringsOfEventNames)
	{
		logger.registerForEvents(stringsOfEventNames);
	}

	/**
	 * @param stringsOfEventNames
	 */
	public void unsubscribeForEvents(List stringsOfEventNames)
	{
		logger.unsubscribeForEvents(stringsOfEventNames);
	}

	/**
	 * @param entry
	 */
	public void warn(LogEntry entry)
	{
		logger.warn(entry);
	}

}