package org.almendra.commons.util.swing;

import javax.swing.JTextArea;
import javax.swing.text.BadLocationException;

import org.almendra.commons.number.NumberUtility;
import org.almendra.commons.util.AssertUtility;
import org.almendra.commons.util.StringUtility;
import org.almendra.commons.util.logging.ErrorLogger;

import EDU.oswego.cs.dl.util.concurrent.ReentrantLock;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Insert the type's description here. Creation date: (08.03.2004 13:22:52)
 * @author: Administrator
 */
public class BillboardPanelController
{
	public static final String LABEL = "BillboardPanelController"; //$NON-NLS-1$
	public static final String BLANK_LINE = "BLANK_LINE"; //$NON-NLS-1$
	protected String separator = "="; //$NON-NLS-1$
	protected List masksOrder = new ArrayList();
	protected List hiddenMasks = new ArrayList();
	protected Map masks = new java.util.Hashtable();
	protected Map valuesByAttribute = new java.util.Hashtable();
	protected Map nodeAttributeStartPositions = new java.util.Hashtable();
	protected JTextArea textArea = null;
	protected ReentrantLock lock = new ReentrantLock();

	/**
	 * NodeViewController constructor comment.
	 */
	protected BillboardPanelController()
	{
		super();
	}

	/**
	 * NodeViewController constructor comment.
	 */
	public BillboardPanelController(JTextArea textArea)
	{
		super();
		this.textArea = textArea;
	}

	/**
	 * NodeViewController constructor comment.
	 */
	public BillboardPanelController(JTextArea textArea, String separator)
	{
		super();
		this.textArea = textArea;
		this.separator = separator;
	}

	/**
	 * NodeViewController constructor comment.
	 */
	protected void fillInitialMask(List attributeMask)
	{
		Iterator it = attributeMask.iterator();
		while (it.hasNext())
		{
			String attribute = (String) it.next();
			setAttributeValuePair(attribute, "", textArea.getLineCount()); //$NON-NLS-1$
		}
	}

	protected void setAttributeValuePair(String attribute, String value, int line)
	{
		if (BLANK_LINE.equals(attribute))
		{
			textArea.append("\n"); //$NON-NLS-1$
			return;
		}
		valuesByAttribute.put(attribute, value);
		String attributeValuePair = "    " + StringUtility.rightPadded(attribute, 42) + " " + separator + "    " + value + "\n"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		int lineCount = textArea.getLineCount();
		if (lineCount <= line)
		{
			for (int i = lineCount; i < line; i++)
			{
				textArea.append("\n"); //$NON-NLS-1$
			}
			textArea.append(attributeValuePair); //$NON-NLS-1$
			return;
		}
		try
		{
			int start = textArea.getLineStartOffset(line);
			int end = textArea.getLineEndOffset(line);
			textArea.replaceRange(attributeValuePair, start, end);
		}
		catch (BadLocationException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			ErrorLogger.exception(e);
		}
	}

	protected Integer getNextFreeItemNumber()
	{
		int itemNumber = 0;
		Iterator it = masks.values().iterator();
		while (it.hasNext())
		{
			List attributeMask = (List) it.next();
			itemNumber += attributeMask.size();
		}
		return new Integer(itemNumber);
	}

	/**
	 * NodeViewController constructor comment.
	 */
	public void insertAttribute(String maskName, String newAttribute, String afterAttributeOrNull, String value)
	{
		try
		{
			lock.acquire();
			if (!masks.containsKey(maskName))
				return;
			List attributeMask = (List) masks.get(maskName);
			if (attributeMask.contains(newAttribute))
				return;
			String afterAttribute = afterAttributeOrNull;
			if (afterAttribute == null || !attributeMask.contains(afterAttribute))
			{
				afterAttribute = (String) attributeMask.get(attributeMask.size() - 1);
				attributeMask.add(newAttribute);
			}
			else
			{
				attributeMask.add(attributeMask.indexOf(afterAttribute) + 1, newAttribute);
			}
			int lastLinePos = getAttributePos(maskName, afterAttribute) + 1;
			textArea.insert("", lastLinePos); //$NON-NLS-1$
			setAttributeValuePair(newAttribute, value, lastLinePos);
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			ErrorLogger.exception(e);
		}
		finally
		{
			lock.release();
		}
	}

	/**
	 * NodeViewController constructor comment.
	 */
	public void setMask(String maskName, List attributeMask)
	{
		try
		{
			lock.acquire();
			if (masks.containsKey(maskName))
				return;
			masks.put(maskName, attributeMask);
			masksOrder.add(maskName);
			fillInitialMask(attributeMask);
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			ErrorLogger.exception(e);
		}
		finally
		{
			lock.release();
		}
	}

	public void incrementValue(String maskName, String attributeName)
	{
		changeValue(maskName, attributeName, true);
	}

	public void decrementValue(String maskName, String attributeName)
	{
		changeValue(maskName, attributeName, false);
	}

	public String getValue(String maskName, String attributeName)
	{
		try
		{
			lock.acquire();
			if(hiddenMasks.contains(maskName))
				return null;
			int attributePos = getAttributePos(maskName, attributeName);
			if (attributePos == -1)
				return null;
			String element = ""; //$NON-NLS-1$
			try
			{
				int start = textArea.getLineStartOffset(attributePos);
				int end = textArea.getLineEndOffset(attributePos);
				element = textArea.getText(start, end - start);
			}
			catch (BadLocationException e)
			{
				System.out.println(e.getMessage());
				e.printStackTrace();
				ErrorLogger.exception(e);
				return null;
			}
			int index = element.indexOf(separator);
			AssertUtility.assertTrue(index != 1);
			return element.substring(index + 1, element.length()).trim();
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			ErrorLogger.exception(e);
		}
		finally
		{
			lock.release();
		}
		return null;
	}
	
	protected void changeValue(String maskName, String attributeName, boolean increment)
	{
		try
		{
			lock.acquire();
			if(hiddenMasks.contains(maskName))
				return;
			int attributePos = getAttributePos(maskName, attributeName);
			if (attributePos == -1)
				return;
			// TODO - check whether element really contains line
			String element = ""; //$NON-NLS-1$
			try
			{
				int start = textArea.getLineStartOffset(attributePos);
				int end = textArea.getLineEndOffset(attributePos);
				element = textArea.getText(start, end - start);
			}
			catch (BadLocationException e)
			{
				System.out.println(e.getMessage());
				e.printStackTrace();
				ErrorLogger.exception(e);
				return;
			}
			int index = element.indexOf(separator);
			AssertUtility.assertTrue(index != 1);
			String value = element.substring(index + 1, element.length()).trim();
			try
			{
				int number = Integer.parseInt(value);
				if (increment)
					number++;
				else
					number--;
				setValue(maskName, attributeName, String.valueOf(number));
			}
			catch (NumberFormatException e)
			{
				System.out.println("not a number: " + value); //$NON-NLS-1$
				System.out.println(e.getMessage());
				e.printStackTrace();
				ErrorLogger.exception(e);
				return;
			}
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			ErrorLogger.exception(e);
		}
		finally
		{
			lock.release();
		}
	}

	protected int getAttributePos(String maskName, String attributeName)
	{
		int nodePos = getMaskStartPosition(maskName);
		if (nodePos == -1)
			return -1;
		int internalAttributePos = ((List) masks.get(maskName)).indexOf(attributeName);
		return nodePos + internalAttributePos;
	}

	protected int getMaskStartPosition(String maskName)
	{
		if (!masksOrder.contains(maskName))
			return -1;
		int startPos = 0;
		Iterator it = masksOrder.iterator();
		while (it.hasNext())
		{
			String currentMaskName = (String) it.next();
			if (currentMaskName.equals(maskName))
				return startPos;
			List mask = (List) masks.get(currentMaskName);
			startPos += mask.size();
		}
		return startPos;
	}

	/**
	 * NodeViewController constructor comment.
	 */
	public void setValue(String maskName, String attributeName, String value)
	{
		try
		{
			lock.acquire();
			if(hiddenMasks.contains(maskName))
				return;
			int attributePos = getAttributePos(maskName, attributeName);
			if (attributePos == -1)
				return;
			int lineCount = textArea.getLineCount();
			for (int i = lineCount; i <= attributePos; i++)
			{
				textArea.append(""); //$NON-NLS-1$
			}
			setAttributeValuePair(attributeName, value, attributePos);
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			ErrorLogger.exception(e);
		}
		finally
		{
			lock.release();
		}
	}

	/**
	 * 
	 */
	public void reset()
	{
		try
		{
			lock.acquire();
			textArea.setText(""); //$NON-NLS-1$
			masks = new Hashtable();
			nodeAttributeStartPositions = new Hashtable();
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			ErrorLogger.exception(e);
		}
		finally
		{
			lock.release();
		}
	}

	/**
	 * @param string
	 */
	public void unsetMask(String maskName)
	{
		try
		{
			lock.acquire();
			if(hiddenMasks.contains(maskName))
				return;
			int nodeStartPos = getMaskStartPosition(maskName);
			if (nodeStartPos == -1)
				return;
			List mask = (List) masks.get(maskName);
			for (int i = 0; i < mask.size(); i++)
			{
				if (textArea.getLineCount() > nodeStartPos)
				{
					try
					{
						int start = textArea.getLineStartOffset(nodeStartPos);
						int end = textArea.getLineEndOffset(nodeStartPos);
						textArea.replaceRange("", start, end); //$NON-NLS-1$
					}
					catch (BadLocationException e)
					{
						System.out.println(e.getMessage());
						e.printStackTrace();
						ErrorLogger.exception(e);
						return;
					}
				}
			}
			masks.remove(maskName);
			masksOrder.remove(maskName);
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			ErrorLogger.exception(e);
		}
		finally
		{
			lock.release();
		}
	}

	/**
	 * 
	 */
	public void resetCounts()
	{
		try
		{
			lock.acquire();
			Iterator it = masks.entrySet().iterator();
			while (it.hasNext())
			{
				Map.Entry entry = (Map.Entry) it.next();
				String maskName = (String) entry.getKey();
				List values = (List) entry.getValue();
				Iterator innerIt = values.iterator();
				while (innerIt.hasNext())
				{
					String attributeName = (String) innerIt.next();
					String value = getValue(maskName, attributeName);
					if(NumberUtility.isNumeric(value))
						setValue(maskName, attributeName, "0"); //$NON-NLS-1$
				}
			}
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			ErrorLogger.exception(e);
		}
		finally
		{
			lock.release();
		}
	}
	
	public List getMaskNames()
	{
		try
		{
			lock.acquire();
			List maskNames = new ArrayList();
			Iterator it = masks.keySet().iterator();
			while (it.hasNext())
			{
				maskNames.add(it.next());
			}
			return maskNames;
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			ErrorLogger.exception(e);
		}
		finally
		{
			lock.release();
		}
		return null;
	}

	/**
	 * @param list
	 */
	public void hideMasks(List maskNames)
	{
		try
		{
			lock.acquire();
			Iterator it = maskNames.iterator();
			while (it.hasNext())
			{
				String maskName = (String) it.next();
				unsetMask(maskName);
				hiddenMasks.add(maskName);
			}
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			ErrorLogger.exception(e);
		}
		finally
		{
			lock.release();
		}
	}
}
