/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.commons.util.xml;

import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;

import org.almendra.commons.util.AssertUtility;
import org.almendra.commons.util.StringUtility;
import org.almendra.commons.util.collection.StringAssociation;


/**
 * @author Oliver
 *
 */
public class XMLUtility
{
	public static final String TAG_TAIL = "/>"; //$NON-NLS-1$
	
	/**
	 * Creates a new <code>XMLUtility</code> object.
	 */
	public XMLUtility()
	{
		super();
	}

	/** 
	 * In a string like "<LogEntry type="INFO" event="NODE_STARTING_UP">" extract value "INFO" for key "type". 
	 * @param entryString
	 * @param string
	 * @return
	 */
	public static String getXMLValue(String entryString, String key)
	{
		key += "=\""; //$NON-NLS-1$
		int index = entryString.indexOf(key);
		if(index == -1)
			return null;
		String value = entryString.substring(index + key.length(), entryString.length());
		if(value.length() == 0)
			return null;
		int end = value.indexOf('"');
		AssertUtility.assertTrue("ERROR in XMLUtility.getXMLValue(" + entryString + ", " + key, end != -1); //$NON-NLS-1$ //$NON-NLS-2$
		return value.substring(0, end);
	}

	public static String getXMLValue(String entryString, String key, String nextKey)
	{
		key += "=\""; //$NON-NLS-1$
		int index = entryString.indexOf(key);
		if(index == -1)
			return null;
		int indexNextKey = entryString.indexOf(nextKey);
		if(indexNextKey == -1)
			return getXMLValue(entryString, key);
		String value = entryString.substring(index + key.length(), indexNextKey);
		return value.trim();
	}

	/**
	 * @param string
	 * @return
	 */
	public static String toExpandedXMLString(String xmlStringParameter)
	{
		String header = xmlStringParameter.substring(0, xmlStringParameter.indexOf(' '));
		StringBuffer buffer = new StringBuffer();
		buffer.append(header);
		
		StringAssociation[] assocs = getXMLAttributeValuePairs(xmlStringParameter);
		for (int i = 0; i < assocs.length; i++)
		{
			StringAssociation assoc = assocs[i];
			buffer.append("\n" + StringUtility.TAB + assoc.getKey() + "=" + insertTab(assoc.getValue(), 3)); //$NON-NLS-1$ //$NON-NLS-2$
		}
		
		buffer.append("\n" + TAG_TAIL); //$NON-NLS-1$
		return buffer.toString();
	}
	
	/**
	 * @param string
	 * @param i
	 * @return
	 */
	protected static String insertTab(String stringParameter, int tabCount)
	{
		String string = stringParameter;		
		StringTokenizer tokenizer = new StringTokenizer(string, "\n"); //$NON-NLS-1$
		if(tokenizer.countTokens() == 1)
			return string;
		StringBuffer buffer = new StringBuffer();
		while(tokenizer.hasMoreElements())
		{
			String nextString = (String) tokenizer.nextElement();
			for (int i = 0; i < tabCount; i++)
				buffer.append(StringUtility.TAB);
			buffer.append(nextString);
			if(tokenizer.hasMoreElements())
				buffer.append("\n"); //$NON-NLS-1$
		}
		return buffer.toString();
	}

	public static StringAssociation[] getXMLAttributeValuePairs(String xmlStringParameter)
	{
		String xmlString = xmlStringParameter;
		String delimeter = "=\""; //$NON-NLS-1$
		int start = xmlString.indexOf(' ');
		int end = xmlString.lastIndexOf(TAG_TAIL);
		xmlString = xmlString.substring(start, end).trim();
		
		int nextDelimeterIndex = xmlString.indexOf(delimeter);
		if(nextDelimeterIndex == -1)
			return new StringAssociation[0];
		
		List assocs = new LinkedList();
		
		while(nextDelimeterIndex != -1)
		{
			String key = xmlString.substring(0, nextDelimeterIndex).trim();
			xmlString = xmlString.substring(nextDelimeterIndex + delimeter.length());
			int chunkEndIndex = xmlString.indexOf('"');
			String value = xmlString.substring(0, chunkEndIndex);
			xmlString = xmlString.substring(chunkEndIndex + 1);
			nextDelimeterIndex = xmlString.indexOf(delimeter);
			assocs.add(new StringAssociation(key, StringUtility.print(value)));
		}
		
		StringAssociation[] assocsArray = new StringAssociation[assocs.size()];
		for (int i = 0; i < assocs.size(); i++)
		{
			assocsArray[i] = (StringAssociation) assocs.get(i);
		}
		return assocsArray;
	}

}
