/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.ade.demo.fibonacci;

import java.io.Serializable;
import java.math.BigInteger;

import org.almendra.janet.cas.scheduling.IWorkLoadBalancingCommand;



public class FibonacciBalancingCommand extends FibonacciCommand implements IWorkLoadBalancingCommand, Serializable
{
	public static final String QualifiedName = "org.almendra.commons.janet.ade.demo.fibonacci.FibonacciBalancingCommand"; //$NON-NLS-1$
	
	protected int completedTimes = 0;
	protected int completedIterations = 0;
	
	protected BigInteger a1 = BigInteger.ZERO;
	protected BigInteger a2 = BigInteger.ONE;
	
	/**
	 * Creates a new <code>FibonacciBalancingCommand</code> object.
	 */
	protected FibonacciBalancingCommand()
	{
		super();
	}
	
	/**
	 * Creates a new <code>FibonacciBalancingCommand</code> object.
	 */
	public FibonacciBalancingCommand(int input, int times)
	{
		super(input, times);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.ICommand#getQualifiedName()
	 */
	public String getQualifiedName()
	{
		return QualifiedName;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.ICommand#toLogString()
	 */
	public String toLogString()
	{
		return "FibonacciBalancingCommand"; //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	public Object clone() throws CloneNotSupportedException
	{
		FibonacciBalancingCommand cmd = (FibonacciBalancingCommand) super.clone();
		return cmd;
	}
	
	/**
	 * @return Returns the completedTimes.
	 */
	public int getCompletedTimes()
	{
		return completedTimes;
	}
	/**
	 * @param completedTimes The completedTimes to set.
	 */
	public void setCompletedTimes(int times)
	{
		this.completedTimes = times;
	}
	/**
	 * @return Returns the completedIterations.
	 */
	public int getCompletedIterations()
	{
		return completedIterations;
	}
	/**
	 * @param completedIterations The completedIterations to set.
	 */
	public void setCompletedIterations(int iterations)
	{
		this.completedIterations = iterations;
	}

	/**
	 * @return Returns the totalTimes.
	 */
	public int getTotalTimes()
	{
		return totalTimes;
	}
	/**
	 * @param totalTimes The totalTimes to set.
	 */
	public void setTotalTimes(int totalTimes)
	{
		this.totalTimes = totalTimes;
	}

	/**
	 * @return Returns the a1.
	 */
	public BigInteger getA1()
	{
		return a1;
	}

	/**
	 * @return Returns the a2.
	 */
	public BigInteger getA2()
	{
		return a2;
	}

	/**
	 * @param a1 The a1 to set.
	 */
	public void setA1(BigInteger a1)
	{
		this.a1 = a1;
	}

	/**
	 * @param a2 The a2 to set.
	 */
	public void setA2(BigInteger a2)
	{
		this.a2 = a2;
	}
}
