/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.ade.demo.fibonacci;

import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.almendra.commons.events.ICallbackHandler;
import org.almendra.commons.threads.ThreadUtility;
import org.almendra.commons.util.collection.StringVector;
import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.node.LocalAgentPath;
import org.almendra.janet.cas.node.gui.NodeViewsRegistry;
import org.almendra.janet.cas.scheduling.CommandAccessor;
import org.almendra.janet.cas.scheduling.IInterpreter;



public class StartInterpreter implements IInterpreter
{
	/**
	 * Creates a new <code>StartInterpreter</code> object.
	 */
	public StartInterpreter()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#execute(org.almendra.commons.janet.cas.scheduling.CommandAccessor)
	 */
	public void execute(CommandAccessor cmdAccessor)
	{
		if(cmdAccessor.getCommand() instanceof StartCommand)
		{
			Map allApplicationObjects = storeApplicationObjects(cmdAccessor);
			if(allApplicationObjects.size() < 3)
				return;
			
			Set nodeNames = getAllNodesNamesWithCapability(cmdAccessor);
			cmdAccessor.setNodeLabel(FibonacciUtility.NODE_NAME);
			setNodeViewsRegistry(cmdAccessor, allApplicationObjects, nodeNames);
			
			ThreadUtility.sleep(3000);	// wait till all nodes have replied by sending their agent paths
		}
	}
	
	/**
	 * @param cmdAccessor
	 * @param descriptor
	 */
	protected void setNodeViewsRegistry(CommandAccessor cmdAccessor, Map allApplicationObjects, Set nodeNames)
	{
		NodeViewsRegistry nodeViewsRegistry = (NodeViewsRegistry) cmdAccessor.getNodeObjectSpace().get(NodeViewsRegistry.LABEL);
		ICallbackHandler handler = new OpenFibonacciDemoViewHandler(cmdAccessor, allApplicationObjects, nodeNames);
		nodeViewsRegistry.addViewEntry(FibonacciDemoViewController.LABEL, handler);
	}
	
	/**
	 * @param cmdAccessor
	 * @return
	 */
	protected Set getAllNodesNamesWithCapability(CommandAccessor cmdAccessor)
	{
		Set nodeNames = new HashSet();
		nodeNames.add(cmdAccessor.getNodeName());
		LocalAgentPath path = new LocalAgentPath(cmdAccessor.getApplicationName(), FibonacciUtility.CAP_NAME, FibonacciUtility.AGENT_NAME);
		List agentPaths = cmdAccessor.getApplication().getAllAgentPaths(path);
		Iterator it = agentPaths.iterator();
		while (it.hasNext())
		{
			AgentPath agentPath = (AgentPath) it.next();
			nodeNames.add(agentPath.getNodeName());
		}
		return nodeNames;
	}

	/**
	 * @param cmdAccessor
	 */
	protected Map storeApplicationObjects(CommandAccessor cmdAccessor)
	{
		Map allApplicationObjects = (Map) cmdAccessor.getNodeObjectSpace().get(FibonacciDemoViewController.APPLICATION_LIST_NAME);
		if(allApplicationObjects == null)
		{
			allApplicationObjects = new Hashtable();
			cmdAccessor.getNodeObjectSpace().put(FibonacciDemoViewController.APPLICATION_LIST_NAME, allApplicationObjects);
		}
		
		ApplicationInfo appInfo = new ApplicationInfo(cmdAccessor.getApplication().getName());
		appInfo.setApp(cmdAccessor.getApplication());
		appInfo.setNodeName(cmdAccessor.getNodeName());
		appInfo.setAgentNames(cmdAccessor.getCapability().getAgentNames());
		allApplicationObjects.put(cmdAccessor.getApplication().getName(), appInfo);
		
		return allApplicationObjects;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#commandNames()
	 */
	public StringVector commandNames()
	{
		return new StringVector(StartCommand.QualifiedName);
	}
}
