/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.ade.observer.simulation.interpreters;

import org.almendra.commons.events.ICallbackHandler;
import org.almendra.commons.util.globalId.UniqueIdGenerator;
import org.almendra.commons.util.inet.InetUtility;
import org.almendra.janet.ade.observer.ObserverAnchor;
import org.almendra.janet.ade.observer.simulation.ObserverAnchorSimulation;
import org.almendra.janet.ade.observer.simulation.gui.ObserverSimulationView;
import org.almendra.janet.ade.observer.simulation.gui.ObserverSimulationViewController;
import org.almendra.janet.ade.observer.simulation.gui.OpenObserverSimulationHandler;
import org.almendra.janet.cas.node.Logger;
import org.almendra.janet.cas.node.Node;
import org.almendra.janet.cas.node.SystemApplication;
import org.almendra.janet.cas.node.gui.NodeViewsRegistry;
import org.almendra.janet.cas.scheduling.CommandAccessor;
import org.almendra.janet.cas.scheduling.IInterpreter;



public class StartInterpreter extends org.almendra.janet.ade.observer.interpreters.StartInterpreter implements IInterpreter
{
	/**
	 * Creates a new <code>StartInterpreter</code> object.
	 */
	public StartInterpreter()
	{
		super();
	}

	protected void setViewTitle(CommandAccessor cmdAccessor)
	{
		setViewTitle(cmdAccessor, ObserverAnchorSimulation.VIEW_TITLE_PREFIX);
	}
	
	/**
	 * @return
	 */
	protected ObserverAnchor getAnchorInstance(SystemApplication application, Logger logger)
	{
		return new ObserverAnchorSimulation(application, logger);
	}
	
	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.ade.observer.interpreters.StartInterpreter#startView(org.almendra.commons.janet.cas.scheduling.CommandAccessor)
	 */
	protected void startView(CommandAccessor cmdAccessor)
	{
		NodeViewsRegistry nodeViewsRegistry = (NodeViewsRegistry) cmdAccessor.getNodeObjectSpace().get(NodeViewsRegistry.LABEL);
		ObserverAnchorSimulation anchor = getSimulatorAnchor(cmdAccessor);
		Node node = cmdAccessor.getNodeIfPrivilege();
		
		ICallbackHandler handler = new OpenObserverSimulationHandler(anchor, node);
		nodeViewsRegistry.addViewEntry(ObserverSimulationView.LABEL, handler);
	}
	
	protected ObserverAnchorSimulation getSimulatorAnchor(CommandAccessor cmdAccessor)
	{
		return (ObserverAnchorSimulation) getAnchor(cmdAccessor);
	}
	
	/**
	 * @param anchor
	 */
	protected String getWorkstationName()
	{
		return InetUtility.getHostName() + "-Simulated-" + UniqueIdGenerator.getMillisWithRandom(); //$NON-NLS-1$
	}
}
