/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.ade.commands;

import org.almendra.commons.reflect.ReflectionUtility;
import org.almendra.janet.ade.distributor.DistributorAnchor;
import org.almendra.janet.ade.executor.ExecutorAnchor;
import org.almendra.janet.ade.executor.interpreters.ExecuteWorkloadBalancingInterpreter;
import org.almendra.janet.cas.scheduling.CommandEnvelope;
import org.almendra.janet.cas.scheduling.ICommand;
import org.almendra.janet.cas.scheduling.ICommandHolder;
import org.almendra.janet.cas.scheduling.IInterpreter;
import org.almendra.janet.cas.scheduling.IWorkLoadBalancingCommand;
import org.almendra.janet.cas.scheduling.IWorkLoadBalancingInterpreter;



public class ExecuteWorkloadBalancingCommand extends ExecuteWorkloadCommand implements ICommand, ICommandHolder
{
	public static final String QualifiedName = "org.almendra.commons.janet.ade.commands.ExecuteWorkloadBalancingCommand"; //$NON-NLS-1$
	
	protected IWorkLoadBalancingCommand command = null;
	protected IWorkLoadBalancingInterpreter suspendableInterpreter = null;
	
	/**
	 * Creates a new <code>ExecuteWorkloadBalancingCommand</code> object.
	 */
	public ExecuteWorkloadBalancingCommand()
	{
		super();
	}

	/**
	 * Creates a new <code>ExecuteWorkloadBalancingCommand</code> object.
	 */
	public ExecuteWorkloadBalancingCommand(IWorkLoadBalancingCommand command, String capabilityPath)
	{
		super(capabilityPath);
		this.command = command;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.ICommand#getQualifiedName()
	 */
	public String getQualifiedName()
	{
		return QualifiedName;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.ICommand#getPriority()
	 */
	public int getPriority()
	{
		return ICommand.ApplicationMaxPriority;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.ICommand#toLogString()
	 */
	public String toLogString()
	{
		return "ExecuteWorkloadBalancingCommand(" + command.toLogString() + ")"; //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	/**
	 * @return Returns the command.
	 */
	public IWorkLoadBalancingCommand getBalancingCommand()
	{
		return command;
	}
	
	/**
	 * @return Returns the command.
	 */
	public ICommand getCommand()
	{
		return command;
	}
	
	/**
	 * @return Returns the capabilityPath.
	 */
	public String getQualifiedCapabilityPath()
	{
		return capabilityPath;
	}
	/**
	 * @return Returns the suspendableInterpreter.
	 */
	public IWorkLoadBalancingInterpreter getSuspendableInterpreter()
	{
		return suspendableInterpreter;
	}
	/**
	 * @param suspendableInterpreter The suspendableInterpreter to set.
	 */
	public void setSuspendableInterpreter(IWorkLoadBalancingInterpreter suspendableInterpreter)
	{
		this.suspendableInterpreter = suspendableInterpreter;
	}
	
	public Object clone() throws CloneNotSupportedException
	{
		ExecuteWorkloadBalancingCommand cmd = (ExecuteWorkloadBalancingCommand) super.clone();
		return cmd;
	}
	
	public void suspend()
	{
		if(suspendableInterpreter != null)
			ReflectionUtility.invoke(suspendableInterpreter, "suspend"); //$NON-NLS-1$
	}
	
	public void resume()
	{
		if(suspendableInterpreter != null)
			ReflectionUtility.invoke(suspendableInterpreter, "resume"); //$NON-NLS-1$
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.ade.commands.ExecuteWorkloadCommand#getLogEventName()
	 */
	public String getLogEventName()
	{
		return DistributorAnchor.LOG_EVENT_BALANCING_COMMAND_DISTRIBUTED;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.ade.commands.ExecuteWorkloadCommand#getLogTypeName()
	 */
	public String getLogTypeName()
	{
		return "IWorkLoadBalancingCommand"; //$NON-NLS-1$
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.ade.commands.ExecuteWorkloadCommand#getInterpreter()
	 */
	public IInterpreter getInterpreter()
	{
		return new ExecuteWorkloadBalancingInterpreter();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.ade.commands.ExecuteWorkloadCommand#workloadCommandArrived(org.almendra.commons.janet.ade.executor.ExecutorAnchor, org.almendra.commons.janet.cas.scheduling.CommandEnvelope, java.lang.String)
	 */
	public void workloadCommandArrived(ExecutorAnchor anchor, CommandEnvelope env, String capabilityPath)
	{
		anchor.workloadBalancingCommandArrived(env, capabilityPath);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.ade.commands.ExecuteWorkloadCommand#prepareBeforeSending()
	 */
	public void prepareBeforeSending()
	{
		setSuspendableInterpreter(null);
	}
	
}
