/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.ade.commands;

import java.io.Serializable;

import org.almendra.janet.cas.scheduling.ICommand;



public class NotifyCPULoadChangedCommand implements ICommand, Serializable
{
	public static final String QualifiedName = "org.almendra.commons.janet.ade.commands.NotifyCPULoadChangedCommand"; //$NON-NLS-1$
	
	public static final int CPULoadAboveThreshold = 1;
	public static final int CPULoadBelowThreshold = 2;
	public static final int CPULoadSignificantChange = 3;
	
	protected int previousNonJanetCPULoad = -1;
	protected int currentNonJanetCPULoad = -1;
	protected String workstationName = null;
	protected int change = -1;
	
	/**
	 * Creates a new <code>NotifyCPULoadChangedCommand</code> object.
	 */
	protected NotifyCPULoadChangedCommand()
	{
		super();
	}
	
	/**
	 * Creates a new <code>NotifyCPULoadChangedCommand</code> object.
	 */
	public NotifyCPULoadChangedCommand(String workstationName, int previousNonJanetCPULoad, int currentNonJanetCPULoad, int cpuLoadAboveOrBelow)
	{
		super();
		this.workstationName = workstationName;
		this.previousNonJanetCPULoad = previousNonJanetCPULoad;
		this.currentNonJanetCPULoad = currentNonJanetCPULoad;
		this.change = cpuLoadAboveOrBelow;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.ICommand#getQualifiedName()
	 */
	public String getQualifiedName()
	{
		return QualifiedName;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.ICommand#getPriority()
	 */
	public int getPriority()
	{
		return ICommand.ApplicationMaxPriority;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.ICommand#toLogString()
	 */
	public String toLogString()
	{
		return "NotifyCPULoadChangedCommand"; //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	public Object clone() throws CloneNotSupportedException
	{
		NotifyCPULoadChangedCommand cmd = (NotifyCPULoadChangedCommand) super.clone();
		return cmd;
	}
	/**
	 * @return Returns the workstationName.
	 */
	public String getWorkstationName()
	{
		return workstationName;
	}
	/**
	 * @return Returns the currentNonJanetCPULoad.
	 */
	public int getCurrentNonJanetCPULoad()
	{
		return currentNonJanetCPULoad;
	}
	/**
	 * @return Returns the previousNonJanetCPULoad.
	 */
	public int getPreviousNonJanetCPULoad()
	{
		return previousNonJanetCPULoad;
	}
	/**
	 * @return Returns the change.
	 */
	public int getChange()
	{
		return change;
	}
}
