/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.ade.commands;

import java.io.Serializable;

import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.scheduling.ICommand;



public class WorkloadDistributionCommand implements ICommand, Serializable
{
	public static final String QualifiedName = "org.almendra.commons.janet.ade.commands.WorkloadDistributionCommand"; //$NON-NLS-1$
	
	protected ExecuteWorkloadCommand command = null;
	protected AgentPath agentPath = null;
	
	/**
	 * Creates a new <code>WorkloadDistributionCommand</code> object.
	 */
	protected WorkloadDistributionCommand()
	{
		super();
	}
	
	/**
	 * Creates a new <code>WorkloadDistributionCommand</code> object.
	 */
	public WorkloadDistributionCommand(AgentPath agentPath, ExecuteWorkloadCommand command)
	{
		super();
		this.agentPath = agentPath;
		this.command = command;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.ICommand#getQualifiedName()
	 */
	public String getQualifiedName()
	{
		return QualifiedName;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.ICommand#getPriority()
	 */
	public int getPriority()
	{
		return ICommand.ApplicationMaxPriority;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.ICommand#toLogString()
	 */
	public String toLogString()
	{
		return "WorkloadDistributionCommand"; //$NON-NLS-1$
	}
	

	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	public Object clone() throws CloneNotSupportedException
	{
		WorkloadDistributionCommand cmd = (WorkloadDistributionCommand) super.clone();
		return cmd;
	}
	/**
	 * @return Returns the command.
	 */
	public ExecuteWorkloadCommand getCommand()
	{
		return command;
	}	

	/**
	 * @return Returns the agentPath.
	 */
	public AgentPath getAgentPath()
	{
		return agentPath;
	}

	/**
	 * @return
	 */
	public String getCapabilityPath()
	{
		return agentPath.getCapabilityPath();
	}
}
