/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.ade.distributor;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.almendra.commons.number.Long;



/**
 * @author Oliver Plohmann
 *
 */
public class CapabilityQueueSizeStore
{
	protected Map nodesByCapability = new Hashtable();
	protected Map capabilitiesByNode = new Hashtable();
	protected Map queueSizeByCapabilityAndNode = new Hashtable();
	
	/**
	 * 
	 */
	public CapabilityQueueSizeStore()
	{
		super();
	}
	
	protected void removeCapabilitiesNotInUse(Map queuesByCapability, String nodeName)
	{
		List unusedCapabilities = new ArrayList();
		Set currentCapabilities = (Set) capabilitiesByNode.get(nodeName);
		if(currentCapabilities != null)
		{			
			Iterator it = currentCapabilities.iterator();
			while (it.hasNext())
			{
				String capabilityPath = (String) it.next();
				if(!queuesByCapability.containsKey(capabilityPath))
					unusedCapabilities.add(capabilityPath);
			}
		}
		Iterator it = unusedCapabilities.iterator();
		while (it.hasNext())
		{
			String unusedCapability = (String) it.next();
			Set nodes = (Set) nodesByCapability.get(unusedCapability);
			if(nodes != null)
				nodes.remove(nodeName);
			Set capabilities = (Set) capabilitiesByNode.get(nodeName);
			if(capabilities != null)
				capabilities.remove(unusedCapability);
			queueSizeByCapabilityAndNode.remove(unusedCapability + "." + nodeName); //$NON-NLS-1$
		}
	}
	
	public void update(Map queuesByCapability, String nodeName)
	{
		removeCapabilitiesNotInUse(queuesByCapability, nodeName);
		insertCapabilitiesInUse(queuesByCapability, nodeName);		
	}
	
	public void removeEntries(String nodeName)
	{
		Set capabilities = (Set) capabilitiesByNode.remove(nodeName);
		if(capabilities == null || capabilities.isEmpty())
			return;		
		Iterator it = capabilities.iterator();
		while(it.hasNext())
		{
			String capabilityPath = (String) it.next();
			Set nodes = (Set) nodesByCapability.get(capabilityPath);
			if(nodes != null)
				nodes.remove(nodeName);
			queueSizeByCapabilityAndNode.remove(capabilityPath + "." + nodeName); //$NON-NLS-1$
		}
	}
	
	protected void insertCapabilitiesInUse(Map queuesByCapability, String nodeName)
	{
		Iterator it = queuesByCapability.entrySet().iterator();
		while (it.hasNext())
		{
			Map.Entry entry = (Map.Entry) it.next();
			String capabilityPath = (String) entry.getKey();
			Long queueSize = (Long) entry.getValue();
			Set nodes = (Set) nodesByCapability.get(capabilityPath);
			if(nodes == null)
			{
				nodes = new HashSet();
				nodesByCapability.put(capabilityPath, nodes);
			}
			nodes.add(nodeName);
			Set capabilities = (Set) capabilitiesByNode.get(nodeName);
			if(capabilities == null)
			{
				capabilities = new HashSet();
				capabilitiesByNode.put(nodeName, capabilities);
			}
			capabilities.add(capabilityPath);
			queueSizeByCapabilityAndNode.put(capabilityPath + "." + nodeName, queueSize); //$NON-NLS-1$
		}	
	}
	
	public Set getCapabilities(String nodeName)
	{
		Set capabilities = (Set) capabilitiesByNode.get(nodeName);
		if(capabilities == null)
			capabilities = new HashSet();
		return capabilities;
	}
	
	public Set getNodes(String capabilityPath)
	{
		Set nodes = (Set) nodesByCapability.get(capabilityPath);
		if(nodes == null)
			nodes = new HashSet();
		return nodes;
	}

	/**
	 * @param nodeName
	 * @param capabilityPath
	 */
	public int getQueueSize(String nodeName, String capabilityPath)
	{
		Long queueSize = (Long) queueSizeByCapabilityAndNode.get(capabilityPath + "." + nodeName); //$NON-NLS-1$
		if(queueSize == null)
			return 0;
		return (int) queueSize.longValue();
	}
}
