/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.ade.distributor;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.almendra.janet.cas.node.AgentPath;


/**
 * @author Oliver Plohmann
 *
 */
public class LeastLoadedNodeDecisionInformation extends DecisionInformation
{
	protected List nodeNamesWithCapability = new ArrayList();
	protected List availableNodeNames = new ArrayList();
	protected String leastLoadedNotAvailableNodeName = null;
	protected Map workstationQSCByNodeName = new Hashtable();
	protected List nodeNamesWithSmallestOverAllQSC = new ArrayList();	
	protected List nodeNamesWithSmallestWorkstationQSC = new ArrayList();
	protected String nodeNameWithSmallestOverAllQSC = null;
	protected Map cqsByNode = new Hashtable();
	protected String nodeNameLeastLoaded = null;
	protected AgentPath originAgentPath = null;
	protected String commandClassName = null;
	protected List leastLoadedNodeNames = new ArrayList();
	protected String lastSelectedNodeName = null;
	
	/**
	 * Creates a new <code>LeastLoadedNodeDecisionInformation</code> object.
	 */
	protected LeastLoadedNodeDecisionInformation()
	{
		super();
	}
	
	/**
	 * Creates a new <code>LeastLoadedNodeDecisionInformation</code> object.
	 */
	public LeastLoadedNodeDecisionInformation(AgentPath originAgentPath, String commandClassName)
	{
		super();
		this.originAgentPath = originAgentPath;
		this.commandClassName = commandClassName;
	}

	/**
	 * @param nodeNamesOfNodesWithCapability
	 */
	public void setNodeNamesWithCapability(List nodeNamesWithCapability)
	{
		this.nodeNamesWithCapability.addAll(nodeNamesWithCapability);
	}
	/**
	 * @return Returns the nodeNamesWithCapability.
	 */
	public List getNodeNamesWithCapability()
	{
		return nodeNamesWithCapability;
	}

	/**
	 * @param availableNodeNames
	 */
	public void setAvailableNodeNames(List availableNodeNames)
	{
		this.availableNodeNames = availableNodeNames;
	}

	/**
	 * @param string
	 */
	public void setLeastLoadedNotAvailableNodeName(String leastLoadedNotAvailableNodeName)
	{
		this.leastLoadedNotAvailableNodeName = leastLoadedNotAvailableNodeName;
	}

	/**
	 * @param string
	 * @param integer
	 */
	public void addWorkstationQSCByNodeName(String nodeName, Integer qsc)
	{
		workstationQSCByNodeName.put(nodeName, qsc);
	}

	/**
	 * @param nodesWithSmallestWorkstationQSC
	 */
	public void addNodeNamesWithSmallestWorkstationQSC(List nodesWithSmallestWorkstationQSC)
	{
		this.nodeNamesWithSmallestWorkstationQSC.addAll(nodesWithSmallestWorkstationQSC); 
	}

	/**
	 * @param nodesWithSmallestOverAllQSC
	 */
	public void addNodeNamesWithSmallestOverAllQSC(List nodeNamesWithSmallestOverAllQSC)
	{
		this.nodeNamesWithSmallestOverAllQSC.addAll(nodeNamesWithSmallestOverAllQSC);
	}

	/**
	 * @param nodeNameWithSmallestOverAllQSC
	 */
	public void setNodeNameWithSmallestOverAllQSC(String nodeNameWithSmallestOverAllQSC)
	{
		this.nodeNameWithSmallestOverAllQSC = nodeNameWithSmallestOverAllQSC;
	}

	/**
	 * @param nodeName
	 * @param cqs
	 */
	public void addCQSByNode(String nodeName, Integer cqs)
	{
		cqsByNode.put(nodeName,cqs);
	}

	/**
	 * @param nodeNameLeastLoaded
	 */
	public void setNodeNameLeastLoaded(String nodeNameLeastLoaded)
	{
		this.nodeNameLeastLoaded = nodeNameLeastLoaded;
	}

	protected void printOnPrim(StringBuffer text, String tab)
	{
		printHeader(text, tab);
		printNodeNamesWithCapability(text, tab);	
		printAvailableNodes(text, tab);
		if(!printLeastLoadedNotAvailableNodeName (text, tab))
			return;
		printWorkstationQSCByNodeName(text, tab); 
		printNodesWithSmallestWorkstationQSC(text, tab); 
		printNodeNamesWithSmallestOverAllQSC(text, tab);
		printNodeNameWithSmallestOverAllQSC(text, tab);
		printCQSByNode(text, tab);
		printNodeNamesLeastLoaded(text, tab);
		printNodeNameLeastLoaded(text, tab); 
	}

	/**
	 * @param text
	 * @param tab
	 */
	protected void printNodeNamesLeastLoaded(StringBuffer text, String tab)
	{
		if(leastLoadedNodeNames.isEmpty())
			return;
		text.append(tab);
		text.append("Least loaded nodes: "); //$NON-NLS-1$
		printOnString(text, leastLoadedNodeNames);
	}

	/**
	 * @param text
	 * @param tab
	 */
	protected void printHeader(StringBuffer text, String tab)
	{
		text.append(tab);
		text.append("Looking for node to migrate command " + commandClassName + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
		text.append(tab);
		text.append("from " + originAgentPath.getQualifiedPathName() + "\n\n"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}

	/**
	 * @param text
	 * @param tab
	 */
	protected void printNodeNameLeastLoaded(StringBuffer text, String tab)
	{
		if(nodeNameLeastLoaded == null)
			return;
		text.append(tab);
		text.append("Least loaded node: " + nodeNameLeastLoaded); //$NON-NLS-1$
		if(lastSelectedNodeName != null)
		{
			text.append(" (ignoring last selected node " + lastSelectedNodeName + ")"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		text.append("\n"); //$NON-NLS-1$
	}

	/**
	 * @param text
	 * @param tab
	 */
	protected void printCQSByNode(StringBuffer text, String tab)
	{
		if(cqsByNode.isEmpty())
			return;
		text.append(tab);
		text.append("Node CQSs by node name: "); //$NON-NLS-1$
		printOnStringAssocStringInteger(text, cqsByNode);
	}

	/**
	 * @param text
	 * @param tab
	 */
	protected void printNodeNameWithSmallestOverAllQSC(StringBuffer text, String tab)
	{
		if(nodeNameWithSmallestOverAllQSC == null)
			return;
		text.append(tab);
		text.append("Node with smallest overall QSC: " + nodeNameWithSmallestOverAllQSC + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * @param text
	 * @param tab
	 */
	protected void printNodeNamesWithSmallestOverAllQSC(StringBuffer text, String tab)
	{
		if(nodeNamesWithSmallestOverAllQSC.isEmpty())
			return;
		text.append(tab);
		text.append("Nodes with smallest overall QSC: "); //$NON-NLS-1$
		printOnString(text, nodeNamesWithSmallestOverAllQSC);
	}

	/**
	 * @param text
	 * @param tab
	 */
	protected void printNodesWithSmallestWorkstationQSC(StringBuffer text, String tab)
	{
		if(nodeNamesWithSmallestWorkstationQSC.isEmpty())
			return;
		text.append(tab);
		text.append("Nodes with smalles workstation QSC: "); //$NON-NLS-1$
		printOnString(text, nodeNamesWithSmallestWorkstationQSC);
	}

	/**
	 * @param text
	 * @param tab
	 */
	protected void printWorkstationQSCByNodeName(StringBuffer text, String tab)
	{
		if(workstationQSCByNodeName.isEmpty())
			return;
		text.append(tab);
		text.append("Workstation QSCs by node name: "); //$NON-NLS-1$
		printOnStringAssocStringInteger(text, workstationQSCByNodeName);
	}

	/**
	 * @param text
	 * @param tab
	 * @return
	 */
	protected boolean printLeastLoadedNotAvailableNodeName(StringBuffer text, String tab)
	{
		if(leastLoadedNotAvailableNodeName != null)
		{
			text.append(tab);
			text.append("No node below CPU threshold available! Picking least loaded node from overloaded nodes: " + leastLoadedNotAvailableNodeName + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}
		return true;
	}

	/**
	 * @param tab
	 * @param text
	 */
	protected void printAvailableNodes(StringBuffer text, String tab)
	{
		if(availableNodeNames.isEmpty())
			return;
		text.append(tab);
		text.append("Available nodes: "); //$NON-NLS-1$
		printOnString(text, availableNodeNames);		
	}

	/**
	 * @param text
	 */
	protected void printNodeNamesWithCapability(StringBuffer text, String tab)
	{
		if(nodeNamesWithCapability.isEmpty())
			return;
		text.append(tab);
		text.append("Nodes with required capability: "); //$NON-NLS-1$
		printOnString(text, nodeNamesWithCapability);	
	}

	/**
	 * @param leastLoadedNodes
	 */
	public void setLeastLoadedNodes(List leastLoadedNodeNames)
	{
		leastLoadedNodeNames.addAll(leastLoadedNodeNames);
	}

	/**
	 * @param lastSelectedNodeName
	 */
	public void setLastSelectedNode(String lastSelectNodeName)
	{
		this.lastSelectedNodeName = lastSelectNodeName;
	}
}
