/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.ade.distributor;

import java.util.List;


/**
 * @author Oliver Plohmann
 *
 */
public class NodeDroppedToQSC1DecisionInformation extends HeaviestLoadedNodeDecisionInformation
{
	protected List destinationCapabilities = null;
	
	/**
	 * Creates a new <code>NodeDroppedToQSC1DecisionInformation</code> object.
	 */
	public NodeDroppedToQSC1DecisionInformation(String nodeNameDroppedToQSC1, List nodeNamesAboveQSC1)
	{
		super(nodeNamesAboveQSC1);
		this.nodeNameDroppedToQSC1 = nodeNameDroppedToQSC1;
	}

	/**
	 * @param list
	 */
	public void setDestinationCapabilities(List destinationCapabilities)
	{
		this.destinationCapabilities = destinationCapabilities;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.ade.distributor.DecisionInformation#printOnPrim(java.lang.StringBuffer, java.lang.String)
	 */
	protected void printOnPrim(StringBuffer text, String tab)
	{
		printNodeNameDroppedToQSC1(text, tab);
		// printNodeNamesAboveQSC1(text, tab);		
		// printDestinationCapabilities(text, tab); - not really of interest
		printCandidateNodeNamesWithRequiredCapabilities(text, tab);
		printCQSByCapabilityName(text, tab);
		printHeaviestLoadedNodeNameWithCapabilityName(text, tab);
		printEvictionComment(text, tab);
	}
	
	/**
	 * @param text
	 * @param tab
	 */
	protected void printEvictionComment(StringBuffer text, String tab)
	{
		text.append(tab);
		text.append("Migrating 1 command from " + heaviestLoadedCapabilityName + " on " + heaviestLoadedNodeName + " to " + nodeNameDroppedToQSC1 + ".\n"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
	}

	protected void printDestinationCapabilities(StringBuffer text, String tab)
	{
		if(destinationCapabilities.isEmpty())
			return;
		text.append(tab);
		text.append("Capabilities on " + nodeNameDroppedToQSC1 + ": "); //$NON-NLS-1$ //$NON-NLS-2$
		printOnString(text, destinationCapabilities);
	}
}
