/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.ade.distributor;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.almendra.janet.cas.node.LocalCapabilityPath;



public class NodeLoadImage extends DistributorAnchorHolder
{
	protected String nodeName = null;
	protected int queueSizeCategory = 0;
	protected long waitingTime = 0;
	protected boolean isEvicting = false;
	protected WorkstationLoadImage workstationLoadImage = null;
	
	/**
	 * Creates a new <code>NodeLoadImage</code> object.
	 */
	protected NodeLoadImage()
	{
		super();
	}
	
	/**
	 * Creates a new <code>NodeLoadImage</code> object.
	 */
	public NodeLoadImage(DistributorAnchor anchor, WorkstationLoadImage workstationLoadImage, String nodeName)
	{
		super(anchor);
		this.workstationLoadImage = workstationLoadImage;
		this.nodeName = nodeName;
	}
	
	/**
	 * @return Returns the queueSizeCategory.
	 */
	public int getQueueSizeCategory()
	{
		return queueSizeCategory;
	}
	/**
	 * @param queueSizeCategory The queueSizeCategory to set.
	 */
	public void setQueueSizeCategory(int queueSizeCategory)
	{
		this.queueSizeCategory = queueSizeCategory;
	}
	/**
	 * @return
	 */
	public boolean isEvicting()
	{
		return isEvicting = false;
	}
	
	/**
	 * @param isEvicting The isEvicting to set.
	 */
	public void setEvicting(boolean isEvicting)
	{
		this.isEvicting = isEvicting;
	}
	/**
	 * @return Returns the nodeName.
	 */
	public String getNodeName()
	{
		return nodeName;
	}
	/**
	 * @return Returns the waitingTime.
	 */
	public long getWaitingTime()
	{
		return waitingTime;
	}
	/**
	 * @param waitingTime The waitingTime to set.
	 */
	public void setWaitingTime(long waitingTime)
	{
		this.waitingTime = waitingTime;
	}
	
	public int getCapabilityQueueSize(String capabilityPath)
	{
		long count = anchor.getWaitingQueuesStore().getQueueSize(nodeName, capabilityPath);
		count += anchor.getExecutingQueuesStore().getQueueSize(nodeName, capabilityPath);
		return (int) count;
	}
	
	public long getAgentsCount(String nodeName, String capabilityPath)
	{
		return anchor.getAgentsCount(nodeName, capabilityPath);
	}

	/**
	 * @return
	 */
	public boolean isAboveThreshold()
	{
		return workstationLoadImage.isAboveThreshold();
	}

	/**
	 * @return
	 */
	public int getNonJanetCPULoad()
	{
		return workstationLoadImage.getNonJanetCPULoad();
	}
	/**
	 * @return Returns the workstationLoadImage.
	 */
	public WorkstationLoadImage getWorkstationLoadImage()
	{
		return workstationLoadImage;
	}

	/**
	 * @param path
	 * @return
	 */
	public int getCapabilityQueueSize(LocalCapabilityPath path)
	{
		return workstationLoadImage.getCapabilityQueueSize(getNodeName(), path);
	}
	
	public static List getNodeNames(List nodeLoadImages)
	{
		List names = new ArrayList();
		Iterator it = nodeLoadImages.iterator();
		while (it.hasNext())
		{
			NodeLoadImage nodeLoadImage = (NodeLoadImage) it.next();
			names.add(nodeLoadImage.getNodeName());
		}
		return names;
	}

	/**
	 * @param i
	 */
	public void setMinQueueSizeCategory(int qsc)
	{
		if(getQueueSizeCategory() < qsc)
			setQueueSizeCategory(qsc);
	}

	/**
	 * @return
	 */
	public boolean isBelowTreshold()
	{
		return !isAboveThreshold();
	}
}
