/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.ade.distributor;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.almendra.commons.util.AssertUtility;
import org.almendra.commons.util.collection.ListUtility;
import org.almendra.janet.cas.node.LocalCapabilityPath;



public class WorkstationLoadImage extends DistributorAnchorHolder
{
	protected String workstationName = null;
	protected Map nodeLoadImages = new Hashtable();
	protected volatile boolean isAboveThreshold = false;
	protected volatile int nonJanetCPULoad = 0;
	
	/**
	 * Creates a new <code>WorkstationLoadImage</code> object.
	 */
	protected WorkstationLoadImage()
	{
		super();
	}
	
	/**
	 * Creates a new <code>WorkstationLoadImage</code> object.
	 */
	public WorkstationLoadImage(DistributorAnchor anchor, String workstationName)
	{
		super(anchor);
		this.workstationName = workstationName;
	}
	
	public void addNodeLoadImage(NodeLoadImage nodeLoadImage)
	{
		AssertUtility.assertFalse(nodeLoadImages.containsKey(nodeLoadImage.getNodeName()));
		nodeLoadImages.put(nodeLoadImage.getNodeName(), nodeLoadImage);
	}
	
	public NodeLoadImage removeNodeLoadImage(String nodeName)
	{
		NodeLoadImage nodeLoadImage = (NodeLoadImage) nodeLoadImages.remove(nodeName);
		if(!hasNodes())
			anchor.deregisterWorkstation(getWorkstationName(), new ArrayList());
		return nodeLoadImage;
	}
	
	public void addNodeLoadImage(String nodeName)
	{
		NodeLoadImage nodeLoadImage = new NodeLoadImage(anchor, this, nodeName);	
		anchor.workstationLoadImageByNodeName.put(nodeName, this);
		addNodeLoadImage(nodeLoadImage);
	}
	
	public NodeLoadImage getNodeLoadImage(String nodeName)
	{
		NodeLoadImage nodeLoadImage = (NodeLoadImage) nodeLoadImages.get(nodeName);
		if(nodeLoadImage == null)
		{
			nodeLoadImage = new NodeLoadImage(anchor, this, nodeName);
			addNodeLoadImage(nodeLoadImage);
		}
		return nodeLoadImage;
	}
	
	/**
	 * @return Returns the workstationName.
	 */
	public String getWorkstationName()
	{
		return workstationName;
	}

	/**
	 * @return
	 */
	public int getProduct()
	{
		int product = 1;
		Iterator it = nodeLoadImages.values().iterator();
		while (it.hasNext())
		{
			NodeLoadImage nodeLoadImage = (NodeLoadImage) it.next();
			int nodeQueueSizeCategory = nodeLoadImage.getQueueSizeCategory();
			product *= nodeQueueSizeCategory;	
		}
		return product;
	}
	
	public List getNodeLoadImages()
	{
		return ListUtility.toList(nodeLoadImages.values());
	}
	
	public List getNodeNames()
	{
		List nodeNames = new ArrayList();
		Iterator it = getNodeLoadImages().iterator();
		while (it.hasNext())
		{
			NodeLoadImage image = (NodeLoadImage) it.next();
			nodeNames.add(image.getNodeName());
		}
		return nodeNames;
	}
	/**
	 * @return Returns the isAboveThreshold.
	 */
	public boolean isAboveThreshold()
	{
		return isAboveThreshold;
	}
	/**
	 * @param isAboveThreshold The isAboveThreshold to set.
	 */
	public void setAboveThreshold(boolean isAboveThreshold)
	{
		this.isAboveThreshold = isAboveThreshold;
	}

	public int getQSCProduct()
	{
		int product = 0;
		List nodeLoadImages = getNodeLoadImages(); 
		if(nodeLoadImages.isEmpty())
			return product;
		product = 1;
		boolean allZero = true;
		Iterator it = nodeLoadImages.iterator();
		while (it.hasNext())
		{
			NodeLoadImage image = (NodeLoadImage) it.next();
			int qsc = image.getQueueSizeCategory();
			if(qsc != 0)
			{
				allZero = false;
				product *= qsc;
			}
		}
		if(allZero)
			return 0;
		return product;
	}
	
	protected void removeAllNodes()
	{
		Iterator it = getNodeNames().iterator();
		while(it.hasNext())
		{
			String nodeName = (String) it.next();
			anchor.workstationLoadImageByNodeName.remove(nodeName);
		}
	}

	/**
	 * @return
	 */
	public boolean hasNodes()
	{
		return !getNodeNames().isEmpty();
	}

	/**
	 * @param nodeNames
	 */
	public void removeNodeLoadImagesNamed(List nodeNames)
	{
		Iterator it = getNodeNames().iterator();
		while(it.hasNext())
		{
			String nodeName = (String) it.next();
			if(nodeNames.contains(nodeName))
				removeNodeLoadImage(nodeName);
		}
	}

	/**
	 * @param string
	 * @param path
	 * @return
	 */
	public int getCapabilityQueueSize(String nodeName, LocalCapabilityPath path)
	{
		return anchor.getCapabilityQueueSize(nodeName, path);
	}

	/**
	 * @return
	 */
	public boolean isIdle()
	{
		return getQSCProduct() == 0;
	}

	/**
	 * @return
	 */
	public List getAvailableNodeNamesWithQSC0()
	{
		List nodeNamesWithQSC0 = new ArrayList();
		Iterator it = nodeLoadImages.values().iterator();
		while (it.hasNext())
		{
			NodeLoadImage nodeLoadImage = (NodeLoadImage) it.next();
			if(nodeLoadImage.isBelowTreshold() && nodeLoadImage.getQueueSizeCategory() == 0)
				nodeNamesWithQSC0.add(nodeLoadImage.getNodeName());
		}
		return nodeNamesWithQSC0;
	}

	/**
	 * @param nonJanetCPULoad
	 */
	public void setNonJanetCPULoad(int nonJanetCPULoad)
	{
		this.nonJanetCPULoad = nonJanetCPULoad;
	}
	/**
	 * @return Returns the nonJanetCPULoad.
	 */
	public int getNonJanetCPULoad()
	{
		return nonJanetCPULoad;
	}

	/**
	 * @return
	 */
	public boolean isBelowThreshold()
	{
		return !isAboveThreshold();
	}
}
