/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.ade.distributor.interpreters;

import java.util.Iterator;
import java.util.List;

import org.almendra.commons.util.collection.StringVector;
import org.almendra.janet.ade.commands.CancelReduceLoadCommand;
import org.almendra.janet.ade.commands.NotifyCPULoadChangedCommand;
import org.almendra.janet.ade.commands.ProcessCPULoadChangedCommand;
import org.almendra.janet.ade.distributor.DistributorAnchor;
import org.almendra.janet.ade.distributor.NodeLoadImage;
import org.almendra.janet.ade.distributor.WorkstationLoadImage;
import org.almendra.janet.cas.agents.MultiAgentProxy;
import org.almendra.janet.cas.scheduling.CommandAccessor;
import org.almendra.janet.cas.scheduling.IInterpreter;



public class NotifyCPULoadChangedInterpreter extends NotifyValueChangedInterpreter implements IInterpreter
{
	/**
	 * Creates a new <code>NotifyCPULoadChangedInterpreter</code> object.
	 */
	public NotifyCPULoadChangedInterpreter()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#execute(org.almendra.commons.janet.cas.scheduling.CommandAccessor)
	 */
	public void execute(CommandAccessor cmdAccessor)
	{
		if(cmdAccessor.getCommand() instanceof NotifyCPULoadChangedCommand)
		{
			NotifyCPULoadChangedCommand command = (NotifyCPULoadChangedCommand) cmdAccessor.getCommand();
			DistributorAnchor anchor = getDistributorAnchor(cmdAccessor);
			
			int change = command.getChange();	
			boolean isBeforeAboveThreshold = anchor.getWorkstationLoadImage(command.getWorkstationName()).isAboveThreshold();
			
			if(change == NotifyCPULoadChangedCommand.CPULoadAboveThreshold)
			{
				anchor.setIsAboveThreshold(command.getWorkstationName(), true, command.getCurrentNonJanetCPULoad());
				if(anchor.isDisableFullEviction())
					return;
				if(isBeforeAboveThreshold)
					return;
				evictCommands(cmdAccessor, anchor, command.getWorkstationName());
				return;
			}
			
			if(change == NotifyCPULoadChangedCommand.CPULoadBelowThreshold)
			{
				anchor.setIsAboveThreshold(command.getWorkstationName(), false, command.getCurrentNonJanetCPULoad());
				if(anchor.isDisableFullEviction())
					return;
				if(!isBeforeAboveThreshold)
					return;
				cancelEviction(cmdAccessor, anchor, command.getWorkstationName());
				checkNodeDroppedToQSC0(cmdAccessor, anchor, command.getWorkstationName());
				return;
			}
			
			if(change == NotifyCPULoadChangedCommand.CPULoadSignificantChange)
			{
				anchor.setNonJanetCPULoad(command.getWorkstationName(), command.getCurrentNonJanetCPULoad());
				return;
			}
		}
	}

	/**
	 * @param cmdAccessor
	 * @param anchor
	 * @param string
	 */
	protected void checkNodeDroppedToQSC0(CommandAccessor cmdAccessor, DistributorAnchor anchor, String workstationName)
	{
		WorkstationLoadImage workstationLoadImage = anchor.getWorkstationLoadImage(workstationName);
		if(workstationLoadImage.isAboveThreshold())
			return;
		Iterator it = workstationLoadImage.getNodeLoadImages().iterator();
		while (it.hasNext())
		{
			NodeLoadImage nodeLoadImage = (NodeLoadImage) it.next();
			if(nodeLoadImage.getQueueSizeCategory() == 0)
				reflectNodeDroppedToQSC0(nodeLoadImage.getNodeName(), cmdAccessor, anchor);
		}
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#commandNames()
	 */
	public StringVector commandNames()
	{
		return new StringVector(NotifyCPULoadChangedCommand.QualifiedName);
	}

	/**
	 * @param anchor
	 * @param workstationName
	 */
	protected void cancelEviction(CommandAccessor cmdAccessor, DistributorAnchor anchor, String workstationName)
	{
		try
		{
			List nodeNames = anchor.getWorkstationLoadImage(workstationName).getNodeNames();
			CancelReduceLoadCommand command = new CancelReduceLoadCommand(); 
			MultiAgentProxy nodeSystemAgents = anchor.getSystemApplication().getAllSystemAgentsOnNodes(nodeNames);
			nodeSystemAgents.accept(command);
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			cmdAccessor.getLogger().exception(e);
		}		
	}

	/**
	 * @param anchor
	 * @param workstationName
	 */
	protected void evictCommands(CommandAccessor cmdAccessor, DistributorAnchor anchor, String workstationName)
	{				
		try
		{
			ProcessCPULoadChangedCommand command = new ProcessCPULoadChangedCommand(workstationName, true); 
			anchor.getBalancerAgent().accept(command);
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			cmdAccessor.getLogger().exception(e);
		}
	}
}
