/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.ade.distributor.interpreters;

import java.util.ArrayList;
import java.util.List;

import org.almendra.commons.util.collection.ListUtility;
import org.almendra.commons.util.collection.StringVector;
import org.almendra.commons.util.swing.TextAreaPanelController;
import org.almendra.janet.ade.commands.ExecuteWorkloadCommand;
import org.almendra.janet.ade.commands.WorkloadDistributionCommand;
import org.almendra.janet.ade.distributor.DistributorAnchor;
import org.almendra.janet.ade.distributor.LeastLoadedNodeDecisionInformation;
import org.almendra.janet.ade.distributor.NodeLoadImage;
import org.almendra.janet.cas.agents.AgentProxy;
import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.node.LocalCapabilityPath;
import org.almendra.janet.cas.node.LogEntry;
import org.almendra.janet.cas.node.Logger;
import org.almendra.janet.cas.scheduling.CommandAccessor;
import org.almendra.janet.cas.scheduling.CommandEnvelope;
import org.almendra.janet.cas.scheduling.IInterpreter;



public class WorkloadDistributionInterpreter extends DistributorInterpreter implements IInterpreter
{
	/**
	 * Creates a new <code>WorkloadDistributionInterpreter</code> object.
	 */
	public WorkloadDistributionInterpreter()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#execute(org.almendra.commons.janet.cas.scheduling.CommandAccessor)
	 */
	public void execute(CommandAccessor cmdAccessor)
	{
		if(cmdAccessor.getCommand() instanceof WorkloadDistributionCommand)
		{
			WorkloadDistributionCommand distributionCommand = (WorkloadDistributionCommand) cmdAccessor.getCommand();
			ExecuteWorkloadCommand executCommand = distributionCommand.getCommand();
			DistributorAnchor anchor = getDistributorAnchor(cmdAccessor);
			TextAreaPanelController freeText = cmdAccessor.getNodeAccessor().getFreeTextArea();
			LeastLoadedNodeDecisionInformation decisionInformation = new LeastLoadedNodeDecisionInformation(distributionCommand.getAgentPath(), executCommand.getCommand().getClass().getName());
			AgentPath selectedAgentPath = getCapabilityWithShortestQueueAtLeastOne(anchor, distributionCommand.getAgentPath(), decisionInformation);
			if(selectedAgentPath == null)
			{
				logPanicCannotFindAnyAgent(cmdAccessor);
				return;
			}
						
			CommandEnvelope env = cmdAccessor.getCommandEnvelopeIfPrivilege();
			env.setCommand(executCommand);
			env.setPredefinedInterpreter(null);

			try
			{
				AgentProxy leastLoadedAgent = env.getCrossAgent(selectedAgentPath);				
				env.setNewDestination(selectedAgentPath, cmdAccessor.getNodeName());
				decisionInformation.printOn(freeText);
				executCommand.setMustAccept(true);
				leastLoadedAgent.accept(env);
				anchor.logCommandDistributed(executCommand, env.getOriginAgentPath(), selectedAgentPath);
			}
			catch (Exception e)
			{
				System.out.println(e.getMessage());
				e.printStackTrace();
				cmdAccessor.getLogger().exception(e);
			}	
		}
	}

	/**
	 * @param logger
	 * @param path
	 */
	protected void logPanicCannotFindAnyAgent(CommandAccessor cmdAccessor)
	{
		// TODO - clean up. Should not have to check this in that way
		Logger logger = cmdAccessor.getLogger();
		if(!logger.isRegistered(DistributorAnchor.LOG_EVENT_PANIC_CANNOT_FIND_ANY_AGENT))
		{
			logger.registerForEvents(registeredLogEvents());
		}
		if(!logger.isSubscribed(DistributorAnchor.LOG_EVENT_PANIC_CANNOT_FIND_ANY_AGENT))
		{
			logger.subscribeForEvents(registeredLogEvents());
		}
		
		LogEntry entry = (LogEntry) logger.getNewLogEntry();
		entry.setEvent(DistributorAnchor.LOG_EVENT_PANIC_CANNOT_FIND_ANY_AGENT);
		entry.setAgentName(cmdAccessor.getCurrentAgent().getName());
		StringBuffer buffer = new StringBuffer();
		buffer.append("no agent found in cluster that accepts command " + cmdAccessor.getCommand().getClass().getName()); //$NON-NLS-1$
		buffer.append(" sent from agent " + cmdAccessor.getCommandEnvelopeIfPrivilege().getOriginAgentPath()); //$NON-NLS-1$
		entry.setArgument(buffer.toString());
		logger.fatal(entry);
	}

	/**
	 * @param agentPath
	 * @return
	 */
	protected AgentPath getCapabilityWithShortestQueueAtLeastOne(DistributorAnchor anchor, AgentPath originAgentPath, LeastLoadedNodeDecisionInformation decisionInformation)
	{				
		LocalCapabilityPath path = new LocalCapabilityPath(originAgentPath.getApplicationName(), originAgentPath.getCapabilityName()); 
		List nodeNames = anchor.getAllNodesWithCapability(path, anchor.getAllConnectedNodeNames());
		List leastLoadedNodesForCapability = anchor.getLeastLoadedCapabilitiesAtLeastOne(nodeNames, path, decisionInformation);
		if(leastLoadedNodesForCapability == null || leastLoadedNodesForCapability.isEmpty())
			return originAgentPath;
		List leastLoadedNodeNamesForCapability = NodeLoadImage.getNodeNames(leastLoadedNodesForCapability);
		return anchor.getAgentPathToNode(getLeastLoadedNodeName(anchor, leastLoadedNodeNamesForCapability, decisionInformation), originAgentPath);
	}

	/**
	 * @param anchor
	 * @param leastLoadedNodesForCapability
	 */
	protected String getLeastLoadedNodeName(DistributorAnchor anchor, List leastLoadedNodesForCapability, LeastLoadedNodeDecisionInformation decisionInformation)
	{
		if(leastLoadedNodesForCapability.size() > 1)
		{
			String lastSelectNodeName = anchor.getLastSelectNodeName();
			if(lastSelectNodeName == null)
			{
				String selectedNodeName = (String) ListUtility.pickAtRandom(leastLoadedNodesForCapability);
				decisionInformation.setNodeNameLeastLoaded(selectedNodeName);
				return setSelectNodeName(anchor, selectedNodeName);
			}
			else
			{				
				int index = leastLoadedNodesForCapability.indexOf(lastSelectNodeName);
				if(index == -1)
				{
					String selectedNodeName = (String) ListUtility.pickAtRandom(leastLoadedNodesForCapability);
					decisionInformation.setNodeNameLeastLoaded(selectedNodeName);
					return setSelectNodeName(anchor, selectedNodeName);
				}
				decisionInformation.setLastSelectedNode(lastSelectNodeName);
				if(index == leastLoadedNodesForCapability.size() - 1)
					index = 0;
				else
					index++;
				String selectedNodeName = (String) leastLoadedNodesForCapability.get(index);
				decisionInformation.setNodeNameLeastLoaded(selectedNodeName);
				return setSelectNodeName(anchor, selectedNodeName);
			}
		}
		String selectedNodeName = (String) leastLoadedNodesForCapability.get(0);
		decisionInformation.setNodeNameLeastLoaded(selectedNodeName);
		return setSelectNodeName(anchor, selectedNodeName);
	}

	/**
	 * @param anchor
	 * @param leastLoadedNodesForCapability
	 * @return
	 */
	protected String setSelectNodeName(DistributorAnchor anchor, String selectedNodeName)
	{		
		anchor.setLastSelectNodeName(selectedNodeName);
		return selectedNodeName;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#commandNames()
	 */
	public StringVector commandNames()
	{
		return new StringVector(WorkloadDistributionCommand.QualifiedName);
	}
	
	public List registeredLogEvents()
	{
		List events = new ArrayList();
		events.add(DistributorAnchor.LOG_EVENT_PANIC_CANNOT_FIND_ANY_AGENT);
		return events;
	}
}
