/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.ade.executor.interpreters;

import org.almendra.commons.Result;
import org.almendra.commons.reflect.ReflectionUtility;
import org.almendra.commons.util.collection.StringVector;
import org.almendra.janet.ade.commands.ExecuteWorkloadBalancingCommand;
import org.almendra.janet.ade.executor.ExecutorAnchor;
import org.almendra.janet.cas.scheduling.CommandAccessor;
import org.almendra.janet.cas.scheduling.IInterpreter;
import org.almendra.janet.cas.scheduling.IWorkLoadBalancingCommand;
import org.almendra.janet.cas.scheduling.IWorkLoadBalancingInterpreter;



public class ExecuteWorkloadBalancingInterpreter extends ExecutorInterpreter implements IInterpreter
{
	protected IWorkLoadBalancingInterpreter interpreter = null;
	
	/**
	 * Creates a new <code>ExecuteWorkloadBalancingInterpreter</code> object.
	 */
	public ExecuteWorkloadBalancingInterpreter()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#execute(org.almendra.commons.janet.cas.scheduling.CommandAccessor)
	 */
	public void execute(CommandAccessor cmdAccessor)
	{
		if(cmdAccessor.getCommand() instanceof ExecuteWorkloadBalancingCommand)
		{
			ExecuteWorkloadBalancingCommand cmd = (ExecuteWorkloadBalancingCommand) cmdAccessor.getCommand();
			IWorkLoadBalancingCommand workloadCommand = cmd.getBalancingCommand();
			IWorkLoadBalancingInterpreter interpreter = getInterpreter(cmdAccessor, workloadCommand);
			try
			{
				interpreter = (IWorkLoadBalancingInterpreter) interpreter.clone();
			}
			catch (CloneNotSupportedException e)
			{
				System.out.println(e.getMessage());
				e.printStackTrace();
				// TODO - continue as non-balancing command, display error, object must implement Cloneable interface
			}
			if(interpreter == null)
			{
				// TODO - display error
				return;
			}
			cmd.setSuspendableInterpreter(interpreter);
			CommandAccessor workloadCommandAccessor = new CommandAccessor(cmdAccessor, workloadCommand);
			String capabilityPath = cmd.getQualifiedCapabilityPath();
			ExecutorAnchor anchor = cmd.getAnchor();
			anchor.workloadBalancingCommandStarted(cmd.getCurrentEnvelope(), capabilityPath);
			interpreter.execute(workloadCommandAccessor);
			if(!cmdAccessor.isSuspended())
				anchor.workloadBalancingCommandFinished(cmd.getCurrentEnvelope(), capabilityPath);			
		}
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#commandNames()
	 */
	public StringVector commandNames()
	{
		return new StringVector(ExecuteWorkloadBalancingCommand.QualifiedName);
	}
	
	/**
	 * @param cmdAccessor
	 * @param workloadSharingCmd
	 * @return
	 */
	protected IWorkLoadBalancingInterpreter getInterpreter(CommandAccessor cmdAccessor, IWorkLoadBalancingCommand cmd)
	{
		interpreter = (IWorkLoadBalancingInterpreter) cmdAccessor.getCapability().getInterpreter(cmd.getQualifiedName());
		if(interpreter == null)
		{
			// TODO - display error
		}
		return interpreter;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.ade.executor.interpreters.ExecuteWorkloadInterpreter#suspend()
	 */
	public Result suspend()
	{		
		return ReflectionUtility.invoke(interpreter, "suspend"); //$NON-NLS-1$
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.ade.executor.interpreters.ExecuteWorkloadInterpreter#resume()
	 */
	public Result resume()
	{
		return ReflectionUtility.invoke(interpreter, "resume"); //$NON-NLS-1$
	}
	

}
