/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.ade.executor.interpreters;

import java.util.List;

import org.almendra.commons.util.collection.StringVector;
import org.almendra.janet.ade.commands.InterruptibleReduceLoadCommand;
import org.almendra.janet.ade.distributor.PartialEvictionAssignment;
import org.almendra.janet.ade.executor.ExecutorAnchor;
import org.almendra.janet.ade.executor.executors.EvictExecuteCommandsExecutor;
import org.almendra.janet.ade.executor.executors.EvictExecuteCommandsFullyExecutor;
import org.almendra.janet.ade.executor.executors.EvictWaitCommandsExecutor;
import org.almendra.janet.ade.executor.executors.EvictWaitCommandsFullyExecutor;
import org.almendra.janet.cas.node.SystemApplication;
import org.almendra.janet.cas.scheduling.CommandAccessor;
import org.almendra.janet.cas.scheduling.IInterpreter;



public class InterruptibleReduceLoadInterpreter extends ExecutorInterpreter implements IInterpreter
{
	/**
	 * Creates a new <code>InterruptibleReduceLoadInterpreter</code> object.
	 */
	public InterruptibleReduceLoadInterpreter()
	{
		super();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#execute(org.almendra.commons.janet.cas.scheduling.CommandAccessor)
	 */
	public void execute(CommandAccessor cmdAccessor)
	{
		if(cmdAccessor.getCommand() instanceof InterruptibleReduceLoadCommand)
		{
			InterruptibleReduceLoadCommand command = (InterruptibleReduceLoadCommand) cmdAccessor.getCommand();
			ExecutorAnchor anchor = getExecutorAnchor(cmdAccessor);
			anchor.setEvictionInProcess(true);
			evictCommands(cmdAccessor, anchor, command);
			anchor.setEvictionInProcess(false);
			anchor.checkQueueSizeCategories();
		}
	}
	
	/**
	 * @param cmdAccessor
	 * @param command
	 */
	protected void evictCommands(CommandAccessor cmdAccessor, ExecutorAnchor anchor, InterruptibleReduceLoadCommand command)
	{
		if(command.isFullEviction())
			evictFully(cmdAccessor, anchor, command.getToken(), command.getEvictionAssignments());
		else
			evict1Command(cmdAccessor, anchor, command.getToken(), command.getEvictionAssignments());
	}

	/**
	 * @param cmdAccessor
	 * @param anchor
	 * @param string
	 * @param list
	 */
	protected void evict1Command(CommandAccessor cmdAccessor, ExecutorAnchor anchor, String token, List evictionAssignments)
	{
		if(!anchor.existsToken(token))
			return;
		
		PartialEvictionAssignment assignment = (PartialEvictionAssignment) evictionAssignments.get(0);
		
		SystemApplication sysApp = cmdAccessor.getSystemApplicationIfPrivilege();
		EvictWaitCommandsExecutor evictWaitExecutor = new EvictWaitCommandsExecutor(
			sysApp, assignment.getDesintationAgent(), assignment.getCapabilityPath(), token);
		anchor.writeProtectedDo(evictWaitExecutor);
		if(evictWaitExecutor.getEvictedCommandsCount() == 1)
			return;
		if(!anchor.existsToken(token))
			return;		
		EvictExecuteCommandsExecutor evictExecuteExecutor = new EvictExecuteCommandsExecutor(
			sysApp, assignment.getDesintationAgent(), assignment.getCapabilityPath(), token);
		anchor.writeProtectedDo(evictExecuteExecutor);
	}

	/**
	 * @param cmdAccessor
	 */
	protected boolean evictFully(CommandAccessor cmdAccessor, ExecutorAnchor anchor, String token, List fullEvictionAssignments)
	{
		boolean interrupted = evictWaitCommandsFully(anchor, token, fullEvictionAssignments);
		if(interrupted)
		{
			anchor.logInterruptedReduceLoadCommand(cmdAccessor.getCommandId());
			return true;
		}
		interrupted = evictExecuteCommandsFully(anchor, token, fullEvictionAssignments);
		if(interrupted)
		{
			anchor.logInterruptedReduceLoadCommand(cmdAccessor.getCommandId());
			return true;
		}	
		return false;
	}

	/**
	 * @param anchor
	 * @return
	 */
	protected boolean evictExecuteCommandsFully(ExecutorAnchor anchor, String token, List fullEvictionAssignments)
	{
		if(!anchor.existsToken(token))
			return true;
		EvictExecuteCommandsFullyExecutor executor = new EvictExecuteCommandsFullyExecutor(token, fullEvictionAssignments);
		anchor.writeProtectedDo(executor);
		return executor.isInterrupted();
	}

	/**
	 * @param anchor
	 * @return
	 */
	protected boolean evictWaitCommandsFully(ExecutorAnchor anchor, String token, List fullEvictionAssignments)
	{
		if(!anchor.existsToken(token))
			return true;
		EvictWaitCommandsFullyExecutor executor = new EvictWaitCommandsFullyExecutor(token, fullEvictionAssignments);
		anchor.writeProtectedDo(executor);
		return executor.isInterrupted();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#commandNames()
	 */
	public StringVector commandNames()
	{
		return new StringVector(InterruptibleReduceLoadCommand.QualifiedName);
	}

}
