/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.demo.fibonacci;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

import org.almendra.commons.IHistorySummary;
import org.almendra.commons.events.IHistorizableCallbackHandler;
import org.almendra.commons.util.AssertUtility;
import org.almendra.commons.util.logging.LoggerAccessor;
import org.almendra.commons.util.swing.*;
import org.almendra.janet.cas.scheduling.CommandHistorySummary;


/**
 * @author Oliver
 *
 */
public class FibonacciCallbackHandler implements IHistorizableCallbackHandler
{
	protected LoggerAccessor logAccessor = null;
	protected List historyItems = new ArrayList();
	protected HistoryTotalsCreator totalsCreator = null;
	protected JTextPaneHolder textPaneHolder = null;
	protected int nthFibonacciNumber = 0; 
	
	/**
	 * Creates a new <code>FibonacciCallbackHandler</code> object.
	 */
	public FibonacciCallbackHandler(int nthFibonacciNumber, LoggerAccessor logAccessor, HistoryTotalsCreator totalsCreator, JTextPaneHolder textPaneHolderOrNull)
	{
		super();
		this.logAccessor = logAccessor;
		this.totalsCreator = totalsCreator;
		this.textPaneHolder = textPaneHolderOrNull;
		this.nthFibonacciNumber = nthFibonacciNumber;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.events.ICallbackHandler#handle(java.lang.Object)
	 */
	public void handle(Object callbackParameter)
	{
		CommandHistorySummary summary = new CommandHistorySummary();
		createHistorySummary(summary);
		
		BigInteger result = (BigInteger) callbackParameter;
		AssertUtility.assertFalse(result == null);
		FibonacciUtility.displayResult(nthFibonacciNumber, result, logAccessor, summary, textPaneHolder);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.IHistorizable#getHistoryItems()
	 */
	public List getHistoryItems()
	{
		return historyItems;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.IHistorizable#addHistoryItems(java.util.List)
	 */
	public void addHistoryItems(List items)
	{
		historyItems.addAll(items);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.IHistorizable#createHistorySummary(org.almendra.commons.IHistorySummary)
	 */
	public void createHistorySummary(IHistorySummary historySummary)
	{
		historySummary.create(getHistoryItems());
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.events.IHistorizableCallbackHandler#handle(java.lang.Object, java.util.List)
	 */
	public void handle(Object parameter, List historyItems)
	{
		addHistoryItems(historyItems);		
		handle(parameter);
		totalsCreator.addHistoryItems(historyItems, textPaneHolder);
	}

}
