package org.almendra.janet.cas.demo.fibonacci;

import java.rmi.RemoteException;

import javax.swing.JOptionPane;

import org.almendra.commons.util.logging.LoggerAccessor;
import org.almendra.commons.util.swing.*;
import org.almendra.janet.cas.agents.IAgentProxy;
import org.almendra.janet.cas.node.ApplicationAccessor;
import org.almendra.janet.cas.node.InvalidCommandException;
import org.almendra.janet.cas.node.LocalAgentPath;
import org.almendra.janet.cas.node.NodeAccessor;


/**
 * Insert the type's description here. Creation date: (17.03.2004 10:01:59)
 * @author: Administrator
 */
public class FibonacciDemoViewController
{
	public static final int MAX_FIBO_NUMBER = 50000;
	public static final String LABEL = "CAS.Fibaonacci Sample"; //$NON-NLS-1$
	
	protected FibonacciDemoView view = null;
	protected ApplicationAccessor app = null;
	protected LocalAgentPath agentPath = null;
	protected JTextPaneHolder textPaneHolder = null;

	/**
	 * FibonacciDemoViewController constructor comment.
	 */
	protected FibonacciDemoViewController(FibonacciDemoView view, ApplicationAccessor app, LocalAgentPath agentPath)
	{
		super();
		this.view = view;
		this.app = app;
		this.agentPath = agentPath;
	}

	protected void shutdown()
	{
		getTextPaneHolder().clear();
	}

	/**
	 * @param fibonacciNumber
	 */
	public void calculate(int nthFibonacciNumber)
	{
		if(nthFibonacciNumber > MAX_FIBO_NUMBER)
		{
			JOptionPane.showMessageDialog(null, "Input value too large (max: 50000)!"); //$NON-NLS-1$
			return;
		}
		
		getTextPaneHolder().clearText();
		NodeAccessor node = app.getNodeAccessor();
		IAgentProxy agents = app.getAllAgents(agentPath);
		
		long startTime = System.currentTimeMillis();
		LoggerAccessor logger = node.getLogAccessor();
		HistoryTotalsCreator totalsCreator = new HistoryTotalsCreator(agents.size(), nthFibonacciNumber, logger, startTime);	
				
		try
		{
			FibonacciCallbackHandler handler = new FibonacciCallbackHandler(nthFibonacciNumber, logger, totalsCreator, getTextPaneHolder());
			agents.accept(new FibonacciCommand(nthFibonacciNumber), handler);
		} 
		catch (RemoteException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			app.getLogger().exception(e);
		} 
		catch (InvalidCommandException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			app.getLogger().exception(e);
		}		
	}

	/**
	 * 
	 */
	public void close()
	{
		view.dispose();
	}

	/**
	 * @return
	 */
	public JTextPaneHolder getTextPaneHolder()
	{
		if(textPaneHolder == null)
		{
			textPaneHolder = new JTextPaneHolder(view.getJTextPane()); 
		}
		return textPaneHolder;
	}
}
