/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.demo.fibonacci;

import org.almendra.commons.events.ICallbackHandler;
import org.almendra.commons.events.IEventHandler;
import org.almendra.commons.util.swing.JTextPaneHolder;
import org.almendra.janet.cas.events.EventHandlerEnvelope;
import org.almendra.janet.cas.events.HandlerList;
import org.almendra.janet.cas.events.ILocalEventRegistry;
import org.almendra.janet.cas.handlers.NodeAboutToShutdownParams;
import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.node.ApplicationAccessor;
import org.almendra.janet.cas.node.LocalAgentPath;
import org.almendra.janet.cas.scheduling.CommandAccessor;


/**
 * @author Oliver Plohmann
 *
 */
public class OpenFibonacciViewHandler implements ICallbackHandler
{
	protected CommandAccessor cmdAccessor = null;
	protected ApplicationAccessor application = null;
	protected LocalAgentPath localAgentPath = null; 
	protected AgentPath agentPath = null;
	protected ILocalEventRegistry eventRegistry = null;
	
	/**
	 * Creates a new <code>OpenFibonacciViewHandler</code> object.
	 */
	protected OpenFibonacciViewHandler()
	{
		super();
	}

	/**
	  * Creates a new <code>OpenFibonacciViewHandler</code> object.
	  */
	public OpenFibonacciViewHandler(CommandAccessor cmdAccessor)
	{
		super();
		this.application = cmdAccessor.getApplication();
		this.localAgentPath = cmdAccessor.getLocalAgentPath();
		this.eventRegistry = cmdAccessor.getNodeEventRegistry();
		this.agentPath = cmdAccessor.getAgentPath();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.events.ICallbackHandler#handle(java.lang.Object)
	 */
	public void handle(Object callbackParameter)
	{
		FibonacciDemoViewController viewController = FibonacciDemoView.open(application, localAgentPath);
		JTextPaneHolder textPaneHolder = viewController.getTextPaneHolder();
		
		IEventHandler handler = new NodeAboutToShutdownHandler(viewController, textPaneHolder);
		EventHandlerEnvelope env = new EventHandlerEnvelope(handler, agentPath); 
		HandlerList handlers = new HandlerList(env);
		eventRegistry.register(
			NodeAboutToShutdownParams.EVENT_NAME, 
			handlers,
			application);
	}
}
