/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.oliverplohmann.de/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.janet.cas.logger;

import java.rmi.RemoteException;
import java.util.LinkedList;
import java.util.List;

import org.almendra.commons.threads.ThreadUtility;
import org.almendra.commons.util.collection.ListUtility;
import org.almendra.commons.util.logging.LogEntry;
import org.almendra.janet.cas.agents.IAgentProxy;
import org.almendra.janet.cas.logger.commands.DisplayLogEntryCommand;
import org.almendra.janet.cas.node.InvalidCommandException;

import EDU.oswego.cs.dl.util.concurrent.ReentrantLock;
import EDU.oswego.cs.dl.util.concurrent.Semaphore;


/**
 * @author Oliver
 *  
 */
public class LogCollector implements Runnable
{
	protected static final long SLEEP_PERIOD = 2000; // ms
	
	protected IAgentProxy consumerAgent = null;
	protected ReentrantLock lock = new ReentrantLock();
	protected List buffer = initBuffer(); 	
	protected Semaphore gate = new Semaphore(0);
	protected volatile boolean isActive = false;

	/**
	 * Creates a new <code>LogCollector</code> object.
	 */
	protected LogCollector()
	{
		super();
	}

	/**
	 * Creates a new <code>LogCollector</code> object.
	 */
	public LogCollector(IAgentProxy consumerAgent)
	{
		super();
		this.consumerAgent = consumerAgent;
		start();
	}

	/**
	 * 
	 */
	public void start()
	{
		// does not work yet
		if(true)
			return;
		if(isActive)
			return;
		isActive = true;
		Thread collectorThread = new Thread(this);
		collectorThread.start();
	}
	
	protected void addAll(List entries)
	{
		try
		{
			lock.acquire();
			buffer.addAll(entries);
			gate.release();
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			lock.release();
		}
	}

	/**
	 * @param agent
	 * @param entry
	 */
	public void log(LogEntry entry)
	{		
		// addAll(ListUtility.toList(entry)); // unquote once working
		log(ListUtility.toList(entry));
	}

	/**
	 * @param agent
	 * @param entries
	 */
	public void log(List entries)
	{
		try
		{
			consumerAgent.accept(new DisplayLogEntryCommand(entries));
		}
		catch (RemoteException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		catch (InvalidCommandException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run()
	{		
		while(isActive)
		{
			forwardLogMsgs();
		}
	}
	
	/**
	 * 
	 */
	protected void forwardLogMsgs()
	{
		try
		{
			gate.acquire();
			ThreadUtility.sleep(SLEEP_PERIOD);
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			gate.release();
		}
		
		try
		{
			lock.acquire();
			// sendLogs();			
			resetGate();
			initBuffer();
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			lock.release();
		}
	}

	/**
	 * 
	 */
	protected List initBuffer()
	{
		buffer = new LinkedList();
		return buffer;
	}

	public void stop()
	{
		isActive = false;
	}
	
	protected void resetGate()
	{		
		try
		{
			long loops = gate.permits();
			for (long i = 0; i < loops; i++)
				gate.acquire();
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			gate = new Semaphore(0);
		}
	}

}
