/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.logger.gui;

import java.util.Arrays;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.event.ListSelectionEvent;

import org.almendra.janet.cas.agents.IAgentProxy;
import org.almendra.janet.cas.logger.LoggerAnchor;
import org.almendra.janet.cas.logger.commands.UnInstallLoggerCommand;
import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.node.ApplicationAccessor;
import org.almendra.janet.cas.node.LogEntry;
import org.almendra.janet.cas.node.NoSuchAgentException;
import org.almendra.janet.cas.node.NodeAccessor;
import org.almendra.janet.cas.scheduling.Acknowledge;


/**
 * @author Oliver
 *
 */
public class LoggerViewController
{
	protected LoggerView view = null;
	protected NodeAccessor node = null;
	protected ApplicationAccessor application = null; 
	protected int[] logListSelectionIndices = new int[0];
	protected Map logEntriesByLogText = new Hashtable();
	
	/**
	 * Creates a new <code>LoggerViewController</code> object.
	 */
	protected LoggerViewController()
	{
		super();
	}
	
	/**
	 * Creates a new <code>LoggerViewController</code> object.
	 */
	public LoggerViewController(LoggerView view)
	{
		super();
		this.view = view;
	}

	/**
	 * @param listSelectionEvent
	 */
	public void logEntriesList_ValueChanged(ListSelectionEvent listSelectionEvent)
	{
		JList logList = (JList) listSelectionEvent.getSource();
		int[] currentLogListSelectionIndices = logList.getSelectedIndices();
		if(Arrays.equals(logListSelectionIndices, currentLogListSelectionIndices))
			return;
		logListSelectionIndices = currentLogListSelectionIndices;
		view.getLogEntryTextPane().setText(""); //$NON-NLS-1$
		Object[] values = logList.getSelectedValues();
		if(values == null || values.length == 0)			
			return;
		StringBuffer expandedXMLString = new StringBuffer();
		Iterator it = Arrays.asList(values).iterator();
		while (it.hasNext())
		{
			String entryString = (String) it.next();
			org.almendra.janet.cas.node.LogEntry entry = (org.almendra.janet.cas.node.LogEntry) logEntriesByLogText.get(entryString);
			expandedXMLString.append(entry.toExpandedXMLString());
			if(it.hasNext())
				expandedXMLString.append("\n"); //$NON-NLS-1$
		}
		view.getLogEntryTextPane().setText(expandedXMLString.toString());
	}

	/**
	 * @param list
	 */
	public void log(List entries)
	{
		DefaultListModel model = (DefaultListModel) view.getLogEntriesList().getModel();
		Iterator it = entries.iterator();
		while (it.hasNext())
		{
			LogEntry entry = (LogEntry) it.next();
			logEntriesByLogText.put(entry.getLogMessage(), entry);
			model.insertElementAt(entry.getLogMessage(), 0);
			view.getStatusMsg2().setText("entries: " + logEntriesByLogText.size());
		}
	}

	/**
	 * @param node
	 */
	public void setNode(NodeAccessor node)
	{
		this.node = node;
	}

	/**
	 * 
	 */
	public void shutdown()
	{
		notifyNodes();
		node.shutdown();
	}

	/**
	 * 
	 */
	protected void notifyNodes()
	{
		IAgentProxy agents = application.getAllAgentsWithCapability(LoggerAnchor.CAPABILITY_PRODUCER_NAME, LoggerAnchor.AGENT_NAME);
		AgentPath ackAgentPath = null;
		try
		{
			ackAgentPath = application.getLocalAgentPath(LoggerAnchor.CAPABILITY_CONSUMER_NAME, LoggerAnchor.AGENT_NAME);
		}
		catch (NoSuchAgentException e)
		{
			// logical error if this happens - do not handle
			System.out.println(e.getMessage());
			e.printStackTrace();
			application.getLogger().exception(e);
		}
		Acknowledge ack = new Acknowledge(ackAgentPath);
		UnInstallLoggerCommand cmd = new UnInstallLoggerCommand(ackAgentPath);
		try
		{
			agents.accept(cmd, ack);
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			application.getLogger().exception(e);
		}
	}

	/**
	 * @param application
	 */
	public void setApplication(ApplicationAccessor application)
	{
		this.application = application;
	}

}
