/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.logger.handlers;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.almendra.commons.events.Event;
import org.almendra.commons.events.IEventHandler;
import org.almendra.janet.cas.agents.IAgent;
import org.almendra.janet.cas.agents.IAgentProxy;
import org.almendra.janet.cas.handlers.ForeignApplicationRegisteredParams;
import org.almendra.janet.cas.logger.LoggerAnchor;
import org.almendra.janet.cas.logger.commands.InstallLoggerCommand;
import org.almendra.janet.cas.logger.gui.LoggerViewController;
import org.almendra.janet.cas.node.AgentPath;


/**
 * @author Oliver
 *
 */
public class ForeignApplicationRegisteredHandler implements IEventHandler
{
	protected LoggerViewController viewController = null;
	protected IAgent currentAgent = null;
	
	/**
	 * Creates a new <code>ForeignApplicationRegisteredHandler</code> object.
	 */
	public ForeignApplicationRegisteredHandler(LoggerViewController viewController, IAgent currentAgent)
	{
		super();
		this.viewController = viewController;
		this.currentAgent = currentAgent;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.events.IEventHandler#occuredEvent(org.almendra.commons.commons.events.Event)
	 */
	public void occured(Event event)
	{
		if(event.getParams() instanceof ForeignApplicationRegisteredParams)
		{
			ForeignApplicationRegisteredParams params = (ForeignApplicationRegisteredParams) event.getParams();
			List matchingAgentPaths = getMatchingPaths(params.getAgentPaths());
			if(matchingAgentPaths.isEmpty())
				return;
			InstallLoggerCommand cmd = new InstallLoggerCommand(currentAgent.getAgentPath());
			Iterator it = params.getNodeNames().iterator();
			while (it.hasNext())
			{
				String nodeName = (String) it.next();
				IAgentProxy foreignAgent = null;
				try
				{
					foreignAgent = currentAgent.getForeignAgent(nodeName, LoggerAnchor.CAPABILITY_PRODUCER_NAME);
					foreignAgent.accept(cmd);
				}
				catch (Exception e)
				{
					System.out.println(e.getMessage());
					e.printStackTrace();
					currentAgent.getLogger().exception(e);
				}				
			}			
		}		
	}

	/**
	 * @param list
	 * @return
	 */
	protected List getMatchingPaths(List agentPaths)
	{
		List matchingAgentPaths = new ArrayList();
		Iterator it = agentPaths.iterator();
		while (it.hasNext())
		{
			AgentPath agentPath = (AgentPath) it.next();
			if(agentPath.getApplicationName().equals(LoggerAnchor.APPLICATION_NAME) && agentPath.getCapabilityName().equals(LoggerAnchor.CAPABILITY_PRODUCER_NAME))
			{
				matchingAgentPaths.add(agentPath);
			}
		}
		return matchingAgentPaths;
	}

}
