/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.cas.logger.interpreters;

import org.almendra.commons.events.Event;
import org.almendra.commons.util.collection.StringVector;
import org.almendra.commons.util.logging.ILogListener;
import org.almendra.commons.util.logging.RemoveLogListenerParams;
import org.almendra.janet.cas.logger.LogCollector;
import org.almendra.janet.cas.logger.commands.UnInstallLoggerCommand;
import org.almendra.janet.cas.scheduling.CommandAccessor;
import org.almendra.janet.cas.spaces.IObjectSpace;



public class UnInstallLoggerInterpreter extends LoggerInterpreter
{
	/**
	 * Creates a new <code>UnInstallLoggerInterpreter</code> object.
	 */
	public UnInstallLoggerInterpreter()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#execute(org.almendra.commons.janet.cas.scheduling.CommandAccessor)
	 */
	public void execute(CommandAccessor cmdAccessor)
	{
		if(cmdAccessor.getCommand() instanceof UnInstallLoggerCommand)
		{
			UnInstallLoggerCommand cmd = (UnInstallLoggerCommand) cmdAccessor.getCommand();
			uninstallLoggerListener(cmdAccessor, cmd);
		}
	}
	
	protected ILogListener uninstallLoggerListener(CommandAccessor cmdAccessor, UnInstallLoggerCommand cmd)
	{
		String listenerKey = cmd.getAgentPathToLoggerServer().getQualifiedPathName();
		String collectorKey = getCollectorKey(listenerKey);
		IObjectSpace os = cmdAccessor.getApplicationObjectSpace();
		LogCollector logCollector = (LogCollector) os.get(collectorKey);
		logCollector.stop();
		ILogListener logListener = (ILogListener) os.get(listenerKey);
		RemoveLogListenerParams params = new RemoveLogListenerParams(logListener, cmdAccessor.getLogger());
		Event event = new Event(RemoveLogListenerParams.EVENT_NAME, params);
		cmdAccessor.getNodeEventRegistry().occured(event);
		return logListener;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#commandNames()
	 */
	public StringVector commandNames()
	{
		return new StringVector(UnInstallLoggerCommand.QualifiedName);
	}
}
