package org.almendra.janet.cas.monitor.gui;


import java.util.ArrayList;
import java.util.List;

import javax.swing.JOptionPane;

import org.almendra.commons.util.StringUtility;
import org.almendra.commons.util.swing.BillboardPanelController;
import org.almendra.janet.cas.agents.IAgentProxy;
import org.almendra.janet.cas.monitor.MonitorConsumerAnchor;
import org.almendra.janet.cas.monitor.MonitorProducerAnchor;
import org.almendra.janet.cas.monitor.commands.UnInstallProducerCommand;
import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.node.ApplicationAccessor;
import org.almendra.janet.cas.node.NoSuchAgentException;
import org.almendra.janet.cas.node.NodeAccessor;
import org.almendra.janet.cas.node.NodeAnchor;
import org.almendra.janet.cas.scheduling.Acknowledge;

/**
 * Insert the type's description here. Creation date: (08.03.2004 13:22:52)
 * @author: Administrator
 */
public class MonitorViewController
{
	protected NodeAccessor node = null;
	protected ApplicationAccessor application = null;
	protected MonitorView view = null;
	protected boolean shuttingDown = false;
	protected BillboardPanelController billboardController = null;

	/**
	 * NodeViewController constructor comment.
	 */
	public MonitorViewController()
	{
		super();
	}

	/**
	 * NodeViewController constructor comment.
	 */
	public MonitorViewController(MonitorView view, NodeAccessor node)
	{
		super();
		this.view = view;
		this.node = node;
		billboardController = new BillboardPanelController(view.getJTextArea());
	}

	/**
	 * NodeViewController constructor comment.
	 */
	public void setMask(String nodeName, List attributeMask)
	{
		billboardController.setMask(nodeName, attributeMask);
	}

	public void incrementValue(String nodeName, String attributeName)
	{
		billboardController.incrementValue(nodeName, attributeName);
	}
	
	public void decrementValue(String nodeName, String attributeName)
	{
		billboardController.decrementValue(nodeName, attributeName);
	}
	
	/**
	 * NodeViewController constructor comment.
	 */
	public void setValue(String nodeName, String attributeName, String value)
	{
		billboardController.setValue(nodeName, attributeName, value);
	}
	
	/**
	 * NodeViewController constructor comment.
	 */
	public void test()
	{
		billboardController.reset();
		
		List maskNode1 = new ArrayList();
		maskNode1.add(BillboardPanelController.BLANK_LINE);
		maskNode1.add(NodeAnchor.NODE_NAME);
		maskNode1.add(NodeAnchor.FOREIGN_COMMANDS_COUNT);
		maskNode1.add(NodeAnchor.FOREIGN_EXECUTING_COMMANDS_COUNT);
		maskNode1.add(NodeAnchor.FOREIGN_EXECUTING_BALANCING_COMMANDS_COUNT);
		
		List maskNode2 = new ArrayList();
		maskNode2.add(BillboardPanelController.BLANK_LINE);
		maskNode2.add(NodeAnchor.NODE_NAME);
		maskNode2.add(NodeAnchor.FOREIGN_COMMANDS_COUNT);
		maskNode2.add(NodeAnchor.FOREIGN_EXECUTING_COMMANDS_COUNT);
		maskNode2.add(NodeAnchor.FOREIGN_EXECUTING_BALANCING_COMMANDS_COUNT);		
		
		List maskNode3 = new ArrayList();
		maskNode3.add(BillboardPanelController.BLANK_LINE);
		maskNode3.add(NodeAnchor.NODE_NAME);
		maskNode3.add(NodeAnchor.FOREIGN_COMMANDS_COUNT);
		maskNode3.add(NodeAnchor.FOREIGN_EXECUTING_COMMANDS_COUNT);
		maskNode3.add(NodeAnchor.FOREIGN_EXECUTING_BALANCING_COMMANDS_COUNT);		
		
		setMask("node-1", maskNode1);
		setValue("node-1", NodeAnchor.NODE_NAME, "AARHUS-0");
		setValue("node-1", NodeAnchor.FOREIGN_COMMANDS_COUNT, "2");
		setValue("node-1", NodeAnchor.FOREIGN_EXECUTING_COMMANDS_COUNT, "5");
		setMask("node-2", maskNode2);
		setValue("node-2", NodeAnchor.NODE_NAME, "AARHUS-1");
		setValue("node-2", NodeAnchor.FOREIGN_COMMANDS_COUNT, "54");
		setValue("node-2", NodeAnchor.FOREIGN_EXECUTING_BALANCING_COMMANDS_COUNT, "51");
		setMask("node-3", maskNode3);
		setValue("node-3", NodeAnchor.NODE_NAME, "AARHUS-2");
		setValue("node-3", NodeAnchor.FOREIGN_COMMANDS_COUNT, "33");
		setValue("node-3", NodeAnchor.FOREIGN_EXECUTING_COMMANDS_COUNT, "5");
		
		unsetMask("node-1");
		unsetMask("node-2");
		unsetMask("node-3");
		
		setMask("node-2", maskNode2);
		setValue("node-2", NodeAnchor.NODE_NAME, "AARHUS-1");
		setValue("node-2", NodeAnchor.FOREIGN_COMMANDS_COUNT, "54");
		setValue("node-2", NodeAnchor.FOREIGN_EXECUTING_BALANCING_COMMANDS_COUNT, "51");
		
		incrementValue("node-2", NodeAnchor.FOREIGN_COMMANDS_COUNT);
		decrementValue("node-2", NodeAnchor.FOREIGN_COMMANDS_COUNT);
		
		unsetMask("node-2");
		unsetMask("node-3");
	}

	/**
	 * @param string
	 */
	public void unsetMask(String nodeName)
	{
		billboardController.unsetMask(nodeName);
	}

	/**
	 * 
	 */
	public void shutdown()
	{
		if(shuttingDown)
			return;
		shuttingDown = true;
		notifyNodes();
		node.shutdown();
	}
	
	/**
	 * 
	 */
	protected void notifyNodes()
	{
		IAgentProxy agents = application.getAllAgentsWithCapability(MonitorProducerAnchor.CAPABILITY_NAME, MonitorProducerAnchor.AGENT_NAME);
		AgentPath ackAgentPath = null;
		try
		{
			ackAgentPath = application.getLocalAgentPath(
					MonitorConsumerAnchor.CAPABILITY_NAME, 
					MonitorConsumerAnchor.AGENT_NAME);
		}
		catch (NoSuchAgentException e)
		{
			// logical error if this happens - do not handle
			System.out.println(e.getMessage());
			e.printStackTrace();
			application.getLogger().exception(e);
		}
		
		try
		{
			Acknowledge ack = new Acknowledge(ackAgentPath);
			UnInstallProducerCommand cmd = new UnInstallProducerCommand(ackAgentPath);
			agents.accept(cmd, ack);
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			application.getLogger().exception(e);
		}
	}
	
	/**
	 * @return Returns the application.
	 */
	public ApplicationAccessor getApplication()
	{
		return application;
	}
	/**
	 * @param application The application to set.
	 */
	public void setApplication(ApplicationAccessor application)
	{
		this.application = application;
	}

	/**
	 * 
	 */
	public void resetCounts()
	{
		billboardController.resetCounts();
	}

	/**
	 * 
	 */
	public void removeNodes()
	{
		List maskNames = billboardController.getMaskNames();
		if(maskNames == null || maskNames.isEmpty())
		{
			JOptionPane.showMessageDialog(view, "No nodes present!"); //$NON-NLS-1$
			return;
		}
		SelectNodeDialog dialog = SelectNodeDialog.open(StringUtility.toArray(maskNames));
		if(dialog.isCanceled())
			return;
		if(dialog.getSelectedItems().isEmpty())
			return;
		billboardController.hideMasks(dialog.getSelectedItems());
	}
	
}
