/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.cas.monitor.handlers;

import java.util.Iterator;

import org.almendra.commons.events.Event;
import org.almendra.commons.events.IEventHandler;
import org.almendra.janet.cas.agents.IAgentProxy;
import org.almendra.janet.cas.handlers.ForeignApplicationRegisteredParams;
import org.almendra.janet.cas.monitor.MonitorProducerAnchor;
import org.almendra.janet.cas.monitor.commands.InstallProducerCommand;
import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.node.SystemApplication;
import org.almendra.janet.cas.scheduling.CommandAccessor;



public class ForeignApplicationRegisteredHandler implements IEventHandler
{
	/**
	 * Creates a new <code>ForeignApplicationRegisteredHandler</code> object.
	 */
	public ForeignApplicationRegisteredHandler()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.events.IEventHandler#occured(org.almendra.commons.commons.events.Event)
	 */
	public void occured(Event event)
	{
		if(event.getParams() instanceof ForeignApplicationRegisteredParams)
		{
			ForeignApplicationRegisteredParams params = (ForeignApplicationRegisteredParams) event.getParams();
			CommandAccessor cmdAccessor = (CommandAccessor)  params.get(CommandAccessor.LABEL);
			
			Iterator it = params.getAgentPaths().iterator();
			while (it.hasNext())
			{
				AgentPath agentPath = (AgentPath) it.next();
				if(isMonitorProducerAgent(agentPath))
				{
					try
					{
						InstallProducerCommand cmd = new InstallProducerCommand(cmdAccessor.getCurrentAgent().getAgentPath());
						IAgentProxy agent = cmdAccessor.getApplication().getAgent(agentPath);
						agent.accept(cmd);
					}
					catch (Exception e)
					{
						System.out.println(e.getMessage());
						e.printStackTrace();
						cmdAccessor.getLogger().exception(e);
					}
				}
			}
		}
	}

	/**
	 * @param agentPath
	 * @return
	 */
	protected boolean isMonitorProducerAgent(AgentPath agentPath)
	{
		if(!SystemApplication.NAME.equals(agentPath.getApplicationName()))
			return false;
		if(!MonitorProducerAnchor.CAPABILITY_NAME.equals(agentPath.getCapabilityName()))
			return false;
		if(!MonitorProducerAnchor.AGENT_NAME.equals(agentPath.getAgentName()))
			return false;
		return true;
	}
}
