/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.cas.monitor.interpreters;

import org.almendra.commons.events.IEventHandler;
import org.almendra.commons.util.collection.StringVector;
import org.almendra.janet.cas.agents.IAgentProxy;
import org.almendra.janet.cas.events.EventHandlerEnvelope;
import org.almendra.janet.cas.events.HandlerList;
import org.almendra.janet.cas.handlers.ForeignApplicationRegisteredParams;
import org.almendra.janet.cas.monitor.MonitorAnchor;
import org.almendra.janet.cas.monitor.MonitorConsumerAnchor;
import org.almendra.janet.cas.monitor.MonitorProducerAnchor;
import org.almendra.janet.cas.monitor.commands.InstallProducerCommand;
import org.almendra.janet.cas.monitor.commands.StartCommand;
import org.almendra.janet.cas.monitor.gui.MonitorView;
import org.almendra.janet.cas.monitor.gui.MonitorViewController;
import org.almendra.janet.cas.monitor.handlers.ForeignApplicationRegisteredHandler;
import org.almendra.janet.cas.scheduling.CommandAccessor;
import org.almendra.janet.cas.scheduling.IInterpreter;



public class StartInterpreter extends AbstractConsumerInterpreter implements IInterpreter
{
	/**
	 * Creates a new <code>StartInterpreter</code> object.
	 */
	public StartInterpreter()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#execute(org.almendra.commons.janet.cas.scheduling.CommandAccessor)
	 */
	public void execute(CommandAccessor cmdAccessor)
	{
		if(cmdAccessor.getCommand() instanceof StartCommand)
		{
			MonitorViewController controller = MonitorView.open(cmdAccessor.getNodeAccessor());
			controller.setApplication(cmdAccessor.getApplication());
			installAnchor(cmdAccessor, controller);
			installProducers(cmdAccessor);
			installHandler(cmdAccessor);
		}
	}

	/**
	 * @param cmdAccessor
	 */
	protected void installHandler(CommandAccessor cmdAccessor)
	{
		cmdAccessor.setNodeLabel(MonitorAnchor.NODE_LABEL);
		IEventHandler handler = new ForeignApplicationRegisteredHandler();
		EventHandlerEnvelope env = new EventHandlerEnvelope(handler, cmdAccessor.getAgentPath()); 
		HandlerList handlers = new HandlerList(env);
		cmdAccessor.getNodeEventRegistry().register(
			ForeignApplicationRegisteredParams.EVENT_NAME, 
			handlers,
			cmdAccessor.getApplication());
	}

	/**
	 * @param cmdAccessor
	 */
	protected void installAnchor(CommandAccessor cmdAccessor, MonitorViewController controller)
	{
		cmdAccessor.getApplicationObjectSpace().put(MonitorConsumerAnchor.LABEL, new MonitorConsumerAnchor(controller));
	}
	
	/**
	 * @param cmdAccessor
	 */
	protected void installProducers(CommandAccessor cmdAccessor)
	{
		try
		{
			InstallProducerCommand cmd = new InstallProducerCommand(cmdAccessor.getCurrentAgent().getAgentPath());
			IAgentProxy agents = cmdAccessor.getCurrentAgent().getAllForeignAgents(
				MonitorProducerAnchor.CAPABILITY_NAME,
				MonitorProducerAnchor.AGENT_NAME);
			agents.accept(cmd);
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			cmdAccessor.getLogger().exception(e);
		}
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#commandNames()
	 */
	public StringVector commandNames()
	{
		return new StringVector(StartCommand.QualifiedName);
	}
}
