/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.cas.monitor.interpreters;

import org.almendra.commons.util.collection.StringVector;
import org.almendra.janet.cas.agents.IAgentProxy;
import org.almendra.janet.cas.events.HandlerList;
import org.almendra.janet.cas.events.ILocalEventRegistry;
import org.almendra.janet.cas.handlers.NodeShutdownParams;
import org.almendra.janet.cas.monitor.MonitorProducerAnchor;
import org.almendra.janet.cas.monitor.commands.UnInstallProducerCommand;
import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.scheduling.CommandAccessor;
import org.almendra.janet.cas.scheduling.IInterpreter;



public class UnInstallProducerInterpreter extends AbstractProducerInterpreter implements IInterpreter
{
	/**
	 * Creates a new <code>UnInstallProducerInterpreter</code> object.
	 */
	public UnInstallProducerInterpreter()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#execute(org.almendra.commons.janet.cas.scheduling.CommandAccessor)
	 */
	public void execute(CommandAccessor cmdAccessor)
	{
		if (cmdAccessor.getCommand() instanceof UnInstallProducerCommand)
		{
			UnInstallProducerCommand cmd = (UnInstallProducerCommand) cmdAccessor.getCommand();
			ILocalEventRegistry registry = cmdAccessor.getNodeEventRegistry();
			AgentPath consumerAgentPath = cmd.getAgentPath();
			MonitorProducerAnchor anchor = getAnchor(cmdAccessor);
			IAgentProxy consumerAgent = (IAgentProxy) anchor.getConsumerAgentPaths().remove(consumerAgentPath.getQualifiedPathName());
			
			// Check whether already shutdown by NodeShutdownHandler
			if(consumerAgent == null)
				return;
			
			cmdAccessor.getNodeIfPrivilege().removeMonitorAgents(consumerAgent);
			
			HandlerList handlerList = (HandlerList) anchor.getShutdownHandlers().remove(consumerAgentPath.getQualifiedPathName());
			registry.deregister(NodeShutdownParams.EVENT_NAME, handlerList);
		}
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#commandNames()
	 */
	public StringVector commandNames()
	{
		return new StringVector(UnInstallProducerCommand.QualifiedName);
	}
}
