/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.simulator;

import java.rmi.AlreadyBoundException;
import java.util.Arrays;
import java.util.Collections;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.almendra.commons.util.StringUtility;


/**
 * @author Oliver
 *
 */
public class Registry
{
	/**
	 * Declared public on Purpose so that the contents can be inspected when testing. 
	 * The simulated mode is for develeopment and test only.
	 */
	public static Registry TheInstance = null;
	
	protected Map registry = Collections.synchronizedMap(new Hashtable());
	
	public Registry()
	{
		super();
	}

	public Object lookup(String key)
	{
		synchronized(registry)
		{
			return registry.get(key);
		}
	}
	
	public void bind(String key, Object object)
	throws AlreadyBoundException
	{
		synchronized(registry)
		{
			if(registry.containsKey(key))
				throw new AlreadyBoundException("object with key " + StringUtility.print(key) + " already bound with registry!"); //$NON-NLS-1$ //$NON-NLS-2$
			registry.put(key, object);
		}
	}
	
	public void unbind(String key)
	{
		synchronized(registry)
		{
			registry.remove(key);
		}
	}
	
	public static Registry get()
	{
		if(TheInstance == null)
			TheInstance = new Registry();
		return TheInstance;
	}
	
	public String[] list()
	{
		synchronized(registry)
		{
			List keys = Arrays.asList(registry.keySet().toArray());
			String[] keysArray = new String[keys.size()];
			for (int i = 0; i < keys.size(); i++)
				keysArray[i] = (String) keys.get(i);
			return keysArray;
		}
	}

	/**
	 * 
	 */
	public static void clear()
	{
		TheInstance = null;
	}
}
