/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.agents;

import java.rmi.RemoteException;

import org.almendra.commons.events.IHistorizableCallbackHandler;
import org.almendra.commons.threads.FutureResult;
import org.almendra.commons.util.StringUtility;
import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.node.InvalidCommandException;
import org.almendra.janet.cas.node.Logger;
import org.almendra.janet.cas.scheduling.Acknowledge;
import org.almendra.janet.cas.scheduling.ICommand;


/**
 * @author Oliver
 *
 */
public abstract class AbstractAgentProxy implements IAgentProxy
{
	protected Logger logger = null;
	
	protected String originNodeName = null;

	/**
	 * Creates a new <code>AbstractAgentProxy</code> object.
	 */
	protected AbstractAgentProxy()
	{
		super();
	}
	
	/**
	 * Creates a new <code>AbstractAgentProxy</code> object.
	 */
	public AbstractAgentProxy(Logger logger)
	{
		super();
		setLogger(logger);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.node.IAgentProxy#accept(org.almendra.commons.janet.cas.logger.schedulers.ICommand)
	 */
	public abstract void accept(ICommand command) 
	throws RemoteException, InvalidCommandException;

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.node.IAgentProxy#accept(org.almendra.commons.janet.cas.logger.schedulers.ICommand, org.almendra.commons.commons.threads.AckHandler)
	 */
	public abstract void accept(ICommand command, Acknowledge ack) 
	throws RemoteException, InvalidCommandException;
	
	public abstract void accept(ICommand command, FutureResult futureResult) 
	throws IllegalArgumentException, RemoteException, InvalidCommandException;
	
	public abstract void accept(ICommand command, IHistorizableCallbackHandler handler) 
	throws RemoteException, InvalidCommandException;

	/**
	 * @return Returns the originNodeName.
	 */
	protected String getOriginNodeName()
	{
		return originNodeName;
	}

	/**
	 * @param originNodeName The originNodeName to set.
	 */
	public void setOriginNodeName(String originNodeName)
	{
		this.originNodeName = originNodeName;
	}

	/**
	 * @return
	 */
	protected abstract AgentPath replyPathSameAgent();

	/**
	 * @return Returns the logger.
	 */
	public Logger getLogger()
	{
		return logger;
	}

	/**
	 * @param logger The logger to set.
	 */
	public void setLogger(Logger logger)
	{		
		this.logger = Logger.getChildLogger(logger);
	}

	/**
	 * @param command
	 */
	protected void checkValidity(ICommand command)
	throws InvalidCommandException
	{
		if(!StringUtility.isValid(command.getQualifiedName()))
			throw new InvalidCommandException("invalid qualified command name: " + StringUtility.print(command.getQualifiedName())); //$NON-NLS-1$
			
	}

}
