/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.oliverplohmann.de/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.janet.cas.agents;

import java.rmi.RemoteException;

import org.almendra.commons.events.IHistorizableCallbackHandler;
import org.almendra.commons.threads.FutureResult;
import org.almendra.janet.cas.node.InvalidCommandException;
import org.almendra.janet.cas.scheduling.Acknowledge;
import org.almendra.janet.cas.scheduling.CommandEnvelope;
import org.almendra.janet.cas.scheduling.ICommand;
import org.almendra.janet.cas.scheduling.IWorkLoadBalancingCommand;
import org.almendra.janet.cas.scheduling.IWorkLoadSharingCommand;


/**
 * @author Oliver
 *  
 */
public interface IAgentProxy
{
	/**
	 * @param command
	 * @throws RemoteException
	 */
	public void accept(ICommand command) throws RemoteException, InvalidCommandException;
	
	/**
	 * @param command
	 * @throws RemoteException
	 */
	public void accept(ICommand command, int kind) throws RemoteException, InvalidCommandException;
	
	/**
	 * @param command
	 * @throws RemoteException
	 */
	public void accept(IWorkLoadSharingCommand command) throws RemoteException, InvalidCommandException;
	
	/**
	 * @param command
	 * @throws RemoteException
	 */
	public void accept(IWorkLoadBalancingCommand command) throws RemoteException, InvalidCommandException;
	
	/**
	 * @param command
	 * @param ackHandler
	 */
	public void accept(ICommand command, Acknowledge ack) 
	throws RemoteException, InvalidCommandException;

	/**
	 * Send <code>commmand</code> to an agent expecting a result to be sent back.
	 * @param command to be sent to another agent
	 * @param replyToAgentPath path to agent that should receive result
	 * @return a future that blocks until the result has arrived or times out.
	 * @throws IllegalArgumentException
	 */
	public void accept(ICommand command, FutureResult futureResult) 
	throws IllegalArgumentException, RemoteException, InvalidCommandException;
	
	/**
	 * 
	 * @param command
	 * @param handler
	 * @throws RemoteException
	 * @throws InvalidCommandException
	 */
	public void accept(ICommand command, IHistorizableCallbackHandler handler) 
	throws RemoteException, InvalidCommandException;
	
	/**
	 * @param command
	 * @param ackHandler
	 */
	public void accept(IWorkLoadSharingCommand command, Acknowledge ack) 
	throws RemoteException, InvalidCommandException;

	/**
	 * Send <code>commmand</code> to an agent expecting a result to be sent back.
	 * @param command to be sent to another agent
	 * @param replyToAgentPath path to agent that should receive result
	 * @return a future that blocks until the result has arrived or times out.
	 * @throws IllegalArgumentException
	 */
	public void accept(IWorkLoadSharingCommand command, FutureResult futureResult) 
	throws IllegalArgumentException, RemoteException, InvalidCommandException;
	
	/**
	 * 
	 * @param command
	 * @param handler
	 * @throws RemoteException
	 * @throws InvalidCommandException
	 */
	public void accept(IWorkLoadBalancingCommand command, IHistorizableCallbackHandler handler) 
	throws RemoteException, InvalidCommandException;
	
	
	/**
	 * @param command
	 * @param ackHandler
	 */
	public void accept(IWorkLoadBalancingCommand command, Acknowledge ack) 
	throws RemoteException, InvalidCommandException;

	/**
	 * Send <code>commmand</code> to an agent expecting a result to be sent back.
	 * @param command to be sent to another agent
	 * @param replyToAgentPath path to agent that should receive result
	 * @return a future that blocks until the result has arrived or times out.
	 * @throws IllegalArgumentException
	 */
	public void accept(IWorkLoadBalancingCommand command, FutureResult futureResult) 
	throws IllegalArgumentException, RemoteException, InvalidCommandException;
	
	/**
	 * 
	 * @param command
	 * @param handler
	 * @throws RemoteException
	 * @throws InvalidCommandException
	 */
	public void accept(IWorkLoadSharingCommand command, IHistorizableCallbackHandler handler) 
	throws RemoteException, InvalidCommandException;
	
	/**
	 * @param command
	 * @throws RemoteException
	 */
	public void accept(CommandEnvelope env) throws RemoteException, InvalidCommandException;
	
	/**
	 * @return
	 */
	public int size();
}
