/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.oliverplohmann.de/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.janet.cas.central.interpreters;

import java.rmi.AccessException;
import java.rmi.AlreadyBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.Registry;

import org.almendra.commons.util.collection.StringVector;
import org.almendra.commons.util.inet.RMIUtility;
import org.almendra.commons.util.inet.RegistryLocation;
import org.almendra.janet.cas.commands.NodeStartedCommand;
import org.almendra.janet.cas.node.CentralAnchor;
import org.almendra.janet.cas.node.Node;
import org.almendra.janet.cas.node.interpreters.NodeStartedTrait;
import org.almendra.janet.cas.scheduling.CommandAccessor;
import org.almendra.janet.cas.scheduling.IInterpreter;
import org.almendra.janet.cas.spaces.IObjectSpace;


/**
 * @author Oliver
 *  
 */
public class NodeStartedInterpreter extends CentralInterpreter implements IInterpreter
{
	public static final String ViewTitle = "Janet.central"; //$NON-NLS-1$
	public static final String Label = "Central"; //$NON-NLS-1$

	protected Registry registry = null;
	protected NodeStartedTrait nodeStartedTrait = new NodeStartedTrait();

	public NodeStartedInterpreter()
	{
		super();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.almendra.commons.cas.node.IInterpreter#execute(org.almendra.commons.cas.node.ICommand)
	 */
	public void execute(CommandAccessor cmdAccessor)
	{
		if (cmdAccessor.getCommand() instanceof NodeStartedCommand)
		{
			NodeStartedCommand command = (NodeStartedCommand) cmdAccessor.getCommand();
			IObjectSpace os = cmdAccessor.getObjectSpace();
			
			CentralAnchor anchor = installNodeAnchor(os, command.getNode());
			
			try
			{
				registerNode(command, os);
			}
			catch (Exception ex)
			{
				System.out.println(ex.getMessage());
				ex.printStackTrace();
				cmdAccessor.log(ex);
				command.getFuture().setException(ex);
				command.getFuture().signal();
				return;
			}
			
			try
			{
				nodeStartedTrait.bindClusterSubobjects(cmdAccessor.getNode(), command);
			}
			catch (Exception ex)
			{
				System.out.println(ex.getMessage() + ": cannot bind cluster subobject"); //$NON-NLS-1$
				ex.printStackTrace();
				cmdAccessor.log(ex);
				command.getFuture().setException(ex);
			}

			// TODO - Arbitrator must not be started before node has registered.
			// not implemented yet.
			// if(supremeScheduler started)
			// start arbitrator as well
			nodeStartedTrait.showGUI(command);
			command.getFuture().signal();
		}
	}

	protected CentralAnchor installNodeAnchor(IObjectSpace os, Node node)
	{
		CentralAnchor anchor = new CentralAnchor(node);
		anchor.setViewTitle(ViewTitle);
		anchor.setLabel(Label);
		os.put(CentralAnchor.LABEL, anchor);
		node.setName(CentralAnchor.NAME);
		return anchor;
	}

	protected void shutdown()
	{
		// TODO - not yet implemented
		// CentralAnchor already started. Only 1 instance must be present.
		// So shutdown this instance
	}

	protected boolean registerNode(NodeStartedCommand command, IObjectSpace os) 
	throws AccessException, RemoteException, AlreadyBoundException
	{
		RegistryLocation location = command.getCentralRegistryLocation();		
		Registry registry = RMIUtility.ensureRegistryExists(location);
		registry.bind(CentralAnchor.RMI_NAME, command.getNode().getAgentDispatcher());
		getAnchor(os).setName(CentralAnchor.NAME);
		return true;
	}

	/**
	 * Call registry to check that it is not simply a dead stub but an active instance
	 * @param registry
	 * @throws RemoteException
	 * @throws AccessException
	 */
	protected void checkAlive(Registry registry) throws RemoteException, AccessException
	{
		if(registry != null)
			registry.list();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#commandNames()
	 */
	public StringVector commandNames()
	{
		return new StringVector(NodeStartedCommand.QualifiedName);
	}

}
