/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.oliverplohmann.de/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.janet.cas.central.interpreters;

import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.util.List;

import org.almendra.commons.util.AssertUtility;
import org.almendra.commons.util.collection.StringVector;
import org.almendra.janet.cas.agents.IAgentProxy;
import org.almendra.janet.cas.commands.RegisterNodeCommand;
import org.almendra.janet.cas.commands.RegisteredNodeCommandEvaluator;
import org.almendra.janet.cas.node.CentralAnchor;
import org.almendra.janet.cas.node.IAgentDispatcher;
import org.almendra.janet.cas.node.NodeLocation;
import org.almendra.janet.cas.node.interpreters.AbstractRegisterNodeTrait;
import org.almendra.janet.cas.node.interpreters.RegisterNodeTrait;
import org.almendra.janet.cas.scheduling.CommandAccessor;
import org.almendra.janet.cas.scheduling.ICommandEvaluator;
import org.almendra.janet.cas.scheduling.IInterpreter;


/**
 * @author Oliver
 *  
 */
public class RegisterNodeInterpreter extends CentralInterpreter implements IInterpreter
{
	protected AbstractRegisterNodeTrait registerNodeTrait = createRegisterNodeTrait();
	
	public RegisterNodeInterpreter()
	{
		super();
	}

	protected AbstractRegisterNodeTrait createRegisterNodeTrait()
	{
		return new RegisterNodeTrait();
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.almendra.commons.cas.node.schedulerAnchor.IInterpreter#execute()
	 */
	public void execute(CommandAccessor cmdAccessor)
	{
		if (cmdAccessor.getCommand() instanceof RegisterNodeCommand)
		{
			RegisterNodeCommand command = (RegisterNodeCommand) cmdAccessor.getCommand();
			CentralAnchor anchor = (CentralAnchor) cmdAccessor.getAbstractNodeAnchorIfPrivilege();			

			// add the new node to the central's cluster image
			IAgentDispatcher agentDispatcher = null;
			
			try
			{
				agentDispatcher = registerNodeTrait.getAgentDispatcher(command.getNodeLocation(), cmdAccessor.getLogger());
				anchor.registerNode(agentDispatcher, command.getNodeLocation());
			}
			catch (RemoteException ex)
			{
				System.out.println(ex.getMessage());
				ex.printStackTrace();
				cmdAccessor.log(ex);
				return;
			}
			catch (NotBoundException ex)
			{
				System.out.println(ex.getMessage());
				ex.printStackTrace();
				cmdAccessor.log(ex);
			}

			// tell new node about existing nodes
			NodeLocation nodeLocation = command.getNodeLocation();
			List nodeLocations = anchor.getCentralClusterImage().getAllForeignNodeLocations();
			boolean found = nodeLocations.remove(nodeLocation);
			AssertUtility.assertTrue(found);
			ICommandEvaluator commandEvaluator = new RegisteredNodeCommandEvaluator(nodeLocations);
			
			try
			{
				IAgentProxy agent = cmdAccessor.getSystemAgent(nodeLocation.getNodeName());
				agent.accept(commandEvaluator.get());
			}
			catch (Exception ex)
			{
				System.out.println(ex.getMessage());
				ex.printStackTrace();
				cmdAccessor.log(ex);
			}
			
			registerNodeTrait.sendEventNodeAboutToRegister(cmdAccessor, nodeLocation.getNodeName());
		}

	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#commandNames()
	 */
	public StringVector commandNames()
	{
		return new StringVector(RegisterNodeCommand.QualifiedName);
	}

}
