/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.commands;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import org.almendra.commons.threads.FutureResult;
import org.almendra.commons.util.inet.RegistryLocation;
import org.almendra.janet.cas.node.CentralAnchor;
import org.almendra.janet.cas.node.Node;
import org.almendra.janet.cas.node.NodeLocation;
import org.almendra.janet.cas.scheduling.ICommand;


/**
 * @author Oliver
 *
 */
public class NodeStartedCommand implements ICommand, Serializable
{
	public static final String QualifiedName = "org.almendra.commons.cas.node.commands.NodeStartedCommand"; //$NON-NLS-1$
	
	protected Node node = null;
	protected FutureResult future = null;
	protected RegistryLocation centralRegistryLocation = null;
	protected RegistryLocation localRegistryLocation = null;
	protected boolean startGUI = false;
	protected List clusterSpaceDescriptors = new ArrayList();
	protected List clusterEventRegistryDescriptors = new ArrayList();
		
	/**
	 * @return Returns the future.
	 */
	public FutureResult getFuture()
	{
		return future;
	}

	protected NodeStartedCommand()
	{
		super();
	}
	
	public NodeStartedCommand(
			Node node, 
			FutureResult future, 
			RegistryLocation centralRegistryLocation, 
			RegistryLocation localRegistryLocation,
			List clusterSpaceDescriptors,
			List clusterEventRegistryDescriptors,
			boolean startGUI)
	{
		super();
		this.node = node;
		this.future = future;
		this.centralRegistryLocation = centralRegistryLocation;
		this.localRegistryLocation = localRegistryLocation;
		this.clusterSpaceDescriptors = clusterSpaceDescriptors;
		this.clusterEventRegistryDescriptors = clusterEventRegistryDescriptors;
		this.startGUI = startGUI;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.cas.node.ICommand#getQualifiedName()
	 */
	public String getQualifiedName()
	{
		return QualifiedName;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.cas.node.ICommand#getPriority()
	 */
	public int getPriority()
	{
		return ICommand.SystemPriority;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.cas.node.ICommand#toLogString()
	 */
	public String toLogString()
	{
		return "NodeStartedCommand"; //$NON-NLS-1$
	}

	/**
	 * @return Returns the node.
	 */
	public Node getNode()
	{
		return node;
	}

	/**
	 * @return Returns the centralRegistryLocation.
	 */
	public RegistryLocation getCentralRegistryLocation()
	{
		return centralRegistryLocation;
	}

	/**
	 * @return Returns the localRegistryLocation.
	 */
	public RegistryLocation getLocalRegistryLocation()
	{
		return localRegistryLocation;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	public Object clone() throws CloneNotSupportedException
	{
		NodeStartedCommand cmd = (NodeStartedCommand) super.clone();
		return cmd;
	}
	
	public NodeLocation getCentralNodeLocation()
	{
		return new NodeLocation(CentralAnchor.NAME, getCentralRegistryLocation());
	}

	/**
	 * @return Returns the startGUI.
	 */
	public boolean isStartGUI()
	{
		return startGUI;
	}

	/**
	 * @return
	 */
	public List getClusterSpaceDescriptors()
	{
		return clusterSpaceDescriptors;
	}

	/**
	 * @return Returns the clusterEventRegistryDescriptors.
	 */
	public List getClusterEventRegistryDescriptors()
	{
		return clusterEventRegistryDescriptors;
	}

}
