/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.events;

import java.util.Iterator;
import java.util.NoSuchElementException;

import org.almendra.commons.events.Event;
import org.almendra.commons.events.EventNotifier;
import org.almendra.commons.events.IEventRegistry;
import org.almendra.commons.util.collection.IListEvaluator;
import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.node.ApplicationAccessor;
import org.almendra.janet.cas.node.Node;


public class EventRegistryEnvelope implements ILocalEventRegistry
{
	protected IEventRegistry eventRegistry = null;
	protected Node node = null;
	
	/**
	 * Creates a new <code>EventRegistryEnvelope</code> object.
	 */
	public EventRegistryEnvelope(Node node, IEventRegistry eventRegistry)
	{
		super();
		this.eventRegistry = eventRegistry;
		this.node = node;
	}	
	
	/**
	 * @param eventName
	 * @param listEvaluator
	 * @throws NoSuchElementException
	 */
	public boolean deregister(String eventName, HandlerList list) throws NoSuchElementException
	{
		return eventRegistry.deregister(eventName, list);
	}

	/**
	 * @param eventName
	 * @return
	 */
	public boolean isRegistered(String eventName)
	{
		return eventRegistry.isRegistered(eventName);
	}

	/**
	 * Creates a new <code>EventRegistryEnvelope</code> object.
	 */
	protected EventRegistryEnvelope()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.events.EventRegistry#getEventNotifier(org.almendra.commons.commons.util.collection.IListEvaluator)
	 */
	protected EventNotifier getEventNotifier(IListEvaluator listEvaluator)
	{
		return new EventNotifier(listEvaluator);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.events.ILocalEventRegistry#register(java.lang.String, org.almendra.commons.commons.util.collection.IListEvaluator)
	 */
	public void register(String eventName, HandlerList list, ApplicationAccessor application) throws IllegalArgumentException
	{
		Iterator it = list.iterator();
		while (it.hasNext())
		{			
			EventHandlerEnvelope handlerEnv = (EventHandlerEnvelope) it.next();
			if(handlerEnv.getAgentPath() == null)
				throw new IllegalArgumentException("no agent path supplied!"); //$NON-NLS-1$
			handlerEnv.setNode(node);
			handlerEnv.setApplication(application);
			// Make sure the user has no way to provide an agent path to an 
			// agent on a different node or a different application. 
			handlerEnv.getAgentPath().setNodeName(AgentPath.LOCAL_NODE_TAG);
			handlerEnv.getAgentPath().setApplicationName(application.getName());
		}
		eventRegistry.register(eventName, list);
	}

	/**
	 * @param event
	 * @throws PropagatedException
	 */
	public void occured(Event event)
	{
		eventRegistry.occured(event);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.events.ILocalEventRegistry#getHandlerCount(java.lang.String)
	 */
	public int getHandlerCount(String eventName)
	{
		return eventRegistry.getHandlerCount(eventName);
	}

}
