/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.cas.events;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.almendra.commons.util.AssertUtility;
import org.almendra.commons.util.collection.IListEvaluator;
import org.almendra.janet.cas.node.AgentPath;



public class HandlerList extends AbstractHandlerList implements IListEvaluator, Serializable
{
	/**
	 * Creates a new <code>HandlerList</code> object.
	 */
	public HandlerList()
	{
		super();
	}
	
	/**
	 * Creates a new <code>HandlerList</code> object.
	 */
	public HandlerList(EventHandlerEnvelope item)
	{
		super();
		add(item);
	}

	public boolean add(EventHandlerEnvelope item)
	{
		AgentPath agentPath = item.getAgentPath();
		AssertUtility.assertNotNull(agentPath);
		agentPath.checkValidity();
		return list.add(item);
	}
	
	/**
	 * @param nodeName
	 */
	public void removeHandlers(String nodeName)
	{
		List handlers = new ArrayList();
		Iterator it = list.iterator();
		while (it.hasNext())
		{
			EventHandlerEnvelope env = (EventHandlerEnvelope) it.next();
			if(env.getNode().equals(nodeName))
			{
				handlers.add(env);
			}
		}
		it = handlers.iterator();
		while (it.hasNext())
		{
			boolean result = list.remove(it.next());
			AssertUtility.assertTrue(result);
		}
	}

}
