/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.oliverplohmann.de/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.janet.cas.node;


import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.almendra.commons.util.AssertUtility;
import org.almendra.janet.cas.agents.*;
import org.almendra.janet.cas.schedulers.ApplicationScheduler;

/**
 * @author Oliver
 * 
 * To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Generation - Code and Comments
 */
public class Application extends AbstractApplication
{
	protected int numOfCreatedAgents = -1;
	protected int numOfCreatedSchedulers = -1;
	protected boolean isPropagated = false;

	/**
	 * @return Returns the isPropagated.
	 */
	public boolean isPropagated()
	{
		return isPropagated;
	}

	/**
	 * @param isPropagated
	 *            The isPropagated to set.
	 */
	public void setPropagated(boolean isPropagated)
	{
		this.isPropagated = isPropagated;
	}	

	public Application()
	{
		super();
	}

	public Application(String name)
	{
		super(name);
	}

	/**
	 * @param logger
	 * @param node
	 * @param name
	 */
	public Application(Logger logger, Node node, String name)
	{
		super(logger, node, name);
	}

	/**
	 * @return Returns the numOfCreatedAgents.
	 */
	public int getNumOfCreatedAgents()
	{
		return numOfCreatedAgents;
	}

	/**
	 * @return Returns the numOfCreatedSchedulers.
	 */
	public int getNumOfCreatedSchedulers()
	{
		return numOfCreatedSchedulers;
	}

	/**
	 * Start all the agents of the capability. If an agent shares its schedulerAnchor
	 * with other agents, the schedulerAnchor is only created once for all these
	 * agents sharing it.
	 * 
	 * @return
	 */
	protected void primStart()
	{
		AssertUtility.assertTrue(isPropagated());
		primStartAgents();
		primStartInitialCommands();
		super.primStart();
	}

	private void primStartAgents()
	{
		Iterator it = getCapabilities().iterator();
		List createdAgentNames = new ArrayList();
		int schedulersCreated = 0;
		while (it.hasNext())
		{
			Capability cap = (Capability) it.next();
			Iterator namesIt = cap.getAgentNames().iterator();
			while (namesIt.hasNext())
			{
				String agentName = (String) namesIt.next();
				if (!createdAgentNames.contains(agentName))
				{
					ApplicationScheduler scheduler = cap.createApplicationScheduler();
					cap.createAgent(agentName, scheduler);
					createdAgentNames.add(agentName);

					// create any other agents that share the schedulerAnchor with
					// the previously created one.
					Iterator sharingAgentsIt = cap.getAgentsSharingSchedulerWith(agentName).iterator();
					while (sharingAgentsIt.hasNext())
					{
						String sharingAgentName = (String) sharingAgentsIt.next();
						if (!createdAgentNames.contains(sharingAgentName))
						{
							Agent agent = new Agent(node, cap, scheduler, node.getLogger(), sharingAgentName);
							getNode().getLocalAgentDispatcher().add(agent);
							createdAgentNames.add(sharingAgentName);
						}
					}
					schedulersCreated++;
				}
			}
		}
		numOfCreatedAgents = createdAgentNames.size();
		numOfCreatedSchedulers = schedulersCreated;
	}

}
