/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.oliverplohmann.de/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.janet.cas.node;

import java.util.List;
import java.util.Map;
import java.util.Set;

import org.almendra.commons.util.exception.InsufficientPrivilegeException;
import org.almendra.janet.cas.agents.AgentProxy;
import org.almendra.janet.cas.agents.IAgentProxy;
import org.almendra.janet.cas.events.ILocalEventRegistry;
import org.almendra.janet.cas.spaces.*;


public class ApplicationAccessor
{
	protected AbstractApplication application = null;

	/**
	 * @param localPath
	 * @return
	 */
	public IAgentProxy getAllAgents(LocalAgentPath localPath)
	{
		return application.getAllAgents(localPath);
	}
	/**
	 * @param capabilityName
	 * @param agentName
	 * @return
	 */
	public IAgentProxy getAllAgentsWithCapability(String capabilityName, String agentName)
	{
		return application.getAllAgentsWithCapability(capabilityName, agentName);
	}
	/**
	 * @return
	 */
	public Logger getLogger()
	{
		return application.getLogger();
	}
	/**
	 * Creates a new <code>ApplicationAccessor</code> object.
	 *  
	 */
	public ApplicationAccessor(AbstractApplication application)
	{
		super();
		this.application = application;
	}

	/**
	 * Creates a new <code>ApplicationAccessor</code> object.
	 *  
	 */
	protected ApplicationAccessor()
	{
		super();
	}

	/**
	 * @return
	 */
	public Set getCapabilities()
	{
		return application.getCapabilities();
	}

	/**
	 * @param capabilityName
	 * @return
	 */
	public Capability getCapability(String capabilityName)
	{
		return application.getCapability(capabilityName);
	}

	/**
	 * @return
	 */
	public IObjectSpace getObjectSpace()
	{
		return application.getObjectSpace();
	}

	/**
	 * @return
	 */
	public String getName()
	{
		return application.getName();
	}

	/**
	 * @param capabilities
	 * @return
	 */
	public boolean hasCapabilities(Set capabilities)
	{
		return application.hasCapabilities(capabilities);
	}

	/**
	 * @return
	 */
	public List registeredLogEvents()
	{
		return application.registeredLogEvents();
	}

	/**
	 * @return
	 */
	public String getNodeName()
	{
		return application.getNode().getName();
	}
	
	public IAgentProxy getAgent(AgentPath path)
	throws NoSuchAgentException, IllegalAgentAccessException
	{
		return application.getAgent(path);
	}

	/**
	 * @return
	 */
	public List getForeignNodeNames()
	{
		return application.getForeignNodeNames();
	}

	/**
	 * Return a list with all the nodes that have an application with the same name registered
	 * including the owner node of this application.
	 * @return
	 */
	public List getNodeNames()
	{
		return application.getNodeNames();
	}

	/**
	 * @return
	 */
	public NodeAccessor getNodeAccessor()
	{
		return application.getNode().getAccessor();
	}

	/**
	 * @param cap_name
	 * @param agent_name
	 * @return
	 */
	public AgentPath getLocalAgentPath(String capName, String agentName) throws NoSuchAgentException
	{
		return application.getLocalAgentPath(capName, agentName);
	}

	/**
	 * @return
	 */
	public List getAllNodeNames()
	{
		return application.getAllNodeNames();
	}
	/**
	 * @return
	 */
	public ILocalEventRegistry getEventRegistry()
	{
		return application.getEventRegistry();
	}
	
	/**
	 * @return
	 */
	public boolean isSystemApplication()
	{
		return application.isSystemApplication();
	}
	
	/**
	 * @param path
	 * @return
	 * @throws NoSuchAgentException
	 * @throws IllegalAgentAccessException
	 */
	public AgentProxy getCrossAgent(AgentPath path) 
	throws NoSuchAgentException, IllegalAgentAccessException, InsufficientPrivilegeException
	{
		return application.getCrossAgent(path);
	}
	/**
	 * @return
	 */
	public List getMonitorMask()
	{
		return application.getMonitorMask();
	}
	/**
	 * @return
	 */
	public Map getMonitorMaskDefaultValues()
	{
		return application.getMonitorMaskDefaultValues();
	}
	/**
	 * @param localPath
	 * @return
	 */
	public List getAllAgentPaths(LocalAgentPath localPath)
	{
		return application.getAllAgentPaths(localPath);
	}
}
