/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.node;

import java.io.Serializable;

/**
 * @author Oliver
 *
 */
public class CapabilityPath extends LocalCapabilityPath implements IPath, Serializable
{
	protected String nodeName = null;
	
	/**
	 * Creates a new <code>CapabilityPath</code> object.
	 */
	protected CapabilityPath()
	{
		super();
	}
	
	/**
	 * Creates a new <code>CapabilityPath</code> object.
	 */
	public CapabilityPath(String nodeName, String applicationName, String capabilityName)
	{
		super();
		init(applicationName, capabilityName);
		this.nodeName = nodeName;
	}

	/**
	 * @return Returns the nodeName.
	 */
	public String getNodeName()
	{
		return nodeName;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object arg)
	{
		if(this == arg)
			return true;
		if(arg instanceof CapabilityPath)
		{
			CapabilityPath path = (CapabilityPath) arg;
			return sameAttributeValues(path);
		}
		return false;
	}

	/**
	 * @param path
	 * @return
	 */
	protected boolean sameAttributeValues(CapabilityPath path)
	{
		if(!super.sameAttributeValues(path))			
			return false;
		return nodeName.equals(path.getNodeName());
	}
	
	/**
	 * @return
	 */
	public String getQualifiedPathName()
	{
		return nodeName + "." + super.getQualifiedPathName(); //$NON-NLS-1$
	}

}
