/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.cas.node;

import java.rmi.registry.Registry;

import org.almendra.commons.util.inet.InetUtility;
import org.almendra.commons.util.inet.RegistryLocation;
import org.almendra.commons.util.xml.XMLParseException;
import org.jdom.Element;



public abstract class ClusterSubobjectDescriptor extends Descriptor
{
	public static final String XMLNameTag = "name"; //$NON-NLS-1$
	public static final String XMLHostnameTag = "hostname"; //$NON-NLS-1$
	public static final String XMLPortTag = "port"; //$NON-NLS-1$
	
	protected String name = null;
	protected RegistryLocation registryLocation = null;
	
	/**
	 * Creates a new <code>ClusterSubobjectDescriptor</code> object.
	 */
	public ClusterSubobjectDescriptor()
	{
		super();
	}
	
	/**
	 * Creates a new <code>ClusterSubobjectDescriptor</code> object.
	 */
	public ClusterSubobjectDescriptor(String name, RegistryLocation registryLocation)
	{
		super();
		this.name = name;
		this.registryLocation = registryLocation;
	}

	/**
	 * @return Returns the name.
	 */
	public String getName()
	{
		return name;
	}

	/**
	 * @return Returns the registryLocation.
	 */
	public RegistryLocation getRegistryLocation()
	{
		return registryLocation;
	}

	protected abstract String getXMLParentTag();
	protected abstract String getXMLNodeTag();	
	
	/**
	 * @param clusterSpaceNode
	 */
	protected void fill(Element element) throws XMLParseException
	{
		name = element.getAttributeValue(XMLNameTag);
		if(name == null)
			throw new XMLParseException("no cluster objectSpace name specified!"); //$NON-NLS-1$
		fillRegistryLocation(element);
	}

	/**
	 * @param element
	 */
	protected void fillRegistryLocation(Element element) throws XMLParseException
	{
		String hostnameString = element.getAttributeValue(XMLHostnameTag);
		if(hostnameString != null && hostnameString.toLowerCase().equals(InetUtility.LOCALHOST))
			hostnameString = InetUtility.getHostName();
		if(hostnameString == null)
			throw new XMLParseException("no central objectSpace hostname found for " + name); //$NON-NLS-1$
		
		int port = -1;
		String portString = element.getAttributeValue(XMLPortTag);
		if(portString == null)
		{
			port = Registry.REGISTRY_PORT;
			registryLocation = new RegistryLocation(hostnameString, port);
			return;
		}
		
		try
		{
			port = Integer.parseInt(portString);
		}
		catch (NumberFormatException ex)
		{
			throw new XMLParseException("invalid port registry number "  + portString + " for central objectSpace: " + name + "!"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
	
		registryLocation = new RegistryLocation(hostnameString, port);		
	}

}
