/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.node;

import org.almendra.commons.util.AssertUtility;
import org.almendra.commons.util.StringUtility;
import org.almendra.commons.util.xml.XMLParseException;
import org.jdom.Element;


/**
 * @author Oliver
 *
 */
public class EventDescriptor
{
	public static final String XMLEventTag = "event"; //$NON-NLS-1$
	public static final String XMLEventNameTag = "name"; //$NON-NLS-1$
	public static final String XMLEventHandlerTag = "handler"; //$NON-NLS-1$
	public static final String XMLAgentPathTag = "agent"; //$NON-NLS-1$
	
	protected String name = null;
	protected String handlerName = null;
	protected String agentPathString = null;
	
	/**
	 * Creates a new <code>EventDescriptor</code> object.
	 */
	protected EventDescriptor()
	{
		super();
	}

	/**
	 * Creates a new <code>EventDescriptor</code> object.
	 */
	public EventDescriptor(String name, String handlerName)
	{
		super();
		this.name = name;
		this.handlerName = handlerName;
	}
	
	/**
	 * @return Returns the handlerName.
	 */
	public String getHandlerName()
	{
		return handlerName;
	}

	/**
	 * @return Returns the name.
	 */
	public String getName()
	{
		return name;
	}

	/**
	 * @param element
	 * @return
	 */
	public static EventDescriptor create(Element element) throws XMLParseException
	{
		EventDescriptor desc = createNoAgent(element);
		desc.agentPathString = element.getAttributeValue(XMLAgentPathTag);
		if(desc.agentPathString == null)
			throw new XMLParseException("no agent path supplied!"); //$NON-NLS-1$
		if(desc.agentPathString.indexOf(".") == -1) //$NON-NLS-1$
			throw new XMLParseException("agent path must be in format \"CAPABILITY_NAME.AGENT_NAME\"!"); //$NON-NLS-1$
		return desc;
	}
	
	public String toTreeNodeString()
	{
		return "event name=\"" + getName() + "\" handler=\"" + getHandlerName() + "\"";  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}

	/**
	 * @return
	 */
	public AgentPath getAgentPath()
	{
		return new AgentPath(AgentPath.LOCAL_NODE_TAG + "." + agentPathString); //$NON-NLS-1$
	}

	/**
	 * @param eventNode
	 * @return
	 */
	public static EventDescriptor createNoAgent(Element element) throws XMLParseException
	{
		String name = element.getAttributeValue(XMLEventNameTag);
		if(name == null)
			throw new XMLParseException("event name not specified!"); //$NON-NLS-1$
		String handler = element.getAttributeValue(XMLEventHandlerTag);
		if(handler == null)
			throw new XMLParseException("handler for event " + name + " not specified!"); //$NON-NLS-1$ //$NON-NLS-2$
		return new EventDescriptor(name, handler);
	}

	/**
	 * @return Returns the agentPathString.
	 */
	public String getAgentPathString()
	{
		return agentPathString;
	}

	/**
	 * @param agentPathString The agentPathString to set.
	 */
	public void setAgentPathString(String agentPathString)
	{
		this.agentPathString = agentPathString;
	}

	/**
	 * @param string
	 */
	public void setApplicationName(String applicationName)
	{
		AssertUtility.assertTrue(StringUtility.isValid(agentPathString));
		agentPathString = applicationName + "." + agentPathString; //$NON-NLS-1$
	}

}
