/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.node;

import org.almendra.commons.util.AssertUtility;
import org.almendra.janet.cas.commands.NodeShutdownRequestCommand;


/**
 * @author Oliver
 *  
 */
public class NodeAnchor extends AbstractNodeAnchor
{
	public static final String REGISTRY_KEY_STARTUP_APPLICATIONS = "StartupApplications"; //$NON-NLS-1$
	public static final String MONITOR_PRODUCER_ANCHOR_LABEL = "MONITOR_PRODUCER_ANCHOR"; //$NON-NLS-1$
	
	// CAS.Monitor attribute definitions
	
	public static final String NODE_NAME = "NODE"; //$NON-NLS-1$
	public static final String NODE_LABEL = "LABEL"; //$NON-NLS-1$	
	
	public static final String MONITOR_MASK_LABEL = "MONITOR_MASK"; //$NON-NLS-1$
	public static final String MONITOR_MASK_DEFAULT_VALUES_LABEL = "MONITOR_MASK_DEFAULT_VALUES"; //$NON-NLS-1$
	
	public static final String LOCAL_COMMANDS_COUNT = "LOCAL COMMANDS"; //$NON-NLS-1$
	public static final String LOCAL_WAITING_COMMANDS_COUNT = "LOCAL WAITING COMMANDS"; //$NON-NLS-1$
	public static final String LOCAL_EXECUTING_COMMANDS_COUNT = "LOCAL EXECUTING COMMANDS"; //$NON-NLS-1$
	public static final String LOCAL_BALANCING_COMMANDS_COUNT = "LOCAL BALANCING COMMANDS"; //$NON-NLS-1$
	public static final String LOCAL_SHARING_COMMANDS_COUNT = "LOCAL BALANCING COMMANDS"; //$NON-NLS-1$
	public static final String LOCAL_EXECUTING_BALANCING_COMMANDS_COUNT = "LOCAL EXECUTING BALANCING COMMANDS"; //$NON-NLS-1$
	public static final String LOCAL_EXECUTING_SHARING_COMMANDS_COUNT = "LOCAL EXECUTING SHARING COMMANDS"; //$NON-NLS-1$		
	
	public static final String FOREIGN_COMMANDS_COUNT = "FOREIGN COMMANDS"; //$NON-NLS-1$
	public static final String FOREIGN_BALANCING_COMMANDS_COUNT = "FOREIGN BALANCING COMMANDS"; //$NON-NLS-1$
	public static final String FOREIGN_SHARING_COMMANDS_COUNT = "FOREIGN SHARING COMMANDS"; //$NON-NLS-1$
	public static final String FOREIGN_WAITING_COMMANDS_COUNT = "FOREIGN WAITING COMMANDS"; //$NON-NLS-1$
	public static final String EVICTED_WAITING_COMMANDS_COUNT = "EVICTED WAITING COMMANDS"; //$NON-NLS-1$
	public static final String FOREIGN_EXECUTING_COMMANDS_COUNT = "FOREIGN EXECUTING COMMANDS"; //$NON-NLS-1$
	public static final String FOREIGN_EXECUTING_BALANCING_COMMANDS_COUNT = "FOREIGN EXECUTING BALANCING COMMANDS"; //$NON-NLS-1$
	public static final String FOREIGN_EXECUTING_SHARING_COMMANDS_COUNT = "FOREIGN EXECUTING SHARING COMMANDS"; //$NON-NLS-1$
	public static final String EVICTED_EXECUTING_COMMANDS_COUNT = "EVICTED EXECUTING COMMANDS"; //$NON-NLS-1$
	public static final String QUEUE_SIZE_CATEGORY = "QSC"; //$NON-NLS-1$
	
	public static final String FOREIGN_TOTAL_COMMANDS_COUNT = "FOREIGN TOTAL COMMANDS"; //$NON-NLS-1$
	public static final String FOREIGN_TOTAL_EXECUTING_COMMANDS_COUNT = "FOREIGN TOTAL EXECUTING COMMANDS"; //$NON-NLS-1$
	
	protected NodeClusterImage clusterImage = new NodeClusterImage();
	
	protected NodeAnchor()
	{
		super();
	}

	/**
	 * @param node
	 */
	public NodeAnchor(Node node)
	{
		super(node);
	}

	/**
	 * @return Returns the clusterImage.
	 */
	public AbstractClusterImage getClusterImage()
	{
		return clusterImage;
	}

	/**
	 * @return Returns the clusterImage.
	 */
	public NodeClusterImage getCentralClusterImage()
	{
		return clusterImage;
	}

	/**
	 * @param nodeName
	 * @param agentDispatcher
	 */
	public void addForeignNode(IAgentDispatcher agentDispatcher, NodeLocation nodeLocation)
	{
		AssertUtility.assertFalse(getName().equalsIgnoreCase(nodeLocation.getNodeName()));
		clusterImage.addNodeImage(agentDispatcher, nodeLocation);
	}

	/**
	 * @return Returns the centralAgentDispatcher.
	 */
	public IAgentDispatcher getCentralDispatcher()
	{
		return clusterImage.getNodeImage(CentralAnchor.NAME).getAgentDispatcher();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.node.AbstractNodeAnchor#disconnectRequest(org.almendra.commons.janet.cas.logger.node.Node)
	 */
	public boolean disconnectRequest(Node node)
	{
		try
		{
			node.getLogger().info(Node.LOG_EVENT_REQUESTING_PERMISSION_TO_SHUT_DOWN);
			NodeShutdownRequestCommand cmd = new NodeShutdownRequestCommand(node.getName());
			node.getSystemApplication().getCentralAgent().accept(cmd);
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			node.getLogger().exception(e);
		}
		return true;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.node.AbstractNodeAnchor#mayDisconnect()
	 */
	public boolean mayDisconnect()
	{
		if(!super.mayDisconnect())
			return false;
		return true;
	}

}
