/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.node;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.almendra.commons.util.collection.MapUtility;
import org.almendra.commons.util.inet.RegistryLocation;
import org.almendra.janet.cas.agents.AgentProxy;
import org.almendra.janet.cas.agents.IAgentProxy;
import org.almendra.janet.cas.agents.MultiAgentProxy;
import org.almendra.janet.cas.agents.SystemAgent;
import org.almendra.janet.cas.commands.RegisterApplicationCommand;
import org.almendra.janet.cas.schedulers.ApplicationScheduler;


/**
 * @author Oliver
 *
 * To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
public class SystemApplication extends AbstractApplication
{
	public static final String NAME = "SYSTEM";  //$NON-NLS-1$
	public static final String CORE_CAPABILITY_NAME = "CORE";  //$NON-NLS-1$
	
	protected RegistryLocation centralRegistryLocation = null;
	protected RegistryLocation localRegistryLocation = null;
	protected List clusterSpaceDescriptors = new ArrayList();
	protected List clusterEventRegistryDescriptors = new ArrayList();
		
	protected SystemApplication()
	{
		super(NAME);
	}

	/**
	 * @param logger
	 * @param node
	 * @param name
	 */
	public SystemApplication(Logger logger, Node node)
	{
		super(logger, node, NAME);
	}

	/**
	 * @return
	 */
	public Capability getCoreCapability()
	{
		return getCapability(CORE_CAPABILITY_NAME);
	}

	/**
	 * @return Returns the centralRegistryLocation.
	 */
	public RegistryLocation getCentralRegistryLocation()
	{
		return centralRegistryLocation;
	}

	/**
	 * @param centralRegistryLocation The centralRegistryLocation to set.
	 */
	public void setCentralRegistryLocation(RegistryLocation registryLocation)
	{
		this.centralRegistryLocation = registryLocation;
	}

	/**
	 * @param foreignNodeName
	 */
	public IAgentProxy getSystemAgent(String foreignNodeName) 
	throws NoSuchAgentException, IllegalAgentAccessException
	{
		return super.getAgent(getSystemAgentPath(foreignNodeName));
	}
	
	/**
	 * @param foreignNodeName
	 */
	public IAgentProxy getSystemAgentsOnWorkstation(String workstationName) 
	throws NoSuchAgentException, IllegalAgentAccessException
	{		
		return getAllSystemAgentsOnNodes(getAllNodeNames(workstationName));
	}
	
	protected List getAllNodeNames(String workstationName)
	{
		List nodeNames = new ArrayList();
		Iterator it = getAllNodeNames().iterator();
		while (it.hasNext())
		{
			String nodeName = (String) it.next();
			String currentWorkstationName = new NodeName(nodeName).getWorkstationName();
			if(currentWorkstationName.equals(workstationName))
				nodeNames.add(nodeName);
		}
		return nodeNames;
	}
	
	/**
	 * @param foreignNodeName
	 */
	public IAgentProxy getSystemAgent() 
	throws NoSuchAgentException, IllegalAgentAccessException
	{
		return super.getAgent(getSystemAgentPath());
	}
	
	public AgentPath getSystemAgentPath()
	{
		return getSystemAgentPath(getNode().getName());
	}

	/**
	 * @return Returns the localRegistryLocation.
	 */
	public RegistryLocation getLocalRegistryLocation()
	{
		return localRegistryLocation;
	}

	/**
	 * @param localRegistryLocation The localRegistryLocation to set.
	 */
	public void setLocalRegistryLocation(RegistryLocation localRegistryLocation)
	{
		this.localRegistryLocation = localRegistryLocation;
	}

	/**
	 * @return
	 */
	public MultiAgentProxy getAllForeignSystemAgents() 
	throws NoSuchAgentException, IllegalAgentAccessException
	{
		return getAllSystemAgentsOnNodes(getAllForeignNodeNames());
	}
	
	/**
	 * @return
	 */
	public MultiAgentProxy getAllForeignSystemAgentsWithCentral() 
	throws NoSuchAgentException, IllegalAgentAccessException
	{
		return getAllSystemAgentsOnNodes(getAllForeignNodeNamesWithCentral());
	}

	public MultiAgentProxy getAllSystemAgentsOnNodes(List nodeNames) 
	throws NoSuchAgentException, IllegalAgentAccessException
	{
		List agentProxies = new ArrayList();
		Iterator it = nodeNames.iterator();
		while (it.hasNext())
		{
			String foreignNodeName = (String) it.next();
			AgentPath agentPath = getSystemAgentPath(foreignNodeName);
			IAgentDispatcher agentDispatcher = getAgentDispatcher(agentPath);
			AgentProxy agentProxy = new AgentProxy(this, agentDispatcher, agentPath, logger);
			agentProxies.add(agentProxy);
		}		
		MultiAgentProxy systemAgentsProxy = new MultiAgentProxy(agentProxies);
		systemAgentsProxy.setOriginNodeName(getNode().getName());
		return systemAgentsProxy;
	}
	
	public MultiAgentProxy getAllSystemAgents() 
	throws NoSuchAgentException, IllegalAgentAccessException
	{
		return getAllSystemAgentsOnNodes(getAllNodeNames());
	}

	public IAgentProxy getCentralAgent()
	{
		AgentPath agentPath = getSystemAgentPath(CentralAnchor.NAME);
		AbstractNodeAnchor anchor = getNode().getSystemAnchor();
		return new AgentProxy(node.getSystemApplication(), anchor.getCentralDispatcher(), agentPath, logger);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.node.AbstractApplication#getNameDescriptor()
	 */
	protected String getNameDescriptor()
	{
		return "systemApplication"; //$NON-NLS-1$
	}

	/**
	 * @return
	 */
	public List getClusterSpaceDescriptors()
	{
		return clusterSpaceDescriptors;
	}

	/**
	 * @param clusterSpaceDescriptors The clusterSpaceDescriptors to set.
	 */
	public void setClusterSpaceDescriptors(List clusterSpaceDescriptors)
	{
		this.clusterSpaceDescriptors = clusterSpaceDescriptors;
	}

	/**
	 * @param list
	 */
	public void setClusterEventRegistryDescriptors(List clusterEventRegistryDescriptors)
	{
		this.clusterEventRegistryDescriptors = clusterEventRegistryDescriptors;
	}

	/**
	 * @return Returns the clusterEventRegistryDescriptors.
	 */
	public List getClusterEventRegistryDescriptors()
	{
		return clusterEventRegistryDescriptors;
	}
	
	public IAgentProxy getSystemAgents(String applicationName)
	{
		List systemAgentPaths = getNode().getClusterImage().getAllForeignSystemAgentPaths(applicationName);
		List systemAgents = new ArrayList();
		Iterator it = systemAgentPaths.iterator();
		while (it.hasNext())
		{
			AgentPath agentPath = (AgentPath) it.next();
			try
			{
				systemAgents.add(getAgent(agentPath));
			} 
			catch (NoSuchAgentException e)
			{
				System.out.println(e.getMessage());
				e.printStackTrace();
				getLogger().exception(e);	// logical error if this happens - do not handle ex
			} 
			catch (IllegalAgentAccessException e)
			{
				System.out.println(e.getMessage());
				e.printStackTrace();
				getLogger().exception(e);	// logical error if this happens - do not handle ex
			}
		}
		return new MultiAgentProxy(systemAgents);
	}

	/**
	 * @param node
	 * @param logger
	 * @return
	 */
	public SystemAgent createAgent(Node node, Logger logger)
	{		
		// TODO - messy workaround, system agent should be created using the same interface as application agents
		SystemAgent systemAgent = new SystemAgent(node, this, logger);
		getCoreCapability().registeredAgentNames.add(SystemAgent.NAME);
		return systemAgent;
	}
	
	public MultiAgentProxy getForeignSystemAgents(List nodeNamesOrNull)
	{
		try
		{
			if (nodeNamesOrNull == null)
				return getAllForeignSystemAgents();
			else
				return getAllSystemAgentsOnNodes(nodeNamesOrNull);
		}
		catch (NoSuchAgentException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			logger.exception(e);
			return null;
		}
		catch (IllegalAgentAccessException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			logger.exception(e);
			return null;
		}
	}

	/**
	 * @return
	 */
	public List getNonCoreCapabilities()
	{
		List nonCoreCapabilities = new LinkedList();
		Iterator it = getCapabilities().iterator();
		while (it.hasNext())
		{
			Capability cap = (Capability) it.next();
			if(!CORE_CAPABILITY_NAME.equals(cap.getName()))
				nonCoreCapabilities.add(cap);
		}
		return nonCoreCapabilities;
	}
	
	/**
	 * @return
	 */
	public boolean hasNonCoreCapabilities()
	{
		return !getNonCoreCapabilities().isEmpty();
	}
	
	protected void propagateRegistered(List nodeNamesOrNull) throws RemoteException
	{
		if(!hasNonCoreCapabilities())
			return;
		
		startNonCoreAgents();		
		registerNonCoreCapabilities(nodeNamesOrNull);
		primStartInitialCommands();
	}

	/**
	 * @param nodeNamesOrNull
	 * @throws RemoteException
	 */
	protected void registerNonCoreCapabilities(List nodeNamesOrNull) throws RemoteException
	{
		MultiAgentProxy agents = getForeignSystemAgents(nodeNamesOrNull);
		if(agents == null)
			return;
		
		try
		{
			RegisterApplicationCommand cmd = new RegisterApplicationCommand(getNodeName(), toForeignDescriptor(), node.getNodeLocation());
			agents.accept(cmd);
		}
		catch (InvalidCommandException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			logger.exception(e);
		}
	}

	/**
	 * 
	 */
	protected void startNonCoreAgents()
	{
		Iterator it = getNonCoreCapabilities().iterator();
		while (it.hasNext())
		{
			Capability cap = (Capability) it.next();			
			Iterator innerIt = cap.getAgentNames().iterator();
			while (innerIt.hasNext())
			{
				String agentName = (String) innerIt.next();
				ApplicationScheduler scheduler = cap.createApplicationScheduler();
				cap.createAgent(agentName, scheduler);
			}			
		}		
	}
	
	public List getNonCoreAgents()
	{
		List nonCoreAgents = new LinkedList();
		Iterator it = getNonCoreCapabilities().iterator();
		while (it.hasNext())
		{
			Capability cap = (Capability) it.next();
			Iterator innerIt = cap.getRegisteredAgentNames().iterator();
			while (innerIt.hasNext())
			{
				String agentName = (String) innerIt.next();
				AgentPath agentPath = new AgentPath(node.getName(), getName(), cap.getName(), agentName);
				nonCoreAgents.add(agentPath);
			}
		}
		return nonCoreAgents;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.node.AbstractApplication#getNumberOfSchedulers()
	 */
	public int getNumberOfSchedulers()
	{
		return getNonCoreCapabilities().size();
	}

	/**
	 * @return
	 */
	public boolean isSystemApplication()
	{
		return true;
	}

	/**
	 * @param agentPaths
	 * @return
	 */
	public String getAgentNameShortestQueue(List agentPaths)
	{
		Map queueSizeByAgentPath = new Hashtable();
		Iterator it = agentPaths.iterator();
		while (it.hasNext())
		{						
			try
			{
				AgentPath agentPath = (AgentPath) it.next();
				IAgentProxy agent = getCrossAgent(agentPath);
				queueSizeByAgentPath.put(agentPath, new Integer(agent.size()));				
			}
			catch (Exception e)
			{
				System.out.println(e.getMessage());
				e.printStackTrace();
				getLogger().exception(e);
			}			
		}
		if(queueSizeByAgentPath.isEmpty())
			return null;
		AgentPath agentPathSmallestQueue = (AgentPath) MapUtility.getKeyWithSmallestIntegerValue(queueSizeByAgentPath);
		return agentPathSmallestQueue.getAgentName();		
	}

	/**
	 * @param nodeName
	 * @return
	 */
	public List getAllApplicationCapabilitesOn(String nodeName)
	{
		NodeAnchor anchor = (NodeAnchor) getNode().getSystemAnchor();
		return anchor.getClusterImage().getAllApplicationCapabilitesOn(nodeName);
	}

	/**
	 * @param localCapabilityPaths
	 * @param nodeNames
	 * @return
	 */
	public List existingCapabilitiesOnOtherNodes(List localCapabilityPaths, List excludingNodeNames)
	{
		NodeAnchor anchor = (NodeAnchor) getNode().getSystemAnchor();
		return anchor.getClusterImage().existingCapabilitiesOnOtherNodes(localCapabilityPaths, excludingNodeNames);
	}

	/**
	 * @param nodeName
	 * @param applicationName
	 * @return
	 */
	public boolean hasApplication(String nodeName, String applicationName)
	{
		NodeAnchor anchor = (NodeAnchor) getNode().getSystemAnchor();
		return anchor.getClusterImage().hasApplication(nodeName, applicationName);
	}
}
