/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.node.gui;

import java.util.Arrays;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import org.almendra.commons.util.StringUtility;
import org.almendra.commons.util.collection.TreeNode;
import org.almendra.commons.util.logging.ILogListener;
import org.almendra.commons.util.logging.LogEntry;
import org.almendra.commons.util.swing.BillboardPanelController;
import org.almendra.commons.util.swing.TextAreaPanelController;
import org.almendra.janet.cas.events.EventHandlerEnvelope;
import org.almendra.janet.cas.events.HandlerList;
import org.almendra.janet.cas.events.ILocalEventRegistry;
import org.almendra.janet.cas.handlers.ApplicationDeregisteredParams;
import org.almendra.janet.cas.handlers.ApplicationRegisteredParams;
import org.almendra.janet.cas.handlers.ForeignApplicationRegisteredParams;
import org.almendra.janet.cas.handlers.NodeDeregisteredParams;
import org.almendra.janet.cas.handlers.NodeShutdownParams;
import org.almendra.janet.cas.node.AbstractApplication;
import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.node.ApplicationAccessor;
import org.almendra.janet.cas.node.LocalAgentPath;
import org.almendra.janet.cas.node.Node;


/**
 * @author Oliver
 *
 */
public class NodeViewController
{
	protected NodeView view = null;
	protected Node node = null;
	protected DefaultTreeModel applicationsRootNode = null;
	protected DefaultMutableTreeNode applicationsNode = null;
	protected DefaultMutableTreeNode localAppsNode = null;
	protected DefaultTreeModel agentsRootNode = null;
	protected DefaultMutableTreeNode agentsNode = null;
	protected DefaultMutableTreeNode localAgentsNode = null;
	protected ILogListener logListener = null;
	protected int[] logListSelectionIndices = new int[0];
	protected Map logEntriesByLogText = new Hashtable();
	protected Map nodeByTreeNode = new Hashtable();
	protected Map appsByTreeNode = new Hashtable();
	protected NodeViewsRegistry nodeViewsRegistry = null; 
	
	/**
	 * Creates a new <code>NodeViewController</code> object.
	 */
	public NodeViewController(NodeView view, Node node)
	{
		super();
		this.view = view;
		this.node = node;
		initialize();		
	}
	
	/**
	 * 
	 */
	protected void initialize()
	{
		initLogger();
		initEvents();
		initBillboard();
		initFreeTextArea();
		initNodeViewTitleAccessor();
		initViewsTab();
	}

	/**
	 * 
	 */
	protected void initViewsTab()
	{
		nodeViewsRegistry = new NodeViewsRegistry(this, view);
		node.getNodeObjectSpace().put(NodeViewsRegistry.LABEL, nodeViewsRegistry);
	}

	/**
	 * 
	 */
	protected void initNodeViewTitleAccessor()
	{
		node.getSystemApplication().getObjectSpace().put(NodeViewTitleAccessor.LABEL, new NodeViewTitleAccessor(view));
	}

	/**
	 * 
	 */
	protected void initFreeTextArea()
	{
		node.getNodeObjectSpace().put(
			TextAreaPanelController.LABEL, 
			view.getFreeTextController(node.getNodeDescriptor().getClearTextTabAfterLines()));
	}

	/**
	 * 
	 */
	protected void initBillboard()
	{
		node.getNodeObjectSpace().put(BillboardPanelController.LABEL, view.getBillboardController());
	}

	/**
	 * 
	 */
	protected void initLogger()
	{
		logListener = new LogListener(this);
		node.getLogger().addListener(logListener);
	}

	protected void initPanels()
	{
		initAgentsPanel();
		initApplicationsPanel();
	}
	
	protected void initAgentsPanel()
	{
		agentsNode = new DefaultMutableTreeNode("agents"); //$NON-NLS-1$
		agentsRootNode = new DefaultTreeModel(agentsNode); //$NON-NLS-1$		
		
		localAgentsNode = new DefaultMutableTreeNode(getNode().getName() + " (local node)"); //$NON-NLS-1$
		agentsNode.add(localAgentsNode);
		
		Iterator it = node.getApplicationRegistry().getAllApplications().iterator();
		while (it.hasNext())
		{
			AbstractApplication app = (AbstractApplication) it.next();
			Iterator pathIt = app.getAllAgentPaths().iterator();
			while (pathIt.hasNext())
			{
				AgentPath agentPath = (AgentPath) pathIt.next();
				addAgent(agentPath);	
			}			
		}
		
		view.getJAgentsTree().setModel(agentsRootNode);
	}
	
	protected void addAgent(AgentPath agentPath)
	{
		String nodeName = agentPath.getNodeName();
		if(node.getName().equals(nodeName))
		{
			localAgentsNode.add(new DefaultMutableTreeNode(agentPath.getLocalAgentName()));
			return;
		}
		DefaultMutableTreeNode nodeAgentNode = getNodeAgentNode(agentPath);
		nodeAgentNode.add(new DefaultMutableTreeNode(agentPath.getLocalAgentName()));
	}
	
	protected DefaultMutableTreeNode getNodeAgentNode(AgentPath agentPath)
	{
		String nodeName = agentPath.getNodeName();
		for (int i = 0; i < agentsNode.getChildCount(); i++)
		{
			DefaultMutableTreeNode nodeAgentNode = (DefaultMutableTreeNode) agentsNode.getChildAt(i);
			if(nodeAgentNode.getUserObject().equals(nodeName))
				return nodeAgentNode;
		}
		DefaultMutableTreeNode nodeAgentNode = new DefaultMutableTreeNode(agentPath.getNodeName());
		agentsNode.add(nodeAgentNode);
		return nodeAgentNode;
	}

	protected void initApplicationsPanel()
	{
		applicationsNode = new DefaultMutableTreeNode("applications"); //$NON-NLS-1$
		applicationsRootNode = new DefaultTreeModel(applicationsNode); //$NON-NLS-1$
		localAppsNode = new DefaultMutableTreeNode(getNode().getName() + " (local node)"); //$NON-NLS-1$
		applicationsNode.add(localAppsNode);
		
		Iterator it = node.getApplicationRegistry().getTreeDescriptors().iterator();
		while (it.hasNext())
		{
			TreeNode appNode = (TreeNode) it.next();
			addApplication(appNode);
		}
		
		view.getJApplicationsTree().setModel(applicationsRootNode);
	}

	/**
	 * @return Returns the node.
	 */
	protected Node getNode()
	{
		return node;
	}

	/**
	 * 
	 */
	public void initEvents()
	{
		ApplicationAccessor app = node.getSystemApplication().getAccessor();
		ILocalEventRegistry eventRegistry = node.getNodeEventRegistry();
		AgentPath agentPath = node.getSystemApplication().getSystemAgentPath();			
		EventHandlerEnvelope item = null;
		
		item = new EventHandlerEnvelope(new ApplicationRegisteredHandler(this), agentPath);		
		eventRegistry.register(ApplicationRegisteredParams.EVENT_NAME, new HandlerList(item), app);
		
		item = new EventHandlerEnvelope(new ForeignApplicationRegisteredHandler(this), agentPath);
		eventRegistry.register(ForeignApplicationRegisteredParams.EVENT_NAME, new HandlerList(item), app);
		
		item = new EventHandlerEnvelope(new NodeDeregisteredHandler(this), agentPath);
		eventRegistry.register(NodeDeregisteredParams.EVENT_NAME, new HandlerList(item), app);
		
		item = new EventHandlerEnvelope(new ApplicationDeregisteredHandler(this), agentPath);
		eventRegistry.register(ApplicationDeregisteredParams.EVENT_NAME, new HandlerList(item), app);
		
		item = new EventHandlerEnvelope(new NodeShutdownHandler(this, logListener), agentPath);
		eventRegistry.register(NodeShutdownParams.EVENT_NAME, new HandlerList(item), app);
	}

	/**
	 * @param application
	 */
	public void addApplication(AbstractApplication application)
	{
		addApplication(application.toTreeDescriptor());
		addAgents(application);
	}
	
	/**
	 * @param application
	 */
	protected void addAgents(AbstractApplication application)
	{
		Iterator it = application.getAllAgentPaths().iterator();
		while (it.hasNext())
		{
			AgentPath agentPath = (AgentPath) it.next();
			addAgent(agentPath);	
		}		
		view.getJAgentsTree().setModel(null);
		view.getJAgentsTree().setModel(agentsRootNode);
	}

	/**
	 * @param application
	 */
	public void addApplication(TreeNode appNode)
	{
		TreeNodeVisitor visitor = new TreeNodeVisitor();
		appNode.visit(visitor);
		localAppsNode.add(visitor.getRootNode());
		appsByTreeNode.put(visitor.getRootNode(), appNode.getValue());
		applicationsRootNode.nodeChanged(localAppsNode);
	}

	/**
	 * @param agentPaths
	 */
	public void addForeignAgentPaths(List agentPaths)
	{
		Iterator it = agentPaths.iterator();
		while (it.hasNext())
		{
			AgentPath agentPath = (AgentPath) it.next();
			addApplication(agentPath);
			addAgent(agentPath);
		}
		view.getJApplicationsTree().setModel(null);
		view.getJApplicationsTree().setModel(applicationsRootNode);
		
		view.getJAgentsTree().setModel(null);
		view.getJAgentsTree().setModel(agentsRootNode);
	}

	protected void addApplication(AgentPath agentPath)
	{
		DefaultMutableTreeNode treeNodeOfNode = getAppParentNode(agentPath.getNodeName());
		DefaultMutableTreeNode previousNode = treeNodeOfNode;
		
		String label = "application name = " + StringUtility.print(agentPath.getApplicationName()); //$NON-NLS-1$			
		DefaultMutableTreeNode nextNode = insert(previousNode, label);
		
		label = "capabilities"; //$NON-NLS-1$
		previousNode = nextNode;
		nextNode = insert(previousNode, label);
		
		label = "capability name = " + StringUtility.print(agentPath.getCapabilityName()); //$NON-NLS-1$
		previousNode = nextNode;
		nextNode = insert(previousNode, label);
					
		label = "agents"; //$NON-NLS-1$
		previousNode = nextNode;
		nextNode = insert(previousNode, label);
		
		label = agentPath.getAgentName();
		previousNode = nextNode;
		nextNode = insert(previousNode, label);								
	}

	protected DefaultMutableTreeNode insert(DefaultMutableTreeNode parent, String newNodeValue)
	{
		for (int i = 0; i < parent.getChildCount(); i++)
		{
			DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode) parent.getChildAt(i);
			String value = (String) currentNode.getUserObject();
			if(newNodeValue.equals(value))
				return currentNode;
		}
		
		DefaultMutableTreeNode newNode = new DefaultMutableTreeNode(newNodeValue);
		parent.add(newNode);
		return newNode;
	}
	
	protected DefaultMutableTreeNode getAppParentNode(String nodeName)
	{
		DefaultMutableTreeNode treeNodeOfNode = null;
		if(nodeByTreeNode.containsKey(nodeName))
		{
			treeNodeOfNode = (DefaultMutableTreeNode) nodeByTreeNode.get(nodeName);
		}
		else
		{
			treeNodeOfNode = new DefaultMutableTreeNode(nodeName);
			nodeByTreeNode.put(nodeName, treeNodeOfNode);
			applicationsNode.add(treeNodeOfNode);
			applicationsRootNode.nodeChanged(applicationsNode);
		}
		return treeNodeOfNode;
	}

	/**
	 * @param string
	 */
	public void nodeDeregistered(String nodeName)
	{
		removeApplicationNode(nodeName);
		removeAgentsNode(nodeName);
	}

	protected void removeApplicationNode(String nodeName)
	{
		for (int i = 0; i < applicationsNode.getChildCount(); i++)
		{
			DefaultMutableTreeNode node = (DefaultMutableTreeNode) applicationsNode.getChildAt(i);
			if(node.getUserObject().equals(nodeName))
			{
				applicationsNode.remove(node);
				nodeByTreeNode.remove(nodeName);
				applicationsRootNode.nodeStructureChanged(applicationsNode);
				break;
			}
		}
	}

	protected void removeAgentsNode(String nodeName)
	{
		for (int i = 0; i < agentsNode.getChildCount(); i++)
		{
			DefaultMutableTreeNode node = (DefaultMutableTreeNode) agentsNode.getChildAt(i);
			if(node.getUserObject().equals(nodeName))
			{
				agentsNode.remove(node);
				agentsRootNode.nodeStructureChanged(agentsNode);
				break;
			}
		}
	}
	
	protected void removeAgents(String applicationName, String nodeName)
	{
		for (int i = 0; i < agentsNode.getChildCount(); i++)
		{
			DefaultMutableTreeNode node = (DefaultMutableTreeNode) agentsNode.getChildAt(i);
			if(node.getUserObject().equals(nodeName))
			{
				for (int j = 0; j < node.getChildCount(); j++)
				{
					DefaultMutableTreeNode agentNode = (DefaultMutableTreeNode) node.getChildAt(j);
					String qualifiedAgentName = (String) agentNode.getUserObject();
					if(new LocalAgentPath(qualifiedAgentName).getApplicationName().equals(applicationName))
					{
						node.remove(agentNode);
						agentsRootNode.nodeStructureChanged(node);
					}
				}
			}
		}
	}
	
	protected void removeApplication(String applicationName, String nodeName)
	{
		for (int i = 0; i < applicationsNode.getChildCount(); i++)
		{
			DefaultMutableTreeNode node = (DefaultMutableTreeNode) applicationsNode.getChildAt(i);
			if(node.getUserObject().equals(nodeName))
			{
				for (int j = 0; j < node.getChildCount(); j++)
				{
					DefaultMutableTreeNode applicationNode = (DefaultMutableTreeNode) node.getChildAt(j);
					String applicationLabel = (String) applicationNode.getUserObject();
					if(StringUtility.contains(applicationLabel, applicationName))
					{
						node.remove(applicationNode);
						applicationsRootNode.nodeStructureChanged(node);
						break;	
					}
				}
			}
		}
	}
	
	public void applicationDeregistered(String applicationName, String nodeName)
	{
		removeAgents(applicationName, nodeName);
		removeApplication(applicationName, nodeName);
		
	}

	public boolean shutdown()
	{
		return getNode().getAccessor().shutdown();		
	}

	/**
	 * @param listSelectionEvent
	 */
	public void logListValueChanged(ListSelectionEvent listSelectionEvent)
	{
		JList logList = (JList) listSelectionEvent.getSource();
		int[] currentLogListSelectionIndices = logList.getSelectedIndices();
		if(Arrays.equals(logListSelectionIndices, currentLogListSelectionIndices))
			return;
		logListSelectionIndices = currentLogListSelectionIndices;
		view.getJLogEntryTextPane().setText(""); //$NON-NLS-1$
		Object[] values = logList.getSelectedValues();
		if(values == null || values.length == 0)			
			return;
		StringBuffer expandedXMLString = new StringBuffer();
		Iterator it = Arrays.asList(values).iterator();
		while (it.hasNext())
		{
			String entryString = (String) it.next();
			org.almendra.janet.cas.node.LogEntry entry = (org.almendra.janet.cas.node.LogEntry) logEntriesByLogText.get(entryString);
			expandedXMLString.append(entry.toExpandedXMLString());
			if(it.hasNext())
				expandedXMLString.append("\n"); //$NON-NLS-1$
		}
		view.getJLogEntryTextPane().setText(expandedXMLString.toString());
	}

	/**
	 * @param entry
	 */
	public void log(LogEntry entry)
	{
		DefaultListModel model = (DefaultListModel) view.getJLogList().getModel();
		logEntriesByLogText.put(entry.getLogMessage(), entry);
		model.insertElementAt(entry.getLogMessage(), 0);
	}

	/**
	 * 
	 */
	public void closeView()
	{
		view.dispose();
	}
	
	protected void viewExitRequest()
	{
		if(shutdown())
			view.dispose();
	}

	/**
	 * 
	 */
	public void openButtonClicked()
	{
		Object[] values = view.getJListViews().getSelectedValues();
		if(values.length == 0)
		{
			JOptionPane.showMessageDialog(view, "No item selected!"); //$NON-NLS-1$
			return;
		}
		for (int i = 0; i < values.length; i++)
		{
			String value = (String) values[i];
			nodeViewsRegistry.open(value);
		}
	}
}
