/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.node.interpreters;

import java.util.Iterator;

import org.almendra.commons.util.collection.StringVector;
import org.almendra.commons.util.exception.InsufficientPrivilegeException;
import org.almendra.commons.util.inet.RMIUtility;
import org.almendra.commons.util.inet.RegistryLocation;
import org.almendra.janet.cas.commands.NodeShutdownCommand;
import org.almendra.janet.cas.events.IClusterEventRegistry;
import org.almendra.janet.cas.node.Node;
import org.almendra.janet.cas.node.NodeDescriptor;
import org.almendra.janet.cas.scheduling.CommandAccessor;


/**
 * @author Oliver
 *
 */
public class NodeShutdownInterpreter extends NodeInterpreter
{

	/**
	 * Creates a new <code>NodeShutdownInterpreter</code> object.
	 */
	public NodeShutdownInterpreter()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.schedulers.IInterpreter#execute(org.almendra.commons.janet.cas.logger.schedulers.CommandAccessor)
	 */
	public void execute(CommandAccessor cmdAccessor)
	{
		if(cmdAccessor.getCommand() instanceof NodeShutdownCommand)
		{
			try
			{
				Node node = cmdAccessor.getNode();
				cmdAccessor.getLogger().info(Node.LOG_EVENT_RECEIVED_PERMISSION_TO_SHUT_DOWN);
				cmdAccessor.getLogger().info(Node.LOG_EVENT_ABOUT_TO_DISCONNECT_FROM_ALL_NODES);
				node.disconnectFromNodesWithoutCentral();
				node.disconnectFromCentral();
				unbindClusterSubobjects(cmdAccessor);
				cmdAccessor.getLogger().info(Node.LOG_EVENT_DISCONNECTED_FROM_ALL_NODES);
				node.stop();
			}
			catch (InsufficientPrivilegeException e)
			{
				System.out.println(e.getMessage());
				e.printStackTrace();
			}											
		}
	}
	
	/**
	 * @param cmdAccessor
	 */
	public static void unbindClusterSubobjects(CommandAccessor cmdAccessor)
	{		
		try
		{			
			String nodeName = cmdAccessor.getNode().getName();
			Iterator it = cmdAccessor.getNode().getClusterEventRegistries().iterator();
			while (it.hasNext())
			{
				IClusterEventRegistry remoteRegistry = (IClusterEventRegistry) it.next();
				remoteRegistry.deregister(nodeName);
			}
			if(!cmdAccessor.getNode().isLastNodeOnWorkstation())
				return;
			RegistryLocation loc = cmdAccessor.getSystemApplication().getCentralRegistryLocation();
			StringVector unboundObjects = RMIUtility.unbind(loc, NodeDescriptor.getClusterSubobjectPrefixes());
			// TODO - write unbound objects to the logger, useful for verification
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			cmdAccessor.getLogger().exception(e);
		}
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#commandNames()
	 */
	public StringVector commandNames()
	{
		return new StringVector(NodeShutdownCommand.QualifiedName);
	}

}
