/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.node.interpreters;

import org.almendra.commons.util.collection.StringVector;
import org.almendra.janet.cas.commands.RegisterNodeCommand;
import org.almendra.janet.cas.node.NodeLocation;
import org.almendra.janet.cas.scheduling.CommandAccessor;


/**
 * @author Oliver
 *
 */
public class RegisterNodeInterpreter extends NodeInterpreter
{

	protected AbstractRegisterNodeTrait registerNodeTrait = createRegisterNodeTrait();
	protected RegisterApplicationTrait registerApplicationTrait = new RegisterApplicationTrait(); 
	
	/**
	 * Creates a new <code>RegisterNodeInterpreter</code> object.
	 */
	public RegisterNodeInterpreter()
	{
		super();
	}
	
	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.commands.RegisteredNodeInterpreter#createRegisterNodeTrait()
	 */
	protected AbstractRegisterNodeTrait createRegisterNodeTrait()
	{
		return new RegisterNodeTrait();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.schedulers.IInterpreter#execute(org.almendra.commons.janet.cas.logger.schedulers.CommandAccessor)
	 */
	public void execute(CommandAccessor cmdAccessor)
	{
		if(cmdAccessor.getCommand() instanceof RegisterNodeCommand)
		{
			RegisterNodeCommand command = (RegisterNodeCommand) cmdAccessor.getCommand();
			String foreignNodeName = command.getNodeLocation().getNodeName();
			
			try
			{
				registerNodeTrait.registerForeignNode(command.getNodeLocation(), cmdAccessor);
				registerNodeTrait.sendEventNodeAboutToRegister(cmdAccessor, foreignNodeName);
				NodeLocation nodeLocation = cmdAccessor.getNodeIfPrivilege().getNodeLocation();
				registerApplicationTrait.foreignNodeRegisterMyApplications(foreignNodeName, nodeLocation, cmdAccessor);
			}
			catch (Exception e)
			{
				System.out.println(e.getMessage());
				e.printStackTrace();
				cmdAccessor.log(e);
			}
		}	
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#commandNames()
	 */
	public StringVector commandNames()
	{
		return new StringVector(RegisterNodeCommand.QualifiedName);
	}

}
