/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.schedulers;

import java.math.BigInteger;

import org.almendra.commons.threads.IExecutable;
import org.almendra.janet.cas.node.AbstractApplication;
import org.almendra.janet.cas.node.Capability;
import org.almendra.janet.cas.node.Logger;


/**
 * @author Oliver
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
public abstract class ArbitratedScheduler extends PriorityCommandScheduler implements IExecutable
{
	public static final int DEFAULT_PRIORITIES = 3;
	public static final int STATE_NEW = 1;
	public static final int STATE_SLEEPING = 2;
	public static final int STATE_SCHEDULED = 3;
	public static final int STATE_WAITING = 4;
	
	protected static BigInteger Count = BigInteger.ZERO;
	
	protected Arbitrator arbitrator = null;
	protected AbstractApplication application = null;
	protected Capability capability = null;
	protected int state = STATE_NEW;
	
	/**
	 * 
	 */
	public ArbitratedScheduler()
	{
		super();
	}

	/**
	 * @param priority
	 * @param verbose
	 */
	public ArbitratedScheduler(int priority, boolean verbose)
	{
		super(priority, verbose);
	}

	/**
	 * @param logger
	 */
	public ArbitratedScheduler(Logger logger)
	{
		super(logger);
	}

	/**
	 * @param logger
	 * @param priority
	 */
	public ArbitratedScheduler(Logger logger, int priority)
	{
		super(logger, priority);
	}

	/**
	 * @param logger
	 * @param priority
	 * @param verbose
	 */
	public ArbitratedScheduler(Logger logger, int priority, boolean verbose)
	{
		super(logger, priority, verbose);
	}

	/**
	 * @param numPriorities
	 */
	public ArbitratedScheduler(int numPriorities)
	{
		super(numPriorities);
	}
	
	/**
	 * @param node
	 * @param arbitrator
	 * @param verbose
	 * @param numPriorities
	 */
	public ArbitratedScheduler(
		Arbitrator arbitrator,
		AbstractApplication application,
		Capability capability, 
		Logger logger,
		boolean verbose,
		int numPriorities)
	{
		super(numPriorities, verbose);
		this.arbitrator = arbitrator;
		this.application = application;
		this.capability = capability;
		this.node = application.getNode();
		setLogger(logger);
		justCreated();
	}

	protected void justCreated()
	{
		state = STATE_NEW;
		arbitrator.created(this);
		Count = Count.add(BigInteger.ONE);
		setName(getLoggerObjectName() + ":" + Count); //$NON-NLS-1$
		logger.getDefaultValues().setSchedulerName(getName());
	}
	
	protected abstract String getLoggerObjectName();

	protected void commandAdded()
	{
		arbitrator.commandAddedTo(this);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.threads.ActiveSchedulingObject#objectAdded()
	 */
	protected void executableAdded(IExecutable executable)
	{
		commandAdded();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.schedulers.CommandScheduler#commandFinished(java.lang.Object)
	 */
	protected void finishedExecution(Object object)
	{
		if(isIdle())
			arbitrator.isIdle(this);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.schedulers.CommandScheduler#getApplicationName()
	 */
	protected String getApplicationName()
	{
		return application.getName();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.schedulers.CommandScheduler#getCapabilityName()
	 */
	protected String getCapabilityName()
	{
		return capability.getName();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.schedulers.CommandScheduler#getCurrentAgentName()
	 */
	protected String getCurrentAgentName()
	{
		
		return capability.getAgentNamesString();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.schedulers.CommandScheduler#getLogName()
	 */
	protected String getLogName()
	{
		return getName();
	}

	public boolean isRunning()
	{
		return state == STATE_WAITING || state == STATE_SCHEDULED;
	}

	public boolean isSleeping()
	{
		return state == STATE_NEW || state == STATE_SLEEPING;
	}

	public boolean isWaiting()
	{
		return state == STATE_WAITING;
	}

	public void setNew()
	{
		state = STATE_NEW;
	}

	public void setScheduled()
	{
		state = STATE_SCHEDULED;
	}

	public void setSleeping()
	{
		state = STATE_SLEEPING;
	}

	public void setWaiting()
	{
		state = STATE_WAITING;
	}

	protected void setLogger(Logger logger)
	{
		super.setLogger(logger);
		logger.getDefaultValues().setObjectName(getLoggerObjectName());
	}
	
	public abstract int getNumberOfSchedulersInSameApplication();
	
	public abstract int getMaxPriority();
}
