/*
 * Created on 18.09.2003
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
 
package org.almendra.janet.cas.schedulers;

import java.util.Iterator;
import java.util.LinkedList;

import org.almendra.commons.util.AssertUtility;
import org.almendra.commons.util.collection.IQueue;

import EDU.oswego.cs.dl.util.concurrent.ReentrantWriterPreferenceReadWriteLock;


import junit.framework.Assert;

/**
 * 
 * @author Oliver Plohmann
 */
public class CommandQueue implements IQueue
{
	protected LinkedList commands = new LinkedList();
	protected ReentrantWriterPreferenceReadWriteLock lock = new ReentrantWriterPreferenceReadWriteLock();
	
	public CommandQueue()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.scheduling.SharedQueue#getNextPrim()
	 */
	public Object getNext()
	{
		try
		{
			lock.writeLock().acquire();
			return commands.removeFirst();
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			lock.writeLock().release();
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.util.collection.IQueue#add(java.lang.Object)
	 */
	public boolean add(Object object)
	{
		try
		{
			lock.writeLock().acquire();
			commands.addLast(object);
			return true;
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			lock.writeLock().release();
		}
		return false;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.util.collection.IQueue#isEmpty()
	 */
	public boolean isEmpty()
	{
		try
		{
			lock.readLock().acquire();
			return commands.isEmpty();
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			lock.readLock().release();
		}
		return true;		
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.util.collection.IQueue#size()
	 */
	public int size()
	{
		try
		{
			lock.readLock().acquire();
			return commands.size();
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			lock.readLock().release();
		}
		return 0;		
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.cas.node.scheduling.SharedQueue#removePrim(java.lang.Object)
	 */
	public Object remove(Object object)
	{
		try
		{
			lock.writeLock().acquire();
			if(commands.remove(object))
				return object;
			AssertUtility.assertTrue("object " + object + " not in list", false); //$NON-NLS-1$ //$NON-NLS-2$
			return null;
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			lock.writeLock().release();
		}
		return null;		
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.util.collection.IQueue#hasNext()
	 */
	public boolean hasNext()
	{
		try
		{
			lock.readLock().acquire();
			return size() > 0;
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			lock.readLock().release();
		}
		return false;		
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.util.collection.IQueue#iterator()
	 */
	public Iterator iterator()
	{
		try
		{
			lock.readLock().acquire();
			return commands.iterator();
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			lock.readLock().release();
		}
		return null;		
	}

}
