/*
 * Created on 18.09.2003
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
 
package org.almendra.janet.cas.schedulers;

import org.almendra.janet.cas.agents.SystemAgent;
import org.almendra.janet.cas.node.Logger;
import org.almendra.janet.cas.node.SchedulerAnchor;
import org.almendra.janet.cas.node.SystemApplication;


/**
 * 
 * @author Oliver Plohmann
 */
public class SupremeScheduler extends PriorityCommandScheduler
{
	public static final String SYSTEM_SCHEDULER_NAME = "SYSTEM_SCHEDULER"; //$NON-NLS-1$
	public static final String LOGGER_OBJECT_NAME = "SYSTEM_SCHEDULER"; //$NON-NLS-1$
	
	/**
	 * @param schedulerAnchor
	 * @param logger
	 */
	public SupremeScheduler(SchedulerAnchor scheduler, Logger logger)
	{
		super(logger);
		node = scheduler.getNode();
		this.scheduler = scheduler;
		init();
	}

	protected SchedulerAnchor scheduler = null;
	
	protected SupremeScheduler()
	{
		super();
		init();
	}

	/**
	 * @param numPriorities
	 */
	public SupremeScheduler(SchedulerAnchor scheduler, Logger logger, int numPriorities)
	{
		super(logger, numPriorities);
		this.scheduler = scheduler;
		init();
	}

	/**
	 * @param node
	 * @param verbose
	 * @param numPriorities
	 */
	public SupremeScheduler(SchedulerAnchor scheduler, Logger logger, boolean verbose, int numPriorities)
	{
		super(logger, numPriorities, verbose);
		this.scheduler = scheduler;
		init();
	}
	
	protected void setLogger(Logger parentLogger)
	{
		super.setLogger(parentLogger);
		logger.getDefaultValues().setSchedulerName(LOGGER_OBJECT_NAME);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.cas.node.scheduling.CommandScheduler#init()
	 */
	protected void init()
	{
		super.init();
		setPriority(SchedulerAnchor.SUPREME_PRIORITY);
		setName(SYSTEM_SCHEDULER_NAME);
	}

	/**
	 * @return
	 */
	public long numOfCommands()
	{
		return available();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.schedulers.CommandScheduler#getCapabilityName()
	 */
	protected String getCapabilityName()
	{
		return SystemApplication.CORE_CAPABILITY_NAME;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.schedulers.CommandScheduler#getApplicationName()
	 */
	protected String getApplicationName()
	{
		return SystemApplication.NAME;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.schedulers.CommandScheduler#getCurrentAgentName()
	 */
	protected String getCurrentAgentName()
	{
		return SystemAgent.NAME;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.schedulers.CommandScheduler#getLogName()
	 */
	protected String getLogName()
	{
		return SYSTEM_SCHEDULER_NAME;
	}

}
