/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.scheduling;

import java.io.Serializable;
import java.util.GregorianCalendar;

import org.almendra.commons.util.StringUtility;
import org.almendra.commons.util.TimeUtility;
import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.node.LocalAgentPath;


/**
 * @author Oliver
 *
 */
public class CommandHistoryItem implements Serializable
{
	protected String creationNodeName = null;
	protected String commandName = null;
	protected AgentPath agentPath = null;
	protected LocalAgentPath localAgentPath = null;
	protected GregorianCalendar creationTime = null;
	protected GregorianCalendar sendTime = null;
	protected GregorianCalendar arrivalTime = null;
	protected GregorianCalendar startExecutionTime = null;
	protected GregorianCalendar endExecutionTime = null;
	
	/**
	 * @return Returns the originNodeAgentName.
	 */
	public String getNodeName()
	{
		return agentPath.getNodeName();
	}

	/**
	 * @return Returns the commandName.
	 */
	public String getCommandName()
	{
		return commandName;
	}

	/**
	 * @param commandName The commandName to set.
	 */
	public void setCommandName(String commandName)
	{
		this.commandName = commandName;
	}

	/**
	 * @return Returns the originApplicationName.
	 */
	public String getApplicationName()
	{
		return agentPath.getApplicationName();
	}

	/**
	 * @return Returns the capabilityName.
	 */
	public String getCapabilityName()
	{
		return agentPath.getCapabilityName();
	}

	/**
	 * @return Returns the originCreationTime.
	 */
	public GregorianCalendar getCreationTime()
	{
		return creationTime;
	}

	public CommandHistoryItem()
	{
		super();
	}
	
	public static CommandHistoryItem requestItem()
	{
		CommandHistoryItem item = new CommandHistoryItem();
		item.setCreationTime();
		return item;
	}
	
	public static CommandHistoryItem responseItem()
	{
		CommandHistoryItem item = new CommandHistoryItem();
		item.setCreationTime();
		return item;
	}

	public void setCreationTime()
	{
		creationTime = (GregorianCalendar) GregorianCalendar.getInstance();
	}
	
	public void setSendTime()
	{
		sendTime = (GregorianCalendar) GregorianCalendar.getInstance();
	}
	
	public void setArrivalTime()
	{
		arrivalTime = (GregorianCalendar) GregorianCalendar.getInstance();
	}
	
	public void setStartExecutionTime()
	{
		startExecutionTime = (GregorianCalendar) GregorianCalendar.getInstance();
	}
	
	public void setEndExecutionTime()
	{
		endExecutionTime  = (GregorianCalendar) GregorianCalendar.getInstance();
	}
	
	/**
	 * @return Returns the agentPath.
	 */
	public AgentPath getAgentPath()
	{
		return agentPath;
	}

	/**
	 * @param agentPath The agentPath to set.
	 */
	public void setAgentPath(AgentPath agentPath)
	{
		this.agentPath = agentPath;
	}

	/**
	 * @return Returns the localAgentPath.
	 */
	public LocalAgentPath getLocalAgentPath()
	{
		return localAgentPath;
	}

	/**
	 * @param localAgentPath The localAgentPath to set.
	 */
	public void setLocalAgentPath(LocalAgentPath localAgentPath)
	{
		this.localAgentPath = localAgentPath;
	}
	
	public String toString()
	{
		StringBuffer buffer = new StringBuffer();
		buffer.append(super.toString());
		
		buffer.append(" { creationTime="); //$NON-NLS-1$
		if(creationTime == null)
			buffer.append("\"null\""); //$NON-NLS-1$
		else
			buffer.append(StringUtility.print(TimeUtility.toPreattyString(creationTime)));
		
		buffer.append(" sendTime="); //$NON-NLS-1$
		if(creationTime == null)
			buffer.append("\"null\""); //$NON-NLS-1$
		else
			buffer.append(StringUtility.print(TimeUtility.toPreattyString(sendTime)));
		
		buffer.append(" arrivalTime="); //$NON-NLS-1$
		if(creationTime == null)
			buffer.append("\"null\""); //$NON-NLS-1$
		else
			buffer.append(StringUtility.print(TimeUtility.toPreattyString(arrivalTime)));

		buffer.append(" startExecutionTime="); //$NON-NLS-1$
		if(creationTime == null)
			buffer.append("\"null\""); //$NON-NLS-1$
		else
			buffer.append(StringUtility.print(TimeUtility.toPreattyString(startExecutionTime)));
		
		buffer.append(" endExecutionTime="); //$NON-NLS-1$
		if(creationTime == null)
			buffer.append("\"null\""); //$NON-NLS-1$
		else
			buffer.append(StringUtility.print(TimeUtility.toPreattyString(endExecutionTime)));
		
		buffer.append(" }"); //$NON-NLS-1$
		return buffer.toString();
	}

	/**
	 * @return Returns the arrivalTime.
	 */
	public GregorianCalendar getArrivalTime()
	{
		return arrivalTime;
	}

	/**
	 * @return Returns the endExecutionTime.
	 */
	public GregorianCalendar getEndExecutionTime()
	{
		return endExecutionTime;
	}

	/**
	 * @return Returns the sendTime.
	 */
	public GregorianCalendar getSendTime()
	{
		return sendTime;
	}

	/**
	 * @return Returns the startExecutionTime.
	 */
	public GregorianCalendar getStartExecutionTime()
	{
		return startExecutionTime;
	}

	/**
	 * @return Returns the creationNodeName.
	 */
	public String getCreationNodeName()
	{
		return creationNodeName;
	}

	/**
	 * @param creationNodeName The creationNodeName to set.
	 */
	public void setCreationNodeName(String creationNode)
	{
		this.creationNodeName = creationNode;
	}

}
