/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.cas.spaces;

import java.io.Serializable;
import java.rmi.Remote;
import java.rmi.RemoteException;


public interface ISerializableObjectSpace extends Remote
{

	/**
	 * @param key
	 * @param attachment
	 * @param attacher
	 */
	public void attach(String key, Serializable attachment, ISerializableAttacher attacher) throws RemoteException;

	/**
	 * @param key
	 * @return
	 */
	public Serializable remove(String key) throws RemoteException;

	/**
	 * @param key
	 * @param object
	 */
	public void put(String key, Serializable object) throws RemoteException;

	/**
	 * @param key
	 * @param detacher
	 */
	public void installDetacher(String key, ISerializableDetacher detacher) throws RemoteException;

	/**
	 * @param key
	 * @param attacher
	 */
	public void installAttacher(String key, ISerializableAttacher attacher) throws RemoteException;

	/**
	 * @param key
	 * @return
	 */
	public Serializable get(String key) throws RemoteException;

	/**
	 * @param key
	 * @param detacher
	 * @return
	 */
	public Serializable detach(String key, ISerializableDetacher detacher) throws RemoteException;

	/**
	 * @param key
	 * @return
	 */
	public Serializable detach(String key) throws NoInstalledDetacher, RemoteException;

	/**
	 * @param key
	 * @return
	 */
	public ISerializableAttacher removeAttacher(String key) throws RemoteException;

	/**
	 * @param key
	 * @return
	 */
	public ISerializableDetacher removeDetacher(String key) throws RemoteException;

	/**
	 * @param key
	 * @param attachment
	 */
	public void attach(String key, Serializable attachment) throws NoInstalledAttacher, RemoteException;
	
	/**
	 * 
	 * @param key
	 * @return
	 */
	public boolean contains(String key) throws RemoteException;

}
