/*
 * 
 * Copyright (C) 2003, 2004 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE JDOM AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.cas.spaces;

import org.almendra.commons.threads.LockUtility;


public class ObjectSpace extends AbstractObjectSpace implements IObjectSpace
{

	/**
	 * Creates a new <code>ObjectSpace</code> object.
	 */
	public ObjectSpace()
	{
		super();
	}

	/*
	 * (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.spaces.IObjectSpace#removeAttacher(java.lang.String)
	 */
	public IAttacher removeAttacher(String key)
	{
		LockUtility.acquire(lock);
		IAttacher attacher = (IAttacher) attachers.remove(key);
		lock.release();
		return attacher;
	}

	/*
	 * (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.spaces.IObjectSpace#removeDetacher(java.lang.String)
	 */
	public IDetacher removeDetacher(String key)
	{
		LockUtility.acquire(lock);
		IDetacher detacher = (IDetacher) detachers.remove(key);
		lock.release();
		return detacher;
	}

	/*
	 * (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.spaces.IObjectSpace#get(java.lang.String)
	 */
	public Object get(String key)
	{
		LockUtility.acquire(lock);
		Object object = objects.get(key);
		lock.release();
		return object;
	}

	/*
	 * (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.spaces.IObjectSpace#remove(java.lang.String)
	 */
	public Object remove(String key)
	{
		LockUtility.acquire(lock);
		Object object = objects.remove(key);
		lock.release();
		return object;
	}

	/*
	 * (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.spaces.IObjectSpace#detach(java.lang.String)
	 */
	public Object detach(String key) throws NoInstalledDetacher
	{
		LockUtility.acquire(lock);
		Object object = objects.get(key);
		IDetacher detacher = (IDetacher) detachers.get(key);
		if (detacher == null)
			throwNoInstalledDetacher(key);
		Object attachment = detacher.detach(object);
		lock.release();
		return attachment;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.spaces.IObjectSpace#put(java.lang.String, java.lang.Object)
	 */
	public void put(String key, Object object)
	{
		LockUtility.acquire(lock);
		objects.put(key, object);
		lock.release();
	}

	/*
	 * (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.spaces.IObjectSpace#installAttacher(java.lang.String, org.almendra.commons.janet.cas.spaces.IAttacher)
	 */
	public void installAttacher(String key, IAttacher attacher)
	{
		LockUtility.acquire(lock);
		attachers.put(key, attacher);
		lock.release();
	}

	/*
	 * (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.spaces.IObjectSpace#installDetacher(java.lang.String, org.almendra.commons.janet.cas.spaces.ISerializableDetacher)
	 */
	public void installDetacher(String key, IDetacher detacher)
	{
		LockUtility.acquire(lock);
		detachers.put(key, detacher);
		lock.release();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.spaces.IObjectSpace#attach(java.lang.String, java.lang.Object)
	 */
	public void attach(String key, Object attachment) throws NoInstalledAttacher
	{
		LockUtility.acquire(lock);
		IAttacher attacher = (IAttacher) attachers.get(key);
		if (attacher == null)
			throwNoInstalledAttacher(key);
		Object object = objects.get(key);
		attacher.attach(attachment, object);
		lock.release();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.spaces.IObjectSpace#attach(java.lang.String, java.lang.Object, org.almendra.commons.janet.cas.spaces.IAttacher)
	 */
	public void attach(String key, Object attachment, IAttacher attacher)
	{
		LockUtility.acquire(lock);
		Object object = objects.get(key);
		attacher.attach(attachment, object);
		lock.release();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.spaces.IObjectSpace#detach(java.lang.String, org.almendra.commons.janet.cas.spaces.IDetacher)
	 */
	public Object detach(String key, IDetacher detacher)
	{
		LockUtility.acquire(lock);
		Object object = objects.get(key);
		Object attachment = detacher.detach(object);
		lock.release();
		return attachment;
	}

}
